@@ -1,7 +1,7 @@
 // Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -47,61 +47,31 @@
 
 namespace
 {
-    // returns a list of full paths of the immediate subdirs of path
-    std::list<std::string> get_immediate_subdirs(std::string path)
-    {
-	Glib::Dir dir(path);
-	std::list<std::string> dirlist;
-
-	for (Glib::Dir::iterator i = dir.begin(), end = dir.end();
-	     i != end; ++i)
-	{
-	    std::string entry = path + *i;
-	    if (Glib::file_test(entry, Glib::FILE_TEST_IS_DIR))
-		dirlist.push_back(entry);
-	}
-
-	return dirlist;
-    }
-
-    // returns a list of the XML file names in path with the ".xml" extension
-    // stripped
-    std::list<std::string> get_xml_files(std::string path, std::string ext)
+    std::list<std::string> get_files(std::string path, std::string ext)
     {
 	std::list<std::string> retlist;
 	Glib::Dir dir(path);
     
 	for (Glib::Dir::iterator i = dir.begin(), end = dir.end(); i != end; ++i)
-	{
+          {
 	    std::string entry = *i;
 	    std::string::size_type idx = entry.rfind(ext);
 	    if (idx != std::string::npos && 
-		idx == entry.length() - ext.length())
-	    {
-		entry.replace(idx, ext.length(), "");  //substitute the ".xml" with ""
-		retlist.push_back(Glib::filename_to_utf8(path + entry));
-	    }
-	}
-	return retlist;
-    }
-    
-    // returns a list of the XML file names in the immediate subdirs of path
-    // with the ".xml" extension stripped
-    std::list<std::string> get_xml_files_in_immediate_subdirs(std::string path, std::string ext)
-    {
-	std::list<std::string> retlist, dirlist = get_immediate_subdirs(path);
-	for (std::list<std::string>::iterator i = dirlist.begin(),
-		 end = dirlist.end(); i != end; ++i)
-	{
-	    std::list<std::string> files = 
-	      get_xml_files(File::add_slash_if_necessary(*i), ext);
-	
-	    retlist.insert(retlist.end(), files.begin(), files.end());
-	}
+                idx == entry.length() - ext.length())
+              retlist.push_back(Glib::filename_to_utf8(path + entry));
+          }
 	return retlist;
     }
 }
 
+std::string File::add_ext_if_necessary(std::string file, std::string ext)
+{
+  if (nameEndsWith(file, ext) == true)
+    return file;
+  else
+    return file + ext;
+}
+
 std::string File::add_slash_if_necessary(std::string dir)
 {
   if (dir.c_str()[strlen(dir.c_str())-1] == '/')
@@ -296,11 +266,12 @@ std::string File::getUserArmysetDir()
   return dir;
 }
 
-std::list<std::string> File::scanFiles(std::string dir, std::string extension)
+std::list<std::string> File::scanForFiles(std::string dir, std::string extension)
 {
-  return get_xml_files_in_immediate_subdirs(dir, extension);
+  return get_files (dir, extension);
 }
 
+
 std::string File::getTilesetDir()
 {
   return add_slash_if_necessary(Configuration::s_dataPath) + TILESETDIR + "/";
@@ -355,7 +326,42 @@ void File::erase_dir(std::string filename)
 {
   rmdir(filename.c_str());
 }
+
+void File::clean_dir(std::string dirname)
+{
+  if (File::exists(dirname) == false)
+    return;
+  Glib::Dir dir(dirname);
+  for (Glib::DirIterator it = dir.begin(); it != dir.end(); it++)
+    File::erase(File::add_slash_if_necessary(dirname) + *it);
+  dir.close();
+  File::erase_dir(dirname);
+}
+
 std::string File::getSetConfigurationFilename(std::string dir, std::string subdir, std::string ext)
 {
   return add_slash_if_necessary(dir) + subdir + "/" + subdir + ext;
 }
+char *File::sanify(const char *string)
+{
+  char *result = NULL;
+  size_t resultlen = 1;
+  size_t len = strlen(string);
+  result = (char*) malloc (resultlen);
+  result[0] = '\0';
+  for (unsigned int i = 0; i < len; i++)
+    {
+      int letter = tolower(string[i]);
+      if (strchr("abcdefghijklmnopqrstuvwxyz0123456789-", letter) == NULL)
+	continue;
+
+      resultlen++;
+      result = (char *) realloc (result, resultlen);
+      if (result)
+	{
+	  result[resultlen-2] = char(letter);
+	  result[resultlen-1] = '\0';
+	}
+    }
+  return result;
+}
