@@ -35,6 +35,7 @@
 #include "shieldset.h"
 #include "cityset.h"
 #include "stackreflist.h"
+#include "LocationList.h"
 
 class MapGenerator;
 class XML_Helper;
@@ -110,14 +111,24 @@ class GameMap: public sigc::trackable
 	static Rectangle get_boundary()
 	    { return Rectangle(0, 0, s_width, s_height); }
 
-        //! Returns the tileset in use
-        Tileset* getTileset() const {return d_tileSet;}
+        static Tileset* getTileset();
+        static Cityset* getCityset();
+        static Shieldset* getShieldset();
 
-        //! Returns the shieldset in use
-        Shieldset* getShieldset() const {return d_shieldSet;}
+        void setTileset(std::string tileset);
 
-        //! Returns the tileset in use
-        Cityset* getCityset() const {return d_citySet;}
+        guint32 getTileSize() const;
+
+        guint32 getTilesetId() const;
+        guint32 getCitysetId() const;
+        guint32 getShieldsetId() const;
+        std::string getTilesetName() const;
+        std::string getCitysetName() const;
+        std::string getShieldsetName() const;
+
+        void setShieldset(std::string shieldset);
+
+        void setCityset(std::string cityset);
 
 	//! Sets the tile object at position (x, y)
 	void setTile(int x, int y, Maptile *tile);
@@ -158,6 +169,8 @@ class GameMap: public sigc::trackable
         static std::list<Vector<int> > getNearbyPoints(Vector<int> pos, int dist);
 	static guint32 countArmyUnits(Vector<int> pos);
 	static MapBackpack *getBackpack(Vector<int> pos);
+        static bool can_search(Stack *stack);
+        static bool can_plant_flag(Stack *stack);
         std::list<MapBackpack*> getBackpacks() const;
 
         //! Get the tile object at position (x,y)
@@ -227,7 +240,12 @@ class GameMap: public sigc::trackable
 	void switchShieldset(Shieldset *shieldset);
 	void switchTileset(Tileset *tileset);
 
-	bool moveBuilding(Vector<int> from, Vector<int> to);
+        void reloadShieldset();
+        void reloadTileset();
+        void reloadCityset();
+        void reloadArmyset(Armyset *armyset);
+
+	bool moveBuilding(Vector<int> from, Vector<int> to, guint32 new_width = 0);
 	bool canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> to, bool making_islands = true);
 	bool canPutStack(guint32 size, Player *p, Vector<int> to);
 	bool moveStack(Stack *stack, Vector<int> to);
@@ -235,16 +253,22 @@ class GameMap: public sigc::trackable
 	void moveBackpack(Vector<int> from, Vector<int> to);
 	guint32 getBuildingSize(Vector<int> tile);
 	Maptile::Building getBuilding(Vector<int> tile);
+        guint32 countBuildings(Maptile::Building building_type);
 	Tile::Type getTerrainType(Vector<int> tile);
 	void setBuilding(Vector<int> tile, Maptile::Building building);
 
+
+        bool putNewCity(Vector<int> tile);
 	bool putCity(City *c, bool keep_owner = false);
 	bool removeCity(Vector<int> pos);
 	bool putRuin(Ruin *r);
+        bool putNewRuin(Vector<int> tile);
 	bool removeRuin(Vector<int> pos);
 	bool putTemple(Temple *t);
+        bool putNewTemple(Vector<int> tile);
 	bool removeTemple(Vector<int> pos);
 	bool putRoad(Road *r);
+        bool putNewRoad(Vector<int> tile);
 	bool removeRoad(Vector<int> pos);
 	bool putBridge(Bridge *b);
 	bool removeBridge(Vector<int> pos);
@@ -254,6 +278,9 @@ class GameMap: public sigc::trackable
 	bool removePort(Vector<int> pos);
 	bool putStack(Stack *s);
 	void removeStack(Stack *s);
+        bool removeLocation (Vector<int> pos);
+        bool eraseTile(Vector<int> pos);
+        bool eraseTiles(Rectangle r);
 
 	Location *getLocation(Vector<int> pos);
 
@@ -269,10 +296,11 @@ class GameMap: public sigc::trackable
         static int calculateTilesPerOverviewMapTile(int width, int height);
         static int calculateTilesPerOverviewMapTile();
 
+        Vector<int> findNearestAreaForBuilding(Maptile::Building building_type, Vector<int> pos, guint32 width);
     protected:
         //! Create the map with the given tileset
-        GameMap(std::string TilesetName, std::string ShieldsetName,
-		std::string Citysetname);
+        GameMap(std::string TilesetName = "", std::string ShieldsetName = "",
+		std::string Citysetname = "");
 
         //! Load the map using the given XML_Helper
         GameMap(XML_Helper* helper);
@@ -297,19 +325,32 @@ class GameMap: public sigc::trackable
 	bool are_those_tiles_similar(Tile::Type outer_tile,Tile::Type inner_tile, bool checking_loneliness);
 	Vector<int> findNearestObjectInDir(Vector<int> pos, Vector<int> dir);
 	void putBuilding(LocationBox *b, Maptile::Building building);
+        void clearBuilding(Vector<int> pos, guint32 width);
 	void removeBuilding(LocationBox *b);
 
 	void updateShips(Vector<int> pos);
 
 
+        static void changeFootprintToSmallerCityset(Location *location, Maptile::Building building_type, guint32 old_tile_width);
+
+        static void relocateLocation(Location *location, Maptile::Building building_type, guint32 tile_width);
+
 	static std::list<Stack*> getNearbyStacks(Vector<int> pos, int dist, bool friendly);
 
 	static bool offmap(int x, int y);
 
+        //void resizeLocations(LocationList<Location*> *list, Maptile::Building building_type, guint32 tile_width);
         // Data
         static GameMap* s_instance;
         static int s_width;
         static int s_height;
+        static Tileset* s_tileset; //not saved
+        static Cityset* s_cityset; //not saved
+        static Shieldset* s_shieldset; //not saved
+
+        std::string d_tileset;
+        std::string d_shieldset;
+        std::string d_cityset;
 
         Tileset* d_tileSet;
         Shieldset* d_shieldSet;
