@@ -1,7 +1,7 @@
 // Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2010 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -76,7 +76,8 @@ using namespace std;
 GameScenario::GameScenario(std::string name,std::string comment, bool turnmode,
 			   GameScenario::PlayMode playmode)
     :d_name(name),d_comment(comment), d_copyright(""), d_license(""),
-    d_turnmode(turnmode), d_playmode(playmode), inhibit_autosave_removal(false)
+    d_turnmode(turnmode), d_playmode(playmode), inhibit_autosave_removal(false),
+    loaded_game_filename("")
 {
     Armysetlist::getInstance();
     Tilesetlist::getInstance();
@@ -91,19 +92,23 @@ GameScenario::GameScenario(std::string name,std::string comment, bool turnmode,
 //savegame has an absolute path
 GameScenario::GameScenario(string savegame, bool& broken)
   :d_turnmode(true), d_playmode(GameScenario::HOTSEAT), 
-    inhibit_autosave_removal(false)
+    inhibit_autosave_removal(false), loaded_game_filename("")
 {
-  Tar_Helper t(savegame, std::ios::in);
-  loadArmysets(&t);
-  loadTilesets(&t);
-  loadCitysets(&t);
-  loadShieldsets(&t);
-  std::string filename = t.getFirstFile(broken);
-  XML_Helper helper(filename, std::ios::in, Configuration::s_zipfiles);
-  broken = loadWithHelper(helper);
-  File::erase(filename);
-  helper.close();
-  t.Close();
+  Tar_Helper t(savegame, std::ios::in, broken);
+  if (broken == false)
+    {
+      loaded_game_filename = savegame;
+      loadArmysets(&t);
+      loadTilesets(&t);
+      loadCitysets(&t);
+      loadShieldsets(&t);
+      std::string filename = t.getFirstFile(broken);
+      XML_Helper helper(filename, std::ios::in, Configuration::s_zipfiles);
+      broken = loadWithHelper(helper);
+      File::erase(filename);
+      helper.close();
+      t.Close();
+    }
 }
 
 bool GameScenario::loadArmysets(Tar_Helper *t)
@@ -126,29 +131,13 @@ bool GameScenario::loadTilesets(Tar_Helper *t)
   bool broken = false;
   std::list<std::string> tilesets;
   tilesets = t->getFilenamesWithExtension(Tileset::file_extension);
+  Tilesetlist *tlist = Tilesetlist::getInstance();
   for (std::list<std::string>::iterator it = tilesets.begin(); 
        it != tilesets.end(); it++)
     {
-      std::string filename = t->getFile(*it, broken);
-      Tileset *tileset = Tileset::create(filename);
-      tileset->setSubDir(File::get_basename(*it));
-
-      //extract all the files and remember where we extracted them
-      std::list<std::string> delfiles;
-      delfiles.push_back(filename);
-      std::list<std::string> files;
-      tileset->getFilenames(files);
-      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
-	delfiles.push_back (t->getFile(*i + ".png", broken));
-
-      std::string subdir = "";
-      guint32 id = 0;
-      Tilesetlist::getInstance()->addToPersonalCollection(tileset, subdir, id);
-
-      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
-	File::erase(*it);
-  
-      Tilesetlist::getInstance()->getTileset(tileset->getId())->instantiateImages();
+      Tileset *tileset = tlist->import(t, *it, broken);
+      if (tileset)
+        tlist->getTileset(tileset->getId())->instantiateImages();
     }
   return !broken;
 }
@@ -158,29 +147,13 @@ bool GameScenario::loadCitysets(Tar_Helper *t)
   bool broken = false;
   std::list<std::string> citysets;
   citysets = t->getFilenamesWithExtension(Cityset::file_extension);
+  Citysetlist *clist = Citysetlist::getInstance();
   for (std::list<std::string>::iterator it = citysets.begin(); 
        it != citysets.end(); it++)
     {
-      std::string filename = t->getFile(*it, broken);
-      Cityset *cityset = Cityset::create(filename);
-      cityset->setSubDir(File::get_basename(*it));
-
-      //extract all the files and remember where we extracted them
-      std::list<std::string> delfiles;
-      delfiles.push_back(filename);
-      std::list<std::string> files;
-      cityset->getFilenames(files);
-      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
-	delfiles.push_back (t->getFile(*i + ".png", broken));
-
-      std::string subdir = "";
-      guint32 id = 0;
-      Citysetlist::getInstance()->addToPersonalCollection(cityset, subdir, id);
-
-      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
-	File::erase(*it);
-  
-      Citysetlist::getInstance()->getCityset(cityset->getId())->instantiateImages();
+      Cityset *cityset = clist->import(t, *it, broken);
+      if (cityset)
+        clist->getCityset(cityset->getId())->instantiateImages();
     }
   return !broken;
 }
@@ -190,37 +163,20 @@ bool GameScenario::loadShieldsets(Tar_Helper *t)
   bool broken = false;
   std::list<std::string> shieldsets;
   shieldsets = t->getFilenamesWithExtension(Shieldset::file_extension);
+  Shieldsetlist *slist = Shieldsetlist::getInstance();
   for (std::list<std::string>::iterator it = shieldsets.begin(); 
        it != shieldsets.end(); it++)
     {
-      std::string filename = t->getFile(*it, broken);
-      Shieldset *shieldset = Shieldset::create(filename);
-      shieldset->setSubDir(File::get_basename(*it));
-
-      //extract all the files and remember where we extracted them
-      std::list<std::string> delfiles;
-      delfiles.push_back(filename);
-      std::list<std::string> files;
-      shieldset->getFilenames(files);
-      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
-	delfiles.push_back (t->getFile(*i + ".png", broken));
-
-      std::string subdir = "";
-      guint32 id = 0;
-      Shieldsetlist::getInstance()->addToPersonalCollection(shieldset, subdir, id);
-
-      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
-	File::erase(*it);
-  
-
-      Shieldsetlist::getInstance()->getShieldset(shieldset->getId())->instantiateImages();
+      Shieldset *shieldset = slist->import(t, *it, broken);
+      if (shieldset)
+        slist->getShieldset(shieldset->getId())->instantiateImages();
     }
   return !broken;
 }
 
 GameScenario::GameScenario(XML_Helper &helper, bool& broken)
   : d_turnmode(true), d_playmode(GameScenario::HOTSEAT),
-    inhibit_autosave_removal(false)
+    inhibit_autosave_removal(false), loaded_game_filename("")
 {
   broken = loadWithHelper(helper);
 }
@@ -559,6 +515,7 @@ bool GameScenario::loadWithHelper(XML_Helper& helper)
 
   bool broken = false;
 
+  helper.registerTag("lordsawar", sigc::mem_fun(this, &GameScenario::load));
   helper.registerTag(d_tag, sigc::mem_fun(this, &GameScenario::load));
   helper.registerTag(Itemlist::d_tag, sigc::mem_fun(this, &GameScenario::load));
   helper.registerTag(Playerlist::d_tag, sigc::mem_fun(this, &GameScenario::load));
@@ -588,7 +545,6 @@ bool GameScenario::loadWithHelper(XML_Helper& helper)
 GameScenario::~GameScenario()
 {
   // GameMap is a Singleton so we need a function to delete it
-  GameMap::deleteInstance();
   Itemlist::deleteInstance();
   Playerlist::deleteInstance();
   Citylist::deleteInstance();
@@ -601,6 +557,7 @@ GameScenario::~GameScenario()
   Roadlist::deleteInstance();
   QuestsManager::deleteInstance();
   VectoredUnitlist::deleteInstance();
+  GameMap::deleteInstance();
 
   if (fl_counter)
     {
@@ -610,9 +567,10 @@ GameScenario::~GameScenario()
   if (Configuration::s_autosave_policy == 1 && 
       inhibit_autosave_removal == false)
     {
-      std::string filename = File::getSavePath() + "autosave.sav";
+      std::string filename = File::getSavePath() + "autosave" + SAVE_EXT;
       File::erase(filename);
     }
+  clean_tmp_dir();
   GameScenarioOptions::s_round = 0;
 } 
 
@@ -635,10 +593,7 @@ std::string GameScenario::getComment(bool translate) const
 bool GameScenario::saveGame(string filename, string extension) const
 {
   bool retval = true;
-  string goodfilename=filename;
-
-  if (File::nameEndsWith(filename, extension) == false)
-    goodfilename += "." + extension;
+  string goodfilename = File::add_ext_if_necessary(filename, extension);
 
   std::string tmpfile = "lw.XXXX";
   int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
@@ -650,12 +605,16 @@ bool GameScenario::saveGame(string filename, string extension) const
   if (retval == false)
     return false;
 
-  Tar_Helper t(goodfilename, std::ios::out);
+  bool broken = false;
+  Tar_Helper t(goodfilename, std::ios::out, broken);
+  if (broken == true)
+    return false;
+
   t.saveFile(tmpfile, File::get_basename(goodfilename, true));
   File::erase(tmpfile);
   std::list<std::string> files;
 
-  Cityset *cs = GameMap::getInstance()->getCityset();
+  Cityset *cs = GameMap::getCityset();
   cs->getFilenames(files);
   t.saveFile(cs->getConfigurationFile());
   for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
@@ -663,7 +622,7 @@ bool GameScenario::saveGame(string filename, string extension) const
     t.saveFile(cs->getFile(*it));
 
   files.clear();
-  Shieldset *ss = GameMap::getInstance()->getShieldset();
+  Shieldset *ss = GameMap::getShieldset();
   ss->getFilenames(files);
   t.saveFile(ss->getConfigurationFile());
   for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
@@ -671,7 +630,7 @@ bool GameScenario::saveGame(string filename, string extension) const
     t.saveFile(ss->getFile(*it));
 
   files.clear();
-  Tileset *ts = GameMap::getInstance()->getTileset();
+  Tileset *ts = GameMap::getTileset();
   ts->getFilenames(files);
   t.saveFile(ts->getConfigurationFile());
   for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
@@ -766,7 +725,7 @@ bool GameScenario::saveWithHelper(XML_Helper &helper) const
 
 bool GameScenario::load(std::string tag, XML_Helper* helper)
 {
-  if (tag == GameScenario::d_tag)
+  if (tag == "lordsawar")
     {
       if (helper->getVersion() != LORDSAWAR_SAVEGAME_VERSION)
 	{
@@ -775,7 +734,10 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
 	  cerr << "savefile offers " <<helper->getVersion() <<".\n";
 	  return false;
 	}
-
+      return true;
+    }
+  if (tag == GameScenario::d_tag)
+    {
       debug("loading scenario")
 
       helper->getData(d_id, "id");
@@ -917,9 +879,9 @@ bool GameScenario::autoSave()
 {
   Glib::ustring filename = "";
   if (Configuration::s_autosave_policy == 2)
-    filename = String::ucompose("autosave-%1.sav", Glib::ustring::format(std::setfill(L'0'), std::setw(3), s_round - 1));
+    filename = String::ucompose("autosave-%1%2", Glib::ustring::format(std::setfill(L'0'), std::setw(3), s_round - 1), SAVE_EXT);
   else if (Configuration::s_autosave_policy == 1)
-    filename = "autosave.sav";
+    filename = "autosave" + SAVE_EXT;
   else
     return true;
   // autosave to the file "autosave.sav". This is crude, but should work
@@ -927,12 +889,12 @@ bool GameScenario::autoSave()
   // As a more enhanced version: autosave to a temporary file, then rename
   // the file. Avoids screwing up the autosave if something goes wrong
   // (and we have a savefile for debugging)
-  if (!saveGame(File::getSavePath() + "tmp.sav"))
+  if (!saveGame(File::getSavePath() + "tmp" + SAVE_EXT))
     {
       std::cerr<< "Autosave failed.\n";
       return false;
     }
-  if (rename(std::string(File::getSavePath() + "tmp.sav").c_str(),
+  if (rename(std::string(File::getSavePath() + "tmp" + SAVE_EXT).c_str(),
 	     std::string(File::getSavePath() + filename).c_str()))
     {
       char* err = strerror(errno);
@@ -1103,7 +1065,9 @@ class ParamLoader
 {
 public:
     ParamLoader(std::string filename, bool &broken) {
-      Tar_Helper t(filename, std::ios::in);
+      Tar_Helper t(filename, std::ios::in, broken);
+      if (broken)
+        return;
       std::string tmpfile = t.getFirstFile(broken);
       XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
       helper.registerTag(GameMap::d_tag, 
@@ -1164,7 +1128,7 @@ public:
 		helper->getData(armyset_id, "armyset");
 		Armysetlist *al = Armysetlist::getInstance();
 		Armyset *armyset = al->getArmyset(armyset_id);
-		game_params.army_theme = armyset->getSubDir();
+		game_params.army_theme = armyset->getBaseName();
 	      }
 
 	    return true;
@@ -1219,9 +1183,11 @@ GameParameters GameScenario::loadGameParameters(std::string filename, bool &brok
 class PlayModeLoader
 {
 public:
-    PlayModeLoader(std::string filename, bool broken) {
+    PlayModeLoader(std::string filename, bool &broken) {
       play_mode = GameScenario::HOTSEAT;
-      Tar_Helper t(filename, std::ios::in);
+      Tar_Helper t(filename, std::ios::in, broken);
+      if (broken)
+        return;
       std::string file = File::get_basename(filename, true);
       std::string tmpfile = t.getFirstFile(broken);
       XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
@@ -1250,6 +1216,8 @@ public:
 GameScenario::PlayMode GameScenario::loadPlayMode(std::string filename, bool &broken)
 {
   PlayModeLoader loader(filename, broken);
+  if (broken)
+    return HOTSEAT;
   return loader.play_mode;
 }
 
@@ -1258,7 +1226,9 @@ class DetailsLoader
 public:
     DetailsLoader(std::string filename, bool &broken) {
       player_count = 0; city_count = 0; name = ""; comment = "";
-      Tar_Helper t(filename, std::ios::in);
+      Tar_Helper t(filename, std::ios::in, broken);
+      if (broken)
+        return;
       std::string tmpfile = t.getFirstFile(broken);
       XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
       helper.registerTag(GameScenario::d_tag, 
@@ -1310,3 +1280,8 @@ void GameScenario::loadDetails(std::string filename, bool &broken, guint32 &play
   return;
 }
 
+void GameScenario::clean_tmp_dir() const
+{
+  if (loaded_game_filename != "")
+    Tar_Helper::clean_tmp_dir(loaded_game_filename);
+}
