@@ -1,6 +1,6 @@
 // Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -85,6 +85,14 @@ struct NewLevelCacheItem
     PixMask* surface;
 };
 
+//the structure to store default tile style pictures in
+struct DefaultTileStyleCacheItem
+{
+    guint32 tilestyle_type;
+    guint32 tilesize;
+    PixMask* surface;
+};
+
 //the structure to store temples in
 struct TempleCacheItem
 {
@@ -225,6 +233,7 @@ struct TileCacheItem
   bool has_grid;
   guint32 tileset;
   guint32 cityset;
+  guint32 shieldset;
   PixMask* surface;
 };
 
@@ -287,6 +296,7 @@ GraphicsCache::GraphicsCache()
     loadMoveBonusPics();
     loadMedalPics();
     loadNewLevelPics();
+    loadDefaultTileStylePics();
 
     d_smallruinedcity = getMiscPicture("smallruinedcity.png");
     d_smallhero = getMiscPicture("hero.png");
@@ -427,6 +437,34 @@ PixMask* GraphicsCache::getShipPic(const Player* p)
     return myitem->surface;
 }
 
+PixMask* GraphicsCache::getDefaultTileStylePic(guint32 type, guint32 size)
+{
+    debug("getting default tile style pic " <type)
+    std::list<DefaultTileStyleCacheItem*>::iterator it;
+    DefaultTileStyleCacheItem* myitem;
+    for (it = d_defaulttilestylelist.begin(); 
+         it != d_defaulttilestylelist.end(); it++)
+    {
+        if ((*it)->tilestyle_type == type &&
+            (*it)->tilesize == size)
+        {
+            myitem = (*it);
+            
+            // put the item on the last place (==last touched)
+            d_defaulttilestylelist.erase(it);
+            d_defaulttilestylelist.push_back(myitem);
+            
+            return myitem->surface;
+        }
+    }
+    // We are still here, so the graphic is not in the cache. 
+    // addDefaultTileStylePic calls checkPictures on its own, so we can 
+    // simply return the surface
+    myitem = addDefaultTileStylePic(type, size);
+
+    return myitem->surface;
+}
+
 PixMask* GraphicsCache::getNewLevelPic(const Player* p, guint32 gender)
 {
     debug("getting new level pic " <<p->getName())
@@ -455,7 +493,7 @@ PixMask* GraphicsCache::getNewLevelPic(const Player* p, guint32 gender)
 
 PixMask* GraphicsCache::getPortPic()
 {
-  return getPortPic(GameMap::getInstance()->getCityset()->getId());
+  return getPortPic(GameMap::getInstance()->getCitysetId());
 }
 
 PixMask* GraphicsCache::getPortPic(guint32 cityset)
@@ -486,7 +524,7 @@ PixMask* GraphicsCache::getPortPic(guint32 cityset)
 
 PixMask* GraphicsCache::getSignpostPic()
 {
-  return getSignpostPic(GameMap::getInstance()->getCityset()->getId());
+  return getSignpostPic(GameMap::getInstance()->getCitysetId());
 }
 
 PixMask* GraphicsCache::getSignpostPic(guint32 cityset)
@@ -549,7 +587,7 @@ PixMask* GraphicsCache::getBagPic(guint32 armyset)
 
 PixMask* GraphicsCache::getExplosionPic()
 {
-  return getExplosionPic(GameMap::getInstance()->getTileset()->getId());
+  return getExplosionPic(GameMap::getInstance()->getTilesetId());
 }
 
 PixMask* GraphicsCache::getExplosionPic(guint32 tileset)
@@ -662,12 +700,13 @@ PixMask* GraphicsCache::getArmyPic(guint32 armyset, guint32 army_id,
 
 PixMask* GraphicsCache::getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid)
 {
-  guint32 tileset = GameMap::getInstance()->getTileset()->getId();
-  guint32 cityset = GameMap::getInstance()->getCityset()->getId();
-  return getTilePic(tile_style_id, fog_type_id, has_bag, has_standard, standard_player_id, stack_size, stack_player_id, army_type_id, has_tower, has_ship, building_type, building_subtype, building_tile, building_player_id, tilesize, has_grid, tileset, cityset);
+  guint32 tileset = GameMap::getInstance()->getTilesetId();
+  guint32 cityset = GameMap::getInstance()->getCitysetId();
+  guint32 shieldset = GameMap::getInstance()->getShieldsetId();
+  return getTilePic(tile_style_id, fog_type_id, has_bag, has_standard, standard_player_id, stack_size, stack_player_id, army_type_id, has_tower, has_ship, building_type, building_subtype, building_tile, building_player_id, tilesize, has_grid, tileset, cityset, shieldset);
 }
 
-PixMask* GraphicsCache::getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid, guint32 tileset, guint32 cityset)
+PixMask* GraphicsCache::getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid, guint32 tileset, guint32 cityset, guint32 shieldset)
 {
     debug("getting tile pic " << tile_style_id << " " <<
 	  fog_type_id << " " << has_bag << " " << has_standard << " " <<
@@ -675,7 +714,8 @@ PixMask* GraphicsCache::getTilePic(int tile_style_id, int fog_type_id, bool has_
 	  " " << army_type_id << " " << has_tower << " " << has_ship << " " << 
 	  building_type << " " << building_subtype << " " << building_tile.x 
 	  << "," << building_tile.y << " " << building_player_id << " " << 
-	  tilesize << " " << has_grid << " " <<tileset);
+	  tilesize << " " << has_grid << " " <<tileset << " " << cityset <<
+          " " << shieldset);
 
     std::list<TileCacheItem*>::iterator it;
     TileCacheItem* myitem;
@@ -699,6 +739,7 @@ PixMask* GraphicsCache::getTilePic(int tile_style_id, int fog_type_id, bool has_
     item.fog_type_id = fog_type_id;
     item.tileset = tileset;
     item.cityset = cityset;
+    item.shieldset = shieldset;
     TileMap::iterator mit = d_tilemap.find(item);
     if (mit != d_tilemap.end())
       {
@@ -751,13 +792,13 @@ PixMask* GraphicsCache::getShieldPic(guint32 shieldset, guint32 type,
         
 PixMask* GraphicsCache::getShieldPic(guint32 type, Player *p)
 {
-  guint32 shieldset = GameMap::getInstance()->getShieldset()->getId();
+  guint32 shieldset = GameMap::getInstance()->getShieldsetId();
   return getShieldPic(shieldset, type, p->getId());
 }
 
 PixMask* GraphicsCache::getTemplePic(int type)
 {
-  guint32 cityset = GameMap::getInstance()->getCityset()->getId();
+  guint32 cityset = GameMap::getInstance()->getCitysetId();
   return getTemplePic(type, cityset);
 }
 PixMask* GraphicsCache::getTemplePic(int type, guint32 cityset)
@@ -789,7 +830,7 @@ PixMask* GraphicsCache::getTemplePic(int type, guint32 cityset)
 
 PixMask* GraphicsCache::getRuinPic(int type)
 {
-  guint32 cityset = GameMap::getInstance()->getCityset()->getId();
+  guint32 cityset = GameMap::getInstance()->getCitysetId();
   return getRuinPic(type, cityset);
 }
 PixMask* GraphicsCache::getRuinPic(int type, guint32 cityset)
@@ -848,7 +889,7 @@ PixMask* GraphicsCache::getDiplomacyPic(int type, Player::DiplomaticState state)
 
 PixMask* GraphicsCache::getRoadPic(int type)
 {
-  return getRoadPic(type, GameMap::getInstance()->getTileset()->getId());
+  return getRoadPic(type, GameMap::getInstance()->getTilesetId());
 }
 PixMask* GraphicsCache::getRoadPic(int type, guint32 tileset)
 {
@@ -879,7 +920,7 @@ PixMask* GraphicsCache::getRoadPic(int type, guint32 tileset)
 
 PixMask* GraphicsCache::getFogPic(int type)
 {
-  return getFogPic(type, GameMap::getInstance()->getTileset()->getId());
+  return getFogPic(type, GameMap::getInstance()->getTilesetId());
 }
 
 PixMask* GraphicsCache::getFogPic(int type, guint32 tileset)
@@ -913,7 +954,7 @@ PixMask* GraphicsCache::getFogPic(int type, guint32 tileset)
 
 PixMask* GraphicsCache::getBridgePic(int type)
 {
-  return getBridgePic(type, GameMap::getInstance()->getTileset()->getId());
+  return getBridgePic(type, GameMap::getInstance()->getTilesetId());
 }
 PixMask* GraphicsCache::getBridgePic(int type, guint32 tileset)
 {
@@ -983,7 +1024,7 @@ PixMask* GraphicsCache::getCityPic(const City* city, guint32 cityset)
 
 PixMask* GraphicsCache::getCityPic(const City* city)
 {
-  guint32 cityset = GameMap::getInstance()->getCityset()->getId();
+  guint32 cityset = GameMap::getInstance()->getCitysetId();
   return getCityPic(city, cityset);
 }
 
@@ -1017,7 +1058,7 @@ PixMask* GraphicsCache::getCityPic(int type, const Player* p, guint32 cityset)
 
 PixMask* GraphicsCache::getTowerPic(const Player* p)
 {
-  guint32 cityset = GameMap::getInstance()->getCityset()->getId();
+  guint32 cityset = GameMap::getInstance()->getCitysetId();
   return getTowerPic(p, cityset);
 }
 
@@ -1079,12 +1120,12 @@ PixMask* GraphicsCache::getFlagPic(guint32 stack_size, const Player *p, guint32
 PixMask* GraphicsCache::getFlagPic(guint32 stack_size, const Player *p)
 {
   return getFlagPic(stack_size, p, 
-		    GameMap::getInstance()->getTileset()->getId());
+		    GameMap::getInstance()->getTilesetId());
 }
 
 PixMask* GraphicsCache::getFlagPic(const Stack* s)
 {
-  return getFlagPic(s, GameMap::getInstance()->getTileset()->getId());
+  return getFlagPic(s, GameMap::getInstance()->getTilesetId());
 }
 PixMask* GraphicsCache::getFlagPic(const Stack* s, guint32 tileset)
 {
@@ -1101,7 +1142,7 @@ PixMask* GraphicsCache::getSelectorPic(guint32 type, guint32 frame,
 					   const Player *p)
 {
   return getSelectorPic(type, frame, p, 
-			GameMap::getInstance()->getTileset()->getId());
+			GameMap::getInstance()->getTilesetId());
 }
 PixMask* GraphicsCache::getSelectorPic(guint32 type, guint32 frame, 
 					   const Player *p, guint32 tileset)
@@ -1175,8 +1216,8 @@ PixMask* GraphicsCache::applyMask(PixMask* image, PixMask* mask, Gdk::Color colo
       std::cerr <<"Warning: mask and original image do not match\n";
       return NULL;
     }
-  if (isNeutral)
-    return result;
+  //if (isNeutral)
+    //return result;
   
   Glib::RefPtr<Gdk::Pixbuf> maskbuf = mask->to_pixbuf();
 
@@ -1291,6 +1332,9 @@ void GraphicsCache::checkPictures()
   while (d_newlevellist.size() > 10)
     eraseLastNewLevelItem();
 
+  while (d_defaulttilestylelist.size() > 3)
+    eraseLastDefaultTileStyleItem();
+
   while (d_templelist.size() > 10)
     eraseLastTempleItem();
 
@@ -1389,7 +1433,7 @@ void GraphicsCache::checkPictures()
 
 }
 
-void GraphicsCache::drawTilePic(PixMask *surface, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 ts, bool has_grid, guint32 tileset, guint32 cityset)
+void GraphicsCache::drawTilePic(PixMask *surface, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 ts, bool has_grid, guint32 tileset, guint32 cityset, guint32 shieldset)
 {
   const Player *player;
   Glib::RefPtr<Gdk::Pixmap> pixmap = surface->get_pixmap();
@@ -1450,9 +1494,7 @@ void GraphicsCache::drawTilePic(PixMask *surface, int fog_type_id, bool has_bag,
   if (has_grid)
     {
       Glib::RefPtr<Gdk::GC> context = surface->get_gc();
-      Gdk::Color line_color = Gdk::Color();
-      line_color.set_rgb_p(0,0,0);
-      context->set_rgb_fg_color(line_color);
+      context->set_rgb_fg_color(GRID_BOX_COLOUR);
       pixmap->draw_rectangle(context, false, 0, 0, ts, ts);
     }
 
@@ -1476,7 +1518,7 @@ TileCacheItem* GraphicsCache::addTilePic(TileCacheItem *item)
 	item->building_subtype << " " << item->building_tile.x << 
 	"," << item->building_tile.y << " " << item->building_player_id << 
 	" " << item->tilesize << " " << item->has_grid << " " << item->tileset
-	" " << item->cityset);
+	" " << item->cityset << " " << item->shieldset);
 
   TileCacheItem* myitem = new TileCacheItem();
   *myitem = *item;
@@ -1497,7 +1539,7 @@ TileCacheItem* GraphicsCache::addTilePic(TileCacheItem *item)
 		  myitem->building_type, myitem->building_subtype, 
 		  myitem->building_tile, myitem->building_player_id, 
 		  myitem->tilesize, myitem->has_grid, myitem->tileset,
-		  myitem->cityset);
+		  myitem->cityset, myitem->shieldset);
     }
 
   //now the final preparation steps:
@@ -1661,6 +1703,34 @@ NewLevelCacheItem* GraphicsCache::addNewLevelPic(const Player* p, guint32 gender
   return myitem;
 }
 
+DefaultTileStyleCacheItem* GraphicsCache::addDefaultTileStylePic(guint32 type,
+                                                                 guint32 tilesize)
+{
+  debug("ADD default tile style pic: " <<type ", size " << size)
+
+  DefaultTileStyleCacheItem* myitem = new DefaultTileStyleCacheItem();
+  myitem->tilestyle_type = type;
+  myitem->tilesize = tilesize;
+          
+  PixMask *img = d_default_tilestyles[type]->copy();
+  PixMask::scale(img, tilesize, tilesize);
+  myitem->surface = img;
+
+  //now the final preparation steps:
+  //a) add the size
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize += myitem->surface->get_depth()/8 * size;
+
+  //b) add the entry to the list
+  d_defaulttilestylelist.push_back(myitem);
+
+  //c) check if the cache size is too large
+  checkPictures();
+
+  //we are finished, so return the pic
+  return myitem;
+}
+
 PortCacheItem* GraphicsCache::addPortPic(guint32 cityset)
 {
   debug("ADD port pic: " << cityset);
@@ -2433,6 +2503,21 @@ void GraphicsCache::eraseLastNewLevelItem()
   delete myitem;
 }
 
+void GraphicsCache::eraseLastDefaultTileStyleItem()
+{
+  if (d_defaulttilestylelist.empty())
+    return;
+
+  DefaultTileStyleCacheItem* myitem = *(d_defaulttilestylelist.begin());
+  d_defaulttilestylelist.erase(d_defaulttilestylelist.begin());
+
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize -= myitem->surface->get_depth()/8 * size;
+
+  delete myitem->surface;
+  delete myitem;
+}
+
 void GraphicsCache::eraseLastExplosionItem()
 {
   if (d_explosionlist.empty())
@@ -2681,6 +2766,21 @@ void GraphicsCache::loadMedalPics()
     d_medalpic[1][i] = medalpics[i];
 }
 
+void GraphicsCache::loadDefaultTileStylePics()
+{
+  std::vector<PixMask* > tilestyle_images;
+  tilestyle_images = 
+    disassemble_row(File::getMiscFile("various/tilestyles.png"), 
+                    DEFAULT_TILESTYLE_TYPES);
+  int count = 0;
+  for (std::vector<PixMask*>::iterator it = tilestyle_images.begin();
+       it != tilestyle_images.end(); it++)
+    {
+      d_default_tilestyles[count] = *it;
+      count++;
+    }
+}
+
 void GraphicsCache::loadNewLevelPics()
 {
   std::vector<PixMask* > half;
@@ -2746,3 +2846,77 @@ PixMask* GraphicsCache::getMiscPicture(std::string picname, bool alpha)
   return loadImage(File::getMiscFile("/various/" + picname), alpha);
 }
 
+void GraphicsCache::reset()
+{
+
+  while (d_citylist.size())
+    eraseLastCityItem();
+
+  while (d_towerlist.size())
+    eraseLastTowerItem();
+
+  while (d_shiplist.size())
+    eraseLastShipItem();
+
+  while (d_plantedstandardlist.size())
+    eraseLastPlantedStandardItem();
+
+  while (d_newlevellist.size())
+    eraseLastNewLevelItem();
+
+  while (d_templelist.size())
+    eraseLastTempleItem();
+
+  while (d_ruinlist.size())
+    eraseLastRuinItem();
+
+  while (d_diplomacylist.size())
+    eraseLastDiplomacyItem();
+
+  while (d_roadlist.size())
+    eraseLastRoadItem();
+
+  while (d_foglist.size())
+    eraseLastFogItem();
+
+  while (d_bridgelist.size())
+    eraseLastBridgeItem();
+
+  while (d_cursorlist.size())
+    eraseLastCursorItem();
+
+  while (d_flaglist.size())
+    eraseLastFlagItem();
+
+  while (d_selectorlist.size())
+    eraseLastSelectorItem();
+
+  while (d_shieldlist.size())
+    eraseLastShieldItem();
+
+  while (d_prodshieldlist.size())
+    eraseLastProdShieldItem();
+
+  while (d_portlist.size())
+    eraseLastPortItem();
+
+  while (d_signpostlist.size())
+    eraseLastSignpostItem();
+
+  while (d_baglist.size())
+    eraseLastBagItem();
+
+  while (d_explosionlist.size())
+    eraseLastExplosionItem();
+
+  while (d_movebonuslist.size())
+    eraseLastMoveBonusItem();
+
+  while (d_tilelist.size())
+    eraseLastTileItem();
+
+  while (d_armylist.size())
+    eraseLastArmyItem();
+
+  return;
+}
