@@ -1,6 +1,6 @@
 // Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
 // Copyright (C) 2004, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ struct MoveBonusCacheItem;
 struct FogCacheItem;
 struct PlantedStandardCacheItem;
 struct NewLevelCacheItem;
+struct DefaultTileStyleCacheItem;
 struct TileCacheItem;
 struct PortCacheItem;
 struct SignpostCacheItem;
@@ -137,7 +138,7 @@ class GraphicsCache
                                 const bool* medals, bool greyed = false);
 	PixMask* getArmyPic(Army *a, bool greyed = false);
 
-	PixMask* getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid, guint32 tileset, guint32 cityset);
+	PixMask* getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid, guint32 tileset, guint32 cityset, guint32 shieldset);
 	PixMask* getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid);
 
         /** Method for getting the shield picture from the cache
@@ -270,6 +271,12 @@ class GraphicsCache
 	 */
         PixMask* getNewLevelPic(const Player* p, guint32 gender);
 
+        /** Method for getting a picture that represents a type of tile style.
+         *  The parameter is related to tilestyle.h:TileStyle::Type.
+         */
+        PixMask* getDefaultTileStylePic(guint32 tilestyle_type, 
+                                        guint32 tilesize);
+
         /** Method for getting a city picture
           * 
           * For simplicity we have extended the basic_image/mask style to
@@ -405,6 +412,9 @@ class GraphicsCache
           */
         static PixMask* getMiscPicture(std::string picname, bool alpha=true);
 
+        //! Erase cached graphics.
+        void reset();
+
     private:
         GraphicsCache();
         ~GraphicsCache();
@@ -466,6 +476,10 @@ class GraphicsCache
 	//! Creates a new new-level picture in the player's colour.
         NewLevelCacheItem* addNewLevelPic(const Player* p, guint32 gender);
 
+        //! Creates a new default tile style picture.
+        DefaultTileStyleCacheItem* addDefaultTileStylePic(guint32 type,
+                                                          guint32 tilesize);
+
         //! Creates a new flag picture with the given parameters.
         FlagCacheItem* addFlagPic(int size, const Player *p, guint32 tileset);
 
@@ -537,6 +551,9 @@ class GraphicsCache
 	//! Erase the oldest picture of a hero gaining a level.
         void eraseLastNewLevelItem();
 
+        //! Erase the oldest picture of a default tile style.
+        void eraseLastDefaultTileStyleItem();
+
         //! Erases the oldest flag cache item
         void eraseLastFlagItem();
 
@@ -579,10 +596,13 @@ class GraphicsCache
 	//! Loads the images associated with heroes gaining a new level.
         void loadNewLevelPics();
 
+        //! Loads the images associated with default tile styles.
+        void loadDefaultTileStylePics();
+
         //! Loads the images for the movement bonuses
         void loadMoveBonusPics();
 
-	void drawTilePic(PixMask *surface, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 ts, bool has_grid, guint32 tileset, guint32 cityset);
+	void drawTilePic(PixMask *surface, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 ts, bool has_grid, guint32 tileset, guint32 cityset, guint32 shieldset);
 
         /** Loads an image
           * 
@@ -635,6 +655,7 @@ class GraphicsCache
         std::list<BagCacheItem*> d_baglist;
         std::list<ExplosionCacheItem*> d_explosionlist;
         std::list<NewLevelCacheItem*> d_newlevellist;
+        std::list<DefaultTileStyleCacheItem*> d_defaulttilestylelist;
 
         //some private surfaces
         PixMask* d_diplomacypic[2][DIPLOMACY_TYPES];
@@ -655,6 +676,7 @@ class GraphicsCache
 	PixMask *d_newlevelmask_male;
 	PixMask *d_newlevel_female;
 	PixMask *d_newlevelmask_female;
+        PixMask *d_default_tilestyles[DEFAULT_TILESTYLE_TYPES];
 };
 
 bool operator <(ArmyCacheItem lhs, ArmyCacheItem rhs);
