@@ -1,6 +1,6 @@
 // Copyright (C) 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2001, 2002, 2004, 2005, 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 class Tileset;
 class SmallTile;
+class Tar_Helper;
 //! Describes a kind of tile that a Stack can traverse.
 /** 
  * Many tiles are put together to form a tileset. Thus, a tile describes a
@@ -138,13 +139,19 @@ class Tile : public std::list<TileStyleSet*>
 	 */
 	TileStyle *getRandomTileStyle (TileStyle::Type style) const;
 
+        guint32 countTileStyles(TileStyle::Type type) const;
+
+        std::list<TileStyle*> getTileStyles(TileStyle::Type type) const;
+
+        TileStyle* getTileStyle(guint32 id) const;
+
 	// Methods the operate on the class data and modify the class
 
 	//! Destroy the images associated with this tile.
 	void uninstantiateImages();
 
 	//! Load the images associated with this tile.
-	void instantiateImages(int tilesize, Tileset *ts);
+	void instantiateImages(int tilesize, Tar_Helper *t);
 
 
 	// Static Methods
