@@ -1,6 +1,6 @@
 // Copyright (C) 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -108,11 +108,16 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
         //! Returns the different tilesizes present in the armysetlist.
 	void getSizes(std::list<guint32> &sizes);
 
-        //! Returns the names of all Armyset objects available to the game.
-	std::list<std::string> getNames();
+        //! Returns whether the given name is also in our list of armysets.
+        bool contains(std::string name) const;
 
         //! Returns the names of armysets that have the given tile size.
-	std::list<std::string> getNames(guint32 tilesize);
+	std::list<std::string> getValidNames(guint32 tilesize);
+
+        //! Returns the names of all Armyset objects available to the game.
+	std::list<std::string> getValidNames() const;
+
+
 
         /** Returns the Id of a specific armyset by name
           * 
@@ -121,8 +126,8 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
 	  *
           * @return the id of the armyset (0 on error)
           */
-	guint32 getArmysetId(std::string armyset, guint32 tilesize);
-	Armyset *getArmyset(guint32 id);
+	guint32 getArmysetId(std::string armyset, guint32 tilesize) const;
+	Armyset *getArmyset(guint32 id) const;
 
 	//! Return the Armyset object by the name of the subdir.
 	/**
@@ -130,7 +135,7 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
 	 *             This value does not contain any slashes, and is
 	 *             presumed to be found inside the army/ directory.
 	 */
-	Armyset *getArmyset(std::string dir);
+	Armyset *getArmyset(std::string dir) const;
 
 	//! Return the name of the subdirectory for a given armyset.
         /** 
@@ -141,21 +146,23 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
 	 *         Armyset::d_dir for more information about the nature of 
 	 *         the return value.
          */
-	std::string getArmysetDir(std::string name, guint32 tilesize);
+	std::string getArmysetDir(std::string name, guint32 tilesize) const;
 
 
 	//! Return an unused armyset number.
 	static int getNextAvailableId(int after = 0);
 
-	void add(Armyset *arymset);
+	void add(Armyset *arymset, std::string file);
 
 	void instantiateImages();
 	void uninstantiateImages();
 	
 
 	Armyset *import(Tar_Helper *t, std::string f, bool &broken);
-    private:
 	bool addToPersonalCollection(Armyset *armyset, std::string &new_subdir, guint32 &new_id);
+
+        std::string findFreeBaseName(std::string basename, guint32 max, guint32 &num) const;
+    private:
         //! Default Constructor.  Loads all armyset objects it can find.
 	/**
 	 * The army/ directory is scanned for armyset directories.
@@ -182,6 +189,13 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
 	 * user's personal collection.
 	 */
 	void loadArmysets(std::list<std::string> armysets);
+
+        //! Returns the names of armysets that have the given tile size.
+	std::list<std::string> getNames(guint32 tilesize);
+
+        //! Returns the names of all Armyset objects available to the game.
+	std::list<std::string> getNames() const;
+
         
         typedef std::map<guint32, std::vector<ArmyProto*> > ArmyPrototypeMap;
         typedef std::map<guint32, std::string> NameMap;
@@ -201,7 +215,7 @@ class Armysetlist : public std::list<Armyset*>, public sigc::trackable
 	//! A map that provides an Armyset when supplying a subdirectory name.
         ArmysetMap d_armysets;
 
-	//! A map that provides an Armyset when supplying a subdirectory name.
+	//! A map that provides an Armyset when supplying an Armyset::d_id.
         ArmysetIdMap d_armysetids;
 
         //! A static pointer for the singleton instance.
