@@ -1,4 +1,4 @@
-// Copyright (C) 2008, 2009 Ben Asselstine
+// Copyright (C) 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -37,10 +37,12 @@ class XML_Helper;
  *
  * The Cityset dictates the size of city images.
  *
- * Citysets are referred to by their subdirectory name.
+ * Citysets are referred to by their base name.  The base name is the last
+ * part of the file's path minus the file extension.
  *
- * The cityset configuration file is a same named XML file inside the 
- * cityset's directory.  E.g. cityset/${Cityset::d_subdir}/${Cityset::d_subdir}.xml.
+ * The cityset configuration file is a tar file that contains an XML file, 
+ * and a set of png files.  Filenames have the following form:
+ * cityset/${Cityset::d_basename}.lwc.
  */
 class Cityset : public sigc::trackable, public Set
 {
@@ -69,17 +71,19 @@ class Cityset : public sigc::trackable, public Set
 	 */
         Cityset(XML_Helper* helper, std::string directory);
 
-	static Cityset *create(std::string file);
+	static Cityset *create(std::string file, bool &unsupported_version);
 	//! Destructor.
         ~Cityset();
 
-	bool save(XML_Helper *helper);
+	bool save(XML_Helper *helper) const;
 
-	//! Get the directory in which the cityset configuration file resides.
-        std::string getSubDir() const {return d_subdir;}
+        bool save(std::string filename, std::string extension) const;
 
-	//! Set the direction where the shieldset configuration file resides.
-        void setSubDir(std::string dir) {d_subdir = dir;}
+	//! Get the base name of the file holding the cityset configuration file.
+        std::string getBaseName() const {return d_basename;}
+
+	//! Set the base name of the shieldset configuration file.
+        void setBaseName(std::string base) {d_basename = base;}
 
         //! Returns the name of the cityset.
         std::string getName() const {return _(d_name.c_str());}
@@ -117,9 +121,11 @@ class Cityset : public sigc::trackable, public Set
 	//! Sets the description of the cityset.
 	void setInfo(std::string description) {d_info = description;};
 
-        //! Returns the width and height in pixels of the city images.
+        //! Returns the width and height in pixels of a square on the map.
         guint32 getTileSize() const {return d_tileSize;}
 
+        void setTileSize(guint32 tile_size) {d_tileSize = tile_size;}
+
 	void setCitiesFilename(std::string s) {d_cities_filename = s;};
 	std::string getCitiesFilename() {return d_cities_filename;};
 	void setRazedCitiesFilename(std::string s) {d_razedcities_filename = s;};
@@ -153,6 +159,9 @@ class Cityset : public sigc::trackable, public Set
 	//! get filenames in this cityset, excepting the configuration file.
 	void getFilenames(std::list<std::string> &files);
 
+        //! Delete the cityset's temporary directory.
+        void clean_tmp_dir() const;
+
 	void instantiateImages();
 	void instantiateImages(std::string port_filename,
 			       std::string signpost_filename,
@@ -163,11 +172,12 @@ class Cityset : public sigc::trackable, public Set
 			       std::string temples_filename);
 	void uninstantiateImages();
 
-	std::string getConfigurationFile();
+	std::string getConfigurationFile() const;
 
 	static std::list<std::string> scanSystemCollection();
 	static std::list<std::string> scanUserCollection();
-
+        static bool copy(std::string src, std::string dest);
+        guint32 countEmptyImageNames() const;
 	guint32 getCityTileWidth() {return d_city_tile_width;};
 	void setCityTileWidth(guint32 tiles) {d_city_tile_width = tiles;};
 	guint32 getTempleTileWidth() {return d_temple_tile_width;};
@@ -186,7 +196,12 @@ class Cityset : public sigc::trackable, public Set
 	bool validateRuinTileWidth();
 	bool validateTempleTileWidth();
 	bool tileWidthsEqual(Cityset *cityset);
+        std::string getFileFromConfigurationFile(std::string file);
+        bool replaceFileInConfigurationFile(std::string file, std::string new_file);
 
+        //! Load the cityset again.
+        void reload();
+        guint32 calculate_preferred_tile_size() const;
     private:
 
         // DATA
@@ -225,13 +240,13 @@ class Cityset : public sigc::trackable, public Set
 	 */
         guint32 d_tileSize;
 
-	//! The subdirectory of the cityset.
+	//! The base name of the cityset.
 	/**
-	 * This is the name of the subdirectory that the Cityset files are
-	 * residing in.  It does not contain a path (e.g. no slashes).
+         * The basename is the final portion of the file's path, but with the
+         * file extension removed.
 	 * Cityset directories sit in the citysets/ directory.
 	 */
-        std::string d_subdir;
+        std::string d_basename;
 
 
 	std::string d_cities_filename;
