@@ -1,4 +1,4 @@
-// Copyright (C) 2008 Ben Asselstine
+// Copyright (C) 2008, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <iostream>
 #include <algorithm>
 #include <expat.h>
+#include <assert.h>
 #include "rectangle.h"
 #include <sigc++/functors/mem_fun.h>
 
@@ -25,11 +26,12 @@
 #include "ucompose.hpp"
 #include "File.h"
 #include "defs.h"
+#include "tarhelper.h"
 
 using namespace std;
 
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
-//#define debug(x)
+//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x)
 
 Citysetlist* Citysetlist::s_instance = 0;
 
@@ -58,7 +60,7 @@ void Citysetlist::loadCitysets(std::list<std::string> citysets)
 	if (!cityset)
 	  continue;
 
-	add(cityset);
+	add(cityset, *i);
       }
 }
 
@@ -78,10 +80,28 @@ Citysetlist::~Citysetlist()
     delete (*it);
 }
 
-std::list<std::string> Citysetlist::getNames()
+std::list<std::string> Citysetlist::getValidNames() const
+{
+  std::list<std::string> names;
+  for (const_iterator it = begin(); it != end(); it++)
+    if ((*it)->validate() == true)
+      names.push_back((*it)->getName());
+  return names;
+}
+
+std::list<std::string> Citysetlist::getValidNames(guint32 tilesize)
 {
   std::list<std::string> names;
   for (iterator it = begin(); it != end(); it++)
+    if ((*it)->getTileSize() == tilesize && (*it)->validate() == true)
+      names.push_back((*it)->getName());
+  return names;
+}
+
+std::list<std::string> Citysetlist::getNames() const
+{
+  std::list<std::string> names;
+  for (const_iterator it = begin(); it != end(); it++)
     names.push_back((*it)->getName());
   return names;
 }
@@ -97,24 +117,31 @@ std::list<std::string> Citysetlist::getNames(guint32 tilesize)
 
 Cityset *Citysetlist::loadCityset(std::string name)
 {
-  debug("Loading cityset " <<File::get_basename(File::get_dirname(name)));
+  debug("Loading cityset " <<File::get_basename(name));
+  bool unsupported_version = false;
 
-  Cityset *cityset = Cityset::create(name);
+  Cityset *cityset = Cityset::create(name, unsupported_version);
   if (!cityset)
-    return NULL;
-    
-  if (cityset->validate() == false)
     {
-      cerr << "Error!  cityset: `" << cityset->getName() << 
-	"' is invalid.  Skipping." << endl;
+      cerr << "Error!  cityset: `" << File::get_basename(name, true) << 
+	"' is malformed.  Skipping." << endl;
+      return NULL;
+    }
+    
+  if (d_citysets.find(cityset->getBaseName()) != d_citysets.end())
+    {
+      Cityset *c = (*d_citysets.find(cityset->getBaseName())).second;
+      cerr << "Error!  cityset: `" << cityset->getConfigurationFile() << 
+	"' shares a duplicate cityset basename `" << c->getBaseName() << "' with `" << c->getConfigurationFile() 
+	<< "'.  Skipping." << endl;
       delete cityset;
       return NULL;
     }
-
+    
   if (d_citysetids.find(cityset->getId()) != d_citysetids.end())
     {
       Cityset *c = (*d_citysetids.find(cityset->getId())).second;
-      cerr << "Error!  cityset: `" << cityset->getName() << 
+      cerr << "Error!  cityset: `" << cityset->getConfigurationFile() << 
 	"' shares a duplicate cityset id with `" << c->getConfigurationFile() 
 	<< "'.  Skipping." << endl;
       delete cityset;
@@ -133,13 +160,13 @@ Cityset *Citysetlist::loadCityset(std::string name)
   return cityset;
 }
 
-void Citysetlist::add(Cityset *cityset)
+void Citysetlist::add(Cityset *cityset, std::string file)
 {
-  std::string subdir = File::get_basename(cityset->getDirectory());
+  std::string basename = File::get_basename(file);
   push_back(cityset);
-  cityset->setSubDir(subdir);
-  d_dirs[String::ucompose("%1 %2", cityset->getName(), cityset->getTileSize())] = subdir;
-  d_citysets[subdir] = cityset;
+  cityset->setBaseName(basename);
+  d_dirs[String::ucompose("%1 %2", cityset->getName(), cityset->getTileSize())] = basename;
+  d_citysets[basename] = cityset;
   d_citysetids[cityset->getId()] = cityset;
 }
 
@@ -152,9 +179,14 @@ void Citysetlist::getSizes(std::list<guint32> &sizes)
     }
 }
 
-std::string Citysetlist::getCitysetDir(std::string name, guint32 tilesize)
+std::string Citysetlist::getCitysetDir(std::string name, guint32 tilesize) const
 {
-  return d_dirs[String::ucompose("%1 %2", name, tilesize)];
+  std::string name_and_size = String::ucompose("%1 %2", name, tilesize);
+  DirMap::const_iterator it = d_dirs.find(name_and_size);
+  if (it == d_dirs.end())
+    return "";
+  else
+    return (*it).second;
 }
 void Citysetlist::instantiateImages()
 {
@@ -167,89 +199,119 @@ void Citysetlist::uninstantiateImages()
     (*it)->uninstantiateImages();
 }
 	
-Cityset *Citysetlist::getCityset(guint32 id) 
+Cityset *Citysetlist::getCityset(guint32 id) const
 { 
-  if (d_citysetids.find(id) == d_citysetids.end())
+  CitysetIdMap::const_iterator it = d_citysetids.find(id);
+  if (it == d_citysetids.end())
     return NULL;
-  return d_citysetids[id];
+  return (*it).second;
 }
 	
-Cityset *Citysetlist::getCityset(std::string dir) 
+Cityset *Citysetlist::getCityset(std::string bname) const
 { 
-  if (d_citysets.find(dir) == d_citysets.end())
+  CitysetMap::const_iterator it = d_citysets.find(bname);
+  if (it == d_citysets.end())
     return NULL;
-  return d_citysets[dir];
+  return (*it).second;
 }
-bool Citysetlist::addToPersonalCollection(Cityset *cityset, std::string &new_subdir, guint32 &new_id)
+
+Cityset *Citysetlist::import(Tar_Helper *t, std::string f, bool &broken)
+{
+  bool unsupported_version = false;
+  std::string filename = t->getFile(f, broken);
+  if (broken)
+    return NULL;
+  Cityset *cityset = Cityset::create(filename, unsupported_version);
+  assert (cityset != NULL);
+  cityset->setBaseName(File::get_basename(f));
+
+  std::string basename = "";
+  guint32 id = 0;
+  addToPersonalCollection(cityset, basename, id);
+
+  return cityset;
+}
+
+std::string Citysetlist::findFreeBaseName(std::string basename, guint32 max, guint32 &num) const
+{
+  std::string new_basename;
+  for (unsigned int count = 1; count < max; count++)
+    {
+      new_basename = String::ucompose("%1%2", basename, count);
+      if (getCityset(new_basename) == NULL)
+        {
+          num = count;
+          break;
+        }
+      else
+        new_basename = "";
+    }
+  return new_basename;
+}
+
+bool Citysetlist::addToPersonalCollection(Cityset *cityset, std::string &new_basename, guint32 &new_id)
 {
   //do we already have this one?
-  if (getCityset(cityset->getSubDir()) == getCityset(cityset->getId()) &&
-      getCityset(cityset->getSubDir()) != NULL)
+      
+  if (getCityset(cityset->getBaseName()) == getCityset(cityset->getId()) 
+      && getCityset(cityset->getBaseName()) != NULL)
     {
       cityset->setDirectory(getCityset(cityset->getId())->getDirectory());
       return true;
     }
 
-  //if the subdir conflicts with any other subdir, then change it.
-  if (getCityset(cityset->getSubDir()) != NULL)
+  //if the basename conflicts with any other basename, then change it.
+  if (getCityset(cityset->getBaseName()) != NULL)
     {
-      bool found = false;
-      for (int count = 0; count < 100; count++)
-	{
-	  new_subdir = String::ucompose("%1%2", cityset->getSubDir(), count);
-	  if (getCityset(new_subdir) == NULL)
-	    {
-	      found = true;
-	      break;
-	    }
-	}
-      if (found == false)
-	return false;
-      cityset->setSubDir(new_subdir);
+      if (new_basename != "" && getCityset(new_basename) == NULL)
+        ;
+      else
+        {
+          guint32 num = 0;
+          std::string new_basename = findFreeBaseName(cityset->getBaseName(), 100, num);
+          if (new_basename == "")
+            return false;
+        }
     }
-  else
-    new_subdir = cityset->getSubDir();
+  else if (new_basename == "")
+    new_basename = cityset->getBaseName();
 
   //if the id conflicts with any other id, then change it
   if (getCityset(cityset->getId()) != NULL)
     {
-      new_id = Citysetlist::getNextAvailableId(cityset->getId());
-      cityset->setId(new_id);
+      if (new_id != 0 && getCityset(new_id) == NULL)
+        cityset->setId(new_id);
+      else
+        {
+          new_id = Citysetlist::getNextAvailableId(cityset->getId());
+          cityset->setId(new_id);
+        }
     }
   else
     new_id = cityset->getId();
 
   //make the directory where the cityset is going to live.
-  std::string directory = 
-    File::getUserCitysetDir() + cityset->getSubDir() + "/";
+  std::string file = File::getUserCitysetDir() + new_basename + Cityset::file_extension;
 
-  if (File::create_dir(directory) == false)
-    return false;
+  cityset->save(file, Cityset::file_extension);
 
-  //okay now we copy the image files into the new directory 
-  std::list<std::string> files;
-  cityset->getFilenames(files);
-  for (std::list<std::string>::iterator it = files.begin(); it != files.end();
-       it++)
-    File::copy(cityset->getFile(*it), directory + *it);
-
-  //save out the cityset file
-  cityset->setDirectory(directory);
-  XML_Helper helper(cityset->getConfigurationFile(), std::ios::out, false);
-  cityset->save(&helper);
-  helper.close();
+  if (new_basename != cityset->getBaseName())
+    cityset->setBaseName(new_basename);
+  cityset->setDirectory(File::get_dirname(file));
+  add (cityset, file);
   return true;
 }
 
 int Citysetlist::getNextAvailableId(int after)
 {
+  bool unsupported_version = false;
   std::list<guint32> ids;
   std::list<std::string> citysets = Cityset::scanSystemCollection();
-  //there might be IDs in invalid armysets.
+  //there might be IDs in invalid citysets.
   for (std::list<std::string>::const_iterator i = citysets.begin(); 
        i != citysets.end(); i++)
     {
-      Cityset *cityset = Cityset::create(*i);
+      Cityset *cityset = Cityset::create(*i, unsupported_version);
       if (cityset != NULL)
 	{
 	  ids.push_back(cityset->getId());
@@ -260,7 +322,7 @@ int Citysetlist::getNextAvailableId(int after)
   for (std::list<std::string>::const_iterator i = citysets.begin(); 
        i != citysets.end(); i++)
     {
-      Cityset *cityset = Cityset::create(*i);
+      Cityset *cityset = Cityset::create(*i, unsupported_version);
       if (cityset != NULL)
 	{
 	  ids.push_back(cityset->getId());
@@ -274,3 +336,23 @@ int Citysetlist::getNextAvailableId(int after)
     }
   return -1;
 }
+
+bool Citysetlist::contains(std::string name) const
+{
+  std::list<std::string> n = getNames();
+  for (std::list<std::string>::iterator it = n.begin(); it != n.end(); it++)
+    {
+      if (*it == name)
+        return true;
+    }
+  return false;
+}
+
+guint32 Citysetlist::getCitysetId(std::string basename) const
+{
+  Cityset *cs = getCityset(basename);
+  if (cs)
+    return cs->getId();
+  else
+    return 0;
+}
