@@ -3,7 +3,7 @@
 // Copyright (C) 2003, 2004, 2005, 2006 Andrea Paternesi
 // Copyright (C) 2004 David Sterba
 // Copyright (C) 2005 Bryan Duff
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,12 +25,18 @@
 #ifndef DEFINITIONS_H
 #define DEFINITIONS_H
 
+#include <gtkmm.h>
 #include <string>
 #include <libintl.h>
 
-#define LORDSAWAR_SAVEGAME_VERSION "0.1.8"
-#define LORDSAWAR_CONFIG_VERSION "0.1.8"
-#define LORDSAWAR_RECENTLY_PLAYED_VERSION "0.1.8"
+#define LORDSAWAR_SAVEGAME_VERSION "0.1.9"
+#define LORDSAWAR_TILESET_VERSION "0.1.9"
+#define LORDSAWAR_ARMYSET_VERSION "0.1.9"
+#define LORDSAWAR_CITYSET_VERSION "0.1.9"
+#define LORDSAWAR_SHIELDSET_VERSION "0.1.9"
+#define LORDSAWAR_CONFIG_VERSION "0.1.9"
+#define LORDSAWAR_RECENTLY_PLAYED_VERSION "0.1.9"
+#define LORDSAWAR_RECENTLY_EDITED_VERSION "0.1.9"
 #define _(string) gettext(string) // Macro for the gettext
 #define __(astring) std::string(gettext(astring.c_str()))
 
@@ -49,6 +55,7 @@ const unsigned int ROAD_TYPES = 15;
 const unsigned int FOG_TYPES = 15;
 const unsigned int BRIDGE_TYPES = 4;
 const unsigned int CURSOR_TYPES = 12;
+const unsigned int DEFAULT_TILESTYLE_TYPES = 18;
 const unsigned int MAX_CITIES_VECTORED_TO_ONE_CITY = 4;
 const unsigned int MAX_TURNS_FOR_VECTORING = 2;
 const unsigned int MAX_BOAT_MOVES = 18;
@@ -96,6 +103,11 @@ const std::string ARMYSET_EXT = ".lwa";
 const std::string TILESET_EXT = ".lwt";
 const std::string CITYSET_EXT = ".lwc";
 const std::string SHIELDSET_EXT = ".lws";
+const std::string MAP_EXT = ".map";
+const std::string SAVE_EXT = ".sav";
+const std::string PBM_EXT = ".trn";
+const std::string RECENTLY_PLAYED_LIST = "recently-played.xml";
+const std::string RECENTLY_EDITED_LIST = "recently-edited.xml";
 
 const unsigned int MIN_PRODUCTION_TURNS_FOR_ARMY_UNITS = 1;
 const unsigned int MAX_PRODUCTION_TURNS_FOR_ARMY_UNITS = 5;
@@ -116,5 +128,19 @@ const unsigned int MAX_SIGHT_FOR_ARMY_UNITS = 25;
 
 const float SIGNPOST_FREQUENCY = 0.0030;
 
+const std::string YELLOW_COLOUR = "#FCFCECEC2020";
+const std::string ORANGE_COLOUR = "#FCFCA0A00000";
+const std::string WHITE_COLOUR = "#FFFFFFFFFFFF";
+const std::string BLACK_COLOUR = "#000000000000";
+const Gdk::Color SEND_VECTORED_UNIT_LINE_COLOUR(YELLOW_COLOUR);
+const Gdk::Color RECEIVE_VECTORED_UNIT_LINE_COLOUR(ORANGE_COLOUR);
+const Gdk::Color SELECTOR_BOX_COLOUR(WHITE_COLOUR);
+const Gdk::Color QUEST_LINE_COLOUR(ORANGE_COLOUR);
+const Gdk::Color QUESTMAP_TARGET_BOX_COLOUR(ORANGE_COLOUR);
+const Gdk::Color ROAD_PLANNER_TARGET_BOX_COLOUR(ORANGE_COLOUR);
+const Gdk::Color GRID_BOX_COLOUR(BLACK_COLOUR);
+const Gdk::Color FOG_COLOUR(BLACK_COLOUR);
+const Gdk::Color VECTORMAP_ACTIVE_BOX_COLOUR(WHITE_COLOUR);
+
 #endif // DEFINITIONS_H
 
