@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -86,6 +86,12 @@
 #include "timed-message-dialog.h"
 #include "backpack-editor-dialog.h"
 #include "MapBackpack.h"
+#include "shieldset-window.h"
+#include "cityset-window.h"
+#include "armyset-window.h"
+#include "tileset-window.h"
+#include "editor-quit-dialog.h"
+#include "smallmap-editor-dialog.h"
 
 
 MainWindow::MainWindow(std::string load_filename)
@@ -223,6 +229,21 @@ MainWindow::MainWindow(std::string load_filename)
     xml->get_widget("edit_map_info_menuitem", edit_map_info_menuitem);
     edit_map_info_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &MainWindow::on_edit_map_info_activated));
+    xml->get_widget("edit_shieldset_menuitem", edit_shieldset_menuitem);
+    edit_shieldset_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_edit_shieldset_activated));
+    xml->get_widget("edit_armyset_menuitem", edit_armyset_menuitem);
+    edit_armyset_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_edit_armyset_activated));
+    xml->get_widget("edit_cityset_menuitem", edit_cityset_menuitem);
+    edit_cityset_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_edit_cityset_activated));
+    xml->get_widget("edit_tileset_menuitem", edit_tileset_menuitem);
+    edit_tileset_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_edit_tileset_activated));
+    xml->get_widget("edit_smallmap_menuitem", edit_smallmap_menuitem);
+    edit_smallmap_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_edit_smallmap_activated));
     
     xml->get_widget("fullscreen_menuitem", fullscreen_menuitem);
     fullscreen_menuitem->signal_activate().connect
@@ -321,7 +342,7 @@ void MainWindow::setup_terrain_radiobuttons()
 					 terrain_type_table->children().end());
 
     // then add new ones from the tile set
-    Tileset *tset = GameMap::getInstance()->getTileset();
+    Tileset *tset = GameMap::getTileset();
     Gtk::RadioButton::Group group;
     bool group_set = false;
     const int no_columns = 6;
@@ -408,10 +429,6 @@ bool MainWindow::on_smallmap_exposed(GdkEventExpose *event)
 
 void MainWindow::init()
 {
-  Armysetlist::getInstance()->instantiateImages();
-  Tilesetlist::getInstance()->instantiateImages();
-  Shieldsetlist::getInstance()->instantiateImages();
-  Citysetlist::getInstance()->instantiateImages();
   show_initial_map();
   Playerlist::getInstance()->setActiveplayer(Playerlist::getInstance()->getNeutral());
   fill_players();
@@ -505,7 +522,7 @@ void MainWindow::set_filled_map(int width, int height, int fill_style, std::stri
     Playerlist::getInstance()->nextPlayer();
 
     // fill the map with tile type
-    Tileset* tset = GameMap::getInstance()->getTileset();
+    Tileset *tset = GameMap::getTileset();
     for (unsigned int i = 0; i < tset->size(); ++i)
     {
 	if ((*tset)[i]->getType() == fill_style)
@@ -516,6 +533,7 @@ void MainWindow::set_filled_map(int width, int height, int fill_style, std::stri
     }
 
     init_map_state();
+    GameMap::getInstance()->calculateBlockedAvenues();
 }
 
 void MainWindow::set_random_map(int width, int height,
@@ -579,7 +597,7 @@ void MainWindow::set_random_map(int width, int height,
     
     gen.setPercentages(water, forest, swamp, hills, mountains);
     
-    gen.setCityset(GameMap::getInstance()->getCityset());
+    gen.setCityset(GameMap::getCityset());
     gen.makeMap(width, height, false);
     GameMap::deleteInstance();
     GameMap::setWidth(width);
@@ -680,6 +698,7 @@ bool MainWindow::on_bigmap_mouse_button_event(GdkEventButton *e)
 	if (smallmap)
 	  smallmap->draw(Playerlist::getActiveplayer());
 	needs_saving = true;
+        update_window_title();
     }
     
     return true;
@@ -772,6 +791,7 @@ void MainWindow::on_new_map_activated()
 			   d.map.tileset, d.map.shieldset, d.map.cityset,
 			   d.map.armyset);
         needs_saving = true;
+        update_window_title();
     }
 }
 
@@ -815,6 +835,9 @@ void MainWindow::on_load_map_activated()
 
 	init_map_state();
 	bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+        needs_saving = false;
+        update_window_title();
+        fill_players();
     }
 }
 
@@ -824,10 +847,17 @@ void MainWindow::on_save_map_activated()
 	on_save_map_as_activated();
     else
     {
-        needs_saving = false;
-	bool success = game_scenario->saveGame(current_save_filename, "map");
+	bool success = game_scenario->saveGame(current_save_filename, MAP_EXT);
 	if (!success)
+          {
 	    show_error(_("Map was not saved!"));
+            on_validate_activated();
+          }
+        else
+          {
+            needs_saving = false;
+            update_window_title();
+          }
     }
 }
 
@@ -892,7 +922,7 @@ void MainWindow::on_save_map_as_activated()
     Gtk::FileChooserDialog chooser(*window, _("Choose a Name"),
 				   Gtk::FILE_CHOOSER_ACTION_SAVE);
     Gtk::FileFilter sav_filter;
-    sav_filter.add_pattern("*.map");
+    sav_filter.add_pattern("*" + MAP_EXT);
     chooser.set_filter(sav_filter);
     chooser.set_current_folder(File::getUserMapDir());
 
@@ -905,13 +935,22 @@ void MainWindow::on_save_map_as_activated()
     
     if (res == Gtk::RESPONSE_ACCEPT)
     {
+        std::string old_save_filename = current_save_filename;
 	current_save_filename = chooser.get_filename();
 	chooser.hide();
 
-	bool success = game_scenario->saveGame(current_save_filename, "map");
+	bool success = game_scenario->saveGame(current_save_filename, MAP_EXT);
 	if (!success)
-	    show_error(_("Map was not saved!"));
-        needs_saving = false;
+          {
+            show_error(_("Map was not saved!"));
+            on_validate_activated();
+            current_save_filename = old_save_filename;
+          }
+        else
+          {
+            needs_saving = false;
+            update_window_title();
+          }
     }
 }
 
@@ -919,15 +958,10 @@ bool MainWindow::quit()
 {
   if (needs_saving)
     {
-      Gtk::Dialog* dialog;
-      Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + 
-					 "/editor-quit-dialog.ui");
-      xml->get_widget("dialog", dialog);
-      dialog->set_transient_for(*window);
-      int response = dialog->run();
-      dialog->hide();
-      delete dialog;
+      EditorQuitDialog d;
+      d.set_parent_window(*window);
+      int response = d.run();
+      d.hide();
       
       if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
 	return false;
@@ -958,6 +992,7 @@ void MainWindow::on_edit_players_activated()
 	if (Playerlist::getInstance()->getPlayer(active->getId()))
 	  Playerlist::getInstance()->setActiveplayer(active);
 	needs_saving = true;
+        update_window_title();
 	fill_players();
       }
 }
@@ -968,9 +1003,136 @@ void MainWindow::on_edit_map_info_activated()
     d.set_parent_window(*window);
     int response = d.run();
     if (response == Gtk::RESPONSE_ACCEPT)
-      needs_saving = true;
+      {
+        needs_saving = true;
+        update_window_title();
+      }
 }
 
+void MainWindow::on_edit_shieldset_activated()
+{
+  Gtk::Main *kit = Gtk::Main::instance();;
+  ShieldSetWindow* shieldset_window;
+  Shieldset *shieldset = GameMap::getShieldset();
+  std::string file = shieldset->getConfigurationFile();
+  shieldset_window = new ShieldSetWindow (file);
+  shieldset_window->shieldset_saved.connect
+    (sigc::mem_fun(this, &MainWindow::on_shieldset_saved));
+  kit->run(shieldset_window->get_window());
+  delete shieldset_window;
+}
+
+void MainWindow::on_shieldset_saved(guint32 id)
+{
+  //did we save the active shieldset?
+  Shieldset *shieldset = GameMap::getShieldset();
+  if (id == shieldset->getId())
+    {
+      GraphicsCache::getInstance()->reset();
+      GameMap::getInstance()->reloadShieldset();
+      fill_players();
+      bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+      redraw();
+      needs_saving = true;
+      update_window_title();
+    }
+}
+
+void MainWindow::on_edit_armyset_activated()
+{
+  Gtk::Main *kit = Gtk::Main::instance();;
+  guint32 army_set_id = Playerlist::getActiveplayer()->getArmyset();
+  Armyset *armyset = Armysetlist::getInstance()->getArmyset(army_set_id);
+  std::string file = armyset->getConfigurationFile();
+  ArmySetWindow* armyset_window = new ArmySetWindow (file);
+  armyset_window->armyset_saved.connect
+    (sigc::mem_fun(this, &MainWindow::on_armyset_saved));
+  kit->run(armyset_window->get_window());
+  delete armyset_window;
+}
+
+void MainWindow::on_armyset_saved(guint32 id)
+{
+  //did we save any of the active armysets?
+  if (Playerlist::getInstance()->hasArmyset(id) == true)
+    {
+      GraphicsCache::getInstance()->reset();
+      Armyset *armyset = Armysetlist::getInstance()->getArmyset(id);
+      GameMap::getInstance()->reloadArmyset(armyset);
+      bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+      redraw();
+      needs_saving = true;
+      update_window_title();
+    }
+}
+
+void MainWindow::on_edit_cityset_activated()
+{
+  Gtk::Main *kit = Gtk::Main::instance();;
+  Cityset *cityset = GameMap::getCityset();
+  std::string file = cityset->getConfigurationFile();
+  CitySetWindow* cityset_window = new CitySetWindow (file);
+  cityset_window->cityset_saved.connect
+    (sigc::mem_fun(this, &MainWindow::on_cityset_saved));
+  kit->run(cityset_window->get_window());
+  delete cityset_window;
+}
+
+void MainWindow::on_cityset_saved(guint32 id)
+{
+  //did we save the active cityset?
+  if (id == GameMap::getInstance()->getCitysetId())
+    {
+      GraphicsCache::getInstance()->reset();
+      GameMap::getInstance()->reloadCityset();
+      bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+      redraw();
+      needs_saving = true;
+      update_window_title();
+    }
+}
+
+void MainWindow::on_edit_smallmap_activated()
+{
+  SmallmapEditorDialog d;
+  d.set_parent_window(*window);
+  bool changed = d.run();
+  d.hide();
+  Rectangle r = Rectangle(0, 0, GameMap::getWidth(), GameMap::getHeight());
+  smallmap->redraw_tiles(r);
+  redraw();
+  if (changed)
+    needs_saving = true;
+  update_window_title();
+}
+
+void MainWindow::on_edit_tileset_activated()
+{
+  Gtk::Main *kit = Gtk::Main::instance();;
+  Tileset *tileset = GameMap::getTileset();
+  std::string file = tileset->getConfigurationFile();
+  TileSetWindow* tileset_window = new TileSetWindow (file);
+  tileset_window->tileset_saved.connect
+    (sigc::mem_fun(this, &MainWindow::on_tileset_saved));
+  kit->run(tileset_window->get_window());
+  delete tileset_window;
+}
+
+void MainWindow::on_tileset_saved(guint32 id)
+{
+  //did we save the active tileset?
+  if (id == GameMap::getInstance()->getTilesetId())
+    {
+      GraphicsCache::getInstance()->reset();
+      GameMap::getInstance()->reloadTileset();
+      bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+      redraw();
+      needs_saving = true;
+      update_window_title();
+    }
+}
+
+
 void MainWindow::on_fullscreen_activated()
 {
     if (fullscreen_menuitem->get_active())
@@ -978,12 +1140,13 @@ void MainWindow::on_fullscreen_activated()
     else
 	window->unfullscreen();
 }
-			 
+
 void MainWindow::on_tile_graphics_toggled()
 {
   bigmap->toggleViewStylesOrTypes();
   bigmap->draw(Playerlist::getViewingplayer());
 }
+
 void MainWindow::on_grid_toggled()
 {
   bigmap->toggle_grid();
@@ -1007,7 +1170,7 @@ void MainWindow::setup_tile_style_buttons(Tile::Type terrain)
 {
   Gtk::RadioButton::Group group;
   //iterate through tilestyles of a certain TERRAIN tile
-  Tileset *tileset = GameMap::getInstance()->getTileset();
+  Tileset *tileset = GameMap::getTileset();
   guint32 index = tileset->getIndex(terrain);
   Tile *tile = (*tileset)[index];
   if (tile == NULL)
@@ -1074,7 +1237,7 @@ void MainWindow::auto_select_appropriate_pointer()
     case Tile::HILLS:
     case Tile::VOID:
 	{
-	  Tileset *tileset = GameMap::getInstance()->getTileset();
+          Tileset *tileset = GameMap::getTileset();
 	  Tile *tile = (*tileset)[tileset->getIndex(get_terrain())];
 	  if (tile->consistsOnlyOfLoneAndOtherStyles())
 	    pointer_items[1].button->set_active();
@@ -1178,6 +1341,7 @@ void MainWindow::on_bigmap_changed(Glib::RefPtr<Gdk::Pixmap> map)
       window->invalidate_rect(r, true);
     }
 }
+
 void MainWindow::on_smallmap_changed(Glib::RefPtr<Gdk::Pixmap> map, Gdk::Rectangle r)
 {
   int width = 0;
@@ -1290,7 +1454,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	d.set_parent_window(*window);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
 
 	// we might have changed something visible
 	redraw();
@@ -1301,7 +1468,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	d.set_parent_window(*window);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
 
 	// we might have changed something visible
 	redraw();
@@ -1312,7 +1482,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	d.set_parent_window(*window);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
 	redraw();
     }
     else if (Signpost *o = dynamic_cast<Signpost *>(object))
@@ -1321,7 +1494,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	d.set_parent_window(*window);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
     }
     else if (Temple *o = dynamic_cast<Temple *>(object))
     {
@@ -1329,7 +1505,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	d.set_parent_window(*window);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
 
 	// we might have changed something visible
 	redraw();
@@ -1339,7 +1518,10 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 	BackpackEditorDialog d(b);
 	int response = d.run();
 	if (response == Gtk::RESPONSE_ACCEPT)
-	  needs_saving = true;
+          {
+            needs_saving = true;
+            update_window_title();
+          }
       }
 }
 
@@ -1361,7 +1543,10 @@ void MainWindow::on_edit_items_activated()
   d.set_parent_window(*window);
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
-    needs_saving = true;
+    {
+      needs_saving = true;
+      update_window_title();
+    }
 }
 
 void MainWindow::on_edit_rewards_activated()
@@ -1370,7 +1555,10 @@ void MainWindow::on_edit_rewards_activated()
   d.set_parent_window(*window);
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
-    needs_saving = true;
+    {
+      needs_saving = true;
+      update_window_title();
+    }
 }
 
 void MainWindow::randomize_city(City *c)
@@ -1387,6 +1575,7 @@ void MainWindow::on_random_all_cities_activated()
   for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
     randomize_city(*it);
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_random_unnamed_cities_activated()
@@ -1398,6 +1587,7 @@ void MainWindow::on_random_unnamed_cities_activated()
 	randomize_city(*it);
     }
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::randomize_ruin(Ruin *r)
@@ -1417,6 +1607,7 @@ void MainWindow::on_random_all_ruins_activated()
   for (Ruinlist::iterator it = rl->begin(); it != rl->end(); it++)
     randomize_ruin(*it);
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_random_unnamed_ruins_activated()
@@ -1428,6 +1619,7 @@ void MainWindow::on_random_unnamed_ruins_activated()
 	randomize_ruin(*it);
     }
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_random_all_temples_activated()
@@ -1445,6 +1637,7 @@ void MainWindow::on_random_all_temples_activated()
 	}
     }
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_random_unnamed_temples_activated()
@@ -1465,6 +1658,7 @@ void MainWindow::on_random_unnamed_temples_activated()
 	}
     }
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::randomize_signpost(Signpost *signpost)
@@ -1485,6 +1679,7 @@ void MainWindow::on_random_all_signs_activated()
   for (Signpostlist::iterator it = sl->begin(); it != sl->end(); it++)
     randomize_signpost(*it);
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_random_unnamed_signs_activated()
@@ -1496,6 +1691,7 @@ void MainWindow::on_random_unnamed_signs_activated()
 	randomize_signpost(*it);
     }
   needs_saving = true;
+  update_window_title();
 }
 
 void MainWindow::on_help_about_activated()
@@ -1562,7 +1758,7 @@ void MainWindow::on_import_map_activated()
 {
     Gtk::FileChooserDialog chooser(*window, _("Choose Game to Load Map from"));
     Gtk::FileFilter sav_filter;
-    sav_filter.add_pattern("*.sav");
+    sav_filter.add_pattern("*" + SAVE_EXT);
     chooser.set_filter(sav_filter);
     chooser.set_current_folder(File::getSavePath());
 
@@ -1602,6 +1798,7 @@ void MainWindow::on_import_map_activated()
 
 	init_map_state();
 	bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+        fill_players();
     }
 }
       
@@ -1618,13 +1815,14 @@ void MainWindow::on_switch_sets_activated()
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       needs_saving = true;
+      update_window_title();
+      GraphicsCache::getInstance()->reset();
       bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
       redraw();
       fill_players();
     }
 }
-    
-	
+
 void MainWindow::on_player_toggled(PlayerItem item)
 {
   Player *p = Playerlist::getInstance()->getPlayer(item.player_id);
@@ -1669,3 +1867,17 @@ void MainWindow::fill_players()
   if (!sensitive)
     players_hbox->set_sensitive(false);
 }
+
+void MainWindow::update_window_title()
+{
+  std::string title = "";
+  if (current_save_filename != "")
+    {
+      if (needs_saving == true)
+        title +="*";
+      title += File::get_basename(current_save_filename, true);
+      title += " - ";
+    }
+  title += _("LordsAWar! Scenario Editor");
+  window->set_title(title);
+}
