@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -43,15 +43,20 @@
 #include "File.h"
 #include "shield.h"
 #include "shieldsetlist.h"
+#include "recently-edited-file-list.h"
 
 #include "ucompose.hpp"
 
 #include "glade-helpers.h"
 #include "masked-image-editor-dialog.h"
+#include "recently-edited-file.h"
+#include "editor-quit-dialog.h"
+#include "editor-recover-dialog.h"
 
 
 ShieldSetWindow::ShieldSetWindow(std::string load_filename)
 {
+  autosave = File::getSavePath() + "autosave" + Shieldset::file_extension;
   needs_saving = false;
   d_shieldset = NULL;
     Glib::RefPtr<Gtk::Builder> xml
@@ -71,6 +76,9 @@ ShieldSetWindow::ShieldSetWindow(std::string load_filename)
     xml->get_widget("load_shieldset_menuitem", load_shieldset_menuitem);
     load_shieldset_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &ShieldSetWindow::on_load_shieldset_activated));
+    xml->get_widget("save_as_menuitem", save_as_menuitem);
+    save_as_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &ShieldSetWindow::on_save_as_activated));
     xml->get_widget("save_shieldset_menuitem", save_shieldset_menuitem);
     save_shieldset_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &ShieldSetWindow::on_save_shieldset_activated));
@@ -111,13 +119,55 @@ ShieldSetWindow::ShieldSetWindow(std::string load_filename)
 
     update_shield_panel();
 
+    if (load_filename != "")
+      current_save_filename = load_filename;
     update_shieldset_menuitems();
 
+    if (File::exists(autosave))
+      {
+        std::string m;
+        std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Shieldset::file_extension);
+        if (files.size() == 0)
+          m = _("Do you want to recover the session?");
+        else
+          {
+            RecentlyEditedShieldsetFile *r = dynamic_cast<RecentlyEditedShieldsetFile*>(files.front());
+            if (r->getName() == "")
+              m = String::ucompose(_("Do you want to recover %1?"),
+                                   File::get_basename(r->getFileName(), true));
+            else if (r->getName() != "" && r->getImagesNeeded() == 0)
+              m = String::ucompose(_("Do you want to recover %1 (%2)?"),
+                                   File::get_basename(r->getFileName(), true),
+                                   r->getName());
+            else if (r->getName() != "" && r->getImagesNeeded() > 0)
+              m = String::ucompose
+                (_("Do you want to recover %1 (%2, %3 images needed)?"),
+                 File::get_basename(r->getFileName(), true),
+                 r->getName(),
+                 r->getImagesNeeded());
+          }
+        EditorRecoverDialog d(m);
+        int response = d.run();
+        d.hide();
+        //ask if we want to recover the autosave.
+        if (response == Gtk::RESPONSE_ACCEPT)
+          {
+            load_shieldset (autosave);
+            update_shieldset_menuitems();
+            update_shield_panel();
+            return;
+          }
+      }
+
     if (load_filename.empty() == false)
       {
-	load_shieldset (load_filename);
-	update_shieldset_menuitems();
-	update_shield_panel();
+	bool success = load_shieldset (load_filename);
+        if (success)
+          {
+            update_shieldset_menuitems();
+            update_shield_panel();
+            update_window_title();
+          }
       }
 }
 
@@ -127,18 +177,14 @@ ShieldSetWindow::update_shieldset_menuitems()
   if (d_shieldset == NULL)
     {
       save_shieldset_menuitem->set_sensitive(false);
+      save_as_menuitem->set_sensitive(false);
       validate_shieldset_menuitem->set_sensitive(false);
       edit_shieldset_info_menuitem->set_sensitive(false);
     }
   else
     {
-      std::string file = d_shieldset->getConfigurationFile();
-      if (File::exists(file) == false)
-	save_shieldset_menuitem->set_sensitive(true);
-      else if (File::is_writable(file) == false)
-	save_shieldset_menuitem->set_sensitive(false);
-      else
-	save_shieldset_menuitem->set_sensitive(true);
+      save_shieldset_menuitem->set_sensitive(true);
+      save_as_menuitem->set_sensitive(true);
       edit_shieldset_info_menuitem->set_sensitive(true);
       validate_shieldset_menuitem->set_sensitive(true);
     }
@@ -200,7 +246,8 @@ void ShieldSetWindow::on_new_shieldset_activated()
   std::string name = "";
   int id = Shieldsetlist::getNextAvailableId(0);
   Shieldset *shieldset = new Shieldset(id, name);
-  ShieldSetInfoDialog d(shieldset, false);
+  ShieldSetInfoDialog d(shieldset, File::getUserShieldsetDir(), "", false,
+                        _("Make a New Shieldset"));
   d.set_parent_window(*window);
   int response = d.run();
   if (response != Gtk::RESPONSE_ACCEPT)
@@ -212,29 +259,32 @@ void ShieldSetWindow::on_new_shieldset_activated()
     delete d_shieldset;
   d_shieldset = shieldset;
   shields_list->clear();
-  std::string dir = File::getUserShieldsetDir() + "/" + d_shieldset->getSubDir();
+  std::string dir = File::getUserShieldsetDir();
   d_shieldset->setDirectory(dir);
-  File::create_dir(dir);
   current_save_filename = d_shieldset->getConfigurationFile();
+  RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
+  refl->updateEntry(current_save_filename);
+  refl->save();
+  d_shieldset->setDirectory(File::get_dirname(autosave));
+  d_shieldset->setBaseName(File::get_basename(autosave));
 
   //populate the list with initial entries.
   for (unsigned int i = Shield::WHITE; i <= Shield::NEUTRAL; i++)
     addNewShield(Shield::Colour(i), Shield::get_default_color_for_no(i));
   update_shieldset_menuitems();
 
-  XML_Helper helper(current_save_filename, std::ios::out, false);
-  d_shieldset->save(&helper);
-  helper.close();
+  d_shieldset->save(autosave, Shieldset::file_extension);
   update_shield_panel();
   needs_saving = true;
+  update_window_title();
 }
 
 void ShieldSetWindow::on_load_shieldset_activated()
 {
   Gtk::FileChooserDialog chooser(*window, 
-				 _("Choose an Shieldset to Load"));
+				 _("Choose a Shieldset to Load"));
   Gtk::FileFilter sav_filter;
-  sav_filter.add_pattern("*" + ARMYSET_EXT);
+  sav_filter.add_pattern("*" + SHIELDSET_EXT);
   chooser.set_filter(sav_filter);
   chooser.set_current_folder(File::getUserShieldsetDir());
 
@@ -247,8 +297,13 @@ void ShieldSetWindow::on_load_shieldset_activated()
 
   if (res == Gtk::RESPONSE_ACCEPT)
     {
-      load_shieldset(chooser.get_filename());
+      bool success = load_shieldset(chooser.get_filename());
       chooser.hide();
+      if (success)
+        {
+          needs_saving = false;
+          update_window_title();
+        }
     }
 
   update_shieldset_menuitems();
@@ -260,40 +315,121 @@ void ShieldSetWindow::on_validate_shieldset_activated()
   std::list<std::string> msgs;
   if (d_shieldset == NULL)
     return;
-  //bool valid;
+  bool valid = d_shieldset->validateNumberOfShields();
+  if (!valid)
+    msgs.push_back(_("The shieldset must have 9 shields in it."));
+
+  for (unsigned int i = Shield::WHITE; i <= Shield::NEUTRAL; i++)
+    {
+      valid = d_shieldset->validateShieldImages(Shield::Colour(i));
+      if (!valid)
+        {
+          std::string s;
+          s = String::ucompose(_("%1 must have all three images specified."),
+                               Shield::colourToString(Shield::Colour(i)));
+          msgs.push_back(s);
+          break;
+        }
+    }
 
   std::string msg = "";
   for (std::list<std::string>::iterator it = msgs.begin(); it != msgs.end();
        it++)
     msg += (*it) + "\n";
 
-  if (msg != "")
-    {
-      Gtk::MessageDialog dialog(*window, msg);
-      dialog.run();
-      dialog.hide();
-    }
+  if (msg == "")
+    msg = _("The shieldset is valid.");
+      
+  Gtk::MessageDialog dialog(*window, msg);
+  dialog.run();
+  dialog.hide();
 
   return;
 }
+
+void ShieldSetWindow::on_save_as_activated()
+{
+  std::string orig_basename = d_shieldset->getBaseName();
+  guint32 orig_id = d_shieldset->getId();
+  d_shieldset->setId(Shieldsetlist::getNextAvailableId(orig_id));
+  ShieldSetInfoDialog d(d_shieldset, File::getUserShieldsetDir(), "", false,
+                        _("Save a Copy of a Shieldset"));
+  d.set_parent_window(*window);
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    {
+      std::string new_basename = d_shieldset->getBaseName();
+      guint32 new_id = d_shieldset->getId();
+      d_shieldset->setId(orig_id);
+      d_shieldset->setBaseName(orig_basename);
+      bool success = Shieldsetlist::getInstance()->addToPersonalCollection(d_shieldset, new_basename, new_id);
+      if (success)
+        {
+          save_shieldset_menuitem->set_sensitive(true);
+          current_save_filename = d_shieldset->getConfigurationFile();
+          RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
+          refl->updateEntry(current_save_filename);
+          refl->save();
+          load_shieldset(current_save_filename);
+          needs_saving = false;
+          update_window_title();
+        }
+      else
+        {
+          std::string msg;
+          msg = _("Error!  Shieldset could not be saved.");
+          Gtk::MessageDialog dialog(*window, msg);
+          dialog.run();
+          dialog.hide();
+          d_shieldset->setId(orig_id);
+        }
+    }
+  else
+    {
+      d_shieldset->setId(orig_id);
+    }
+}
+
 void ShieldSetWindow::on_save_shieldset_activated()
 {
   if (current_save_filename.empty())
     current_save_filename = d_shieldset->getConfigurationFile();
   
-  XML_Helper helper(current_save_filename, std::ios::out, false);
-  d_shieldset->save(&helper);
-  helper.close();
-  needs_saving = false;
+  bool success = d_shieldset->save(autosave, Shieldset::file_extension);
+  if (success)
+    {
+      RecentlyEditedFileList::getInstance()->updateEntry(current_save_filename);
+      RecentlyEditedFileList::getInstance()->save();
+      success = Shieldset::copy (autosave, current_save_filename);
+      if (success == true)
+        {
+          needs_saving = false;
+          update_window_title();
+          shieldset_saved.emit(d_shieldset->getId());
+        }
+    }
+  if (!success)
+    {
+      std::string msg;
+      msg = _("Error!  Shieldset could not be saved.");
+      Gtk::MessageDialog dialog(*window, msg);
+      dialog.run();
+      dialog.hide();
+    }
 }
 
 void ShieldSetWindow::on_edit_shieldset_info_activated()
 {
-  ShieldSetInfoDialog d(d_shieldset, true);
+  ShieldSetInfoDialog d(d_shieldset, File::get_dirname(current_save_filename), 
+                        File::get_basename(current_save_filename), true, 
+                        _("Edit Shieldset Information"));
   d.set_parent_window(*window);
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
-    needs_saving = true;
+    {
+      needs_saving = true;
+      update_window_title();
+    }
 }
 
 void ShieldSetWindow::on_help_about_activated()
@@ -349,28 +485,48 @@ void ShieldSetWindow::fill_shield_info(Shield*shield)
     }
 }
 
-void ShieldSetWindow::load_shieldset(std::string filename)
+bool ShieldSetWindow::load_shieldset(std::string filename)
 {
+  std::string old_current_save_filename = current_save_filename;
   current_save_filename = filename;
+  if (filename != autosave)
+    Shieldset::copy(current_save_filename, autosave);
+  else
+    {
+      //go get the real name of the file
+      std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Shieldset::file_extension);
+      if (files.size() > 0)
+        current_save_filename = files.front()->getFileName();
+    }
 
   std::string name = File::get_basename(filename);
 
-  Shieldset *shieldset = Shieldset::create(filename);
+  bool unsupported_version = false;
+  Shieldset *shieldset = Shieldset::create(autosave, unsupported_version);
   if (shieldset == NULL)
     {
       std::string msg;
-      msg = "Error!  Shieldset could not be loaded.";
+      if (unsupported_version)
+        msg = _("Error!  The version of the shieldset is not supported.");
+      else
+        msg = _("Error!  Shieldset could not be loaded.");
       Gtk::MessageDialog dialog(*window, msg);
+      current_save_filename = old_current_save_filename;
       dialog.run();
       dialog.hide();
-      return;
+      return false;
+    }
+  if (File::nameEndsWith(current_save_filename, "/autosave" + Shieldset::file_extension) == false)
+    {
+      RecentlyEditedFileList::getInstance()->addEntry(current_save_filename);
+      RecentlyEditedFileList::getInstance()->save();
     }
   shields_list->clear();
   if (d_shieldset)
     delete d_shieldset;
   d_shieldset = shieldset;
 
-  d_shieldset->setSubDir(name);
+  d_shieldset->setBaseName(File::get_basename(autosave));
   d_shieldset->instantiateImages();
   for(Shieldset::iterator i = d_shieldset->begin(); 
       i != d_shieldset->end(); i++)
@@ -383,21 +539,18 @@ void ShieldSetWindow::load_shieldset(std::string filename)
 	shields_treeview->get_selection()->select(row);
     }
   update_shield_panel();
-
+  update_window_title();
+  return true;
 }
+
 bool ShieldSetWindow::quit()
 {
   if (needs_saving == true)
     {
-      Gtk::Dialog* dialog;
-      Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + 
-					 "/editor-quit-dialog.ui");
-      xml->get_widget("dialog", dialog);
-      dialog->set_transient_for(*window);
-      int response = dialog->run();
-      dialog->hide();
-      delete dialog;
+      EditorQuitDialog d;
+      int response = d.run();
+      d.set_parent_window(*window);
+      d.hide();
       
       if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
 	return false;
@@ -409,12 +562,17 @@ bool ShieldSetWindow::quit()
     }
   else
     window->hide();
+  File::erase(autosave);
+  if (d_shieldset)
+    delete d_shieldset;
   return true;
 }
+
 bool ShieldSetWindow::on_window_closed(GdkEventAny*)
 {
   return !quit();
 }
+
 void ShieldSetWindow::on_quit_activated()
 {
   quit();
@@ -432,21 +590,22 @@ void ShieldSetWindow::on_change_smallpic_clicked()
       std::string filename = "";
       ShieldStyle *ss = shield->getFirstShieldstyle(ShieldStyle::SMALL);
       if (ss->getImageName() != "")
-	filename = d_shieldset->getFile(ss->getImageName());
-      MaskedImageEditorDialog d(filename);
+	filename = d_shieldset->getFileFromConfigurationFile(ss->getImageName() +".png");
+      MaskedImageEditorDialog d(filename, d_shieldset);
       d.set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
       d.run();
+      if (filename != "")
+        File::erase(filename);
       if (d.get_selected_filename() != "")
 	{
 	  std::string file = File::get_basename(d.get_selected_filename());
-	  if (d.get_selected_filename() != d_shieldset->getFile(file))
+	  if (d.get_selected_filename() != filename)
 	    {
-	      //fixme:warn on overrwite.
-	      File::copy (d.get_selected_filename(), 
-			  d_shieldset->getFile(file));
+              d_shieldset->replaceFileInConfigurationFile(ss->getImageName()+".png", d.get_selected_filename());
+              ss->setImageName(file);
+              needs_saving = true;
+              update_window_title();
 	    }
-	  ss->setImageName(file);
-	  needs_saving = true;
 	  update_shield_panel();
 	}
     }
@@ -464,21 +623,20 @@ void ShieldSetWindow::on_change_mediumpic_clicked()
       std::string filename = "";
       ShieldStyle *ss = shield->getFirstShieldstyle(ShieldStyle::MEDIUM);
       if (ss->getImageName() != "")
-	filename = d_shieldset->getFile(ss->getImageName());
-      MaskedImageEditorDialog d(filename);
+	filename = d_shieldset->getFileFromConfigurationFile(ss->getImageName() +".png");
+      MaskedImageEditorDialog d(filename, d_shieldset);
       d.set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
       d.run();
       if (d.get_selected_filename() != "")
 	{
 	  std::string file = File::get_basename(d.get_selected_filename());
 	  if (d.get_selected_filename() != d_shieldset->getFile(file))
-	    {
-	      //fixme:warn on overrwite.
-	      File::copy (d.get_selected_filename(), 
-			  d_shieldset->getFile(file));
-	    }
-	  ss->setImageName(file);
-	  needs_saving = true;
+            {
+              d_shieldset->replaceFileInConfigurationFile(ss->getImageName()+".png", d.get_selected_filename());
+              ss->setImageName(file);
+              needs_saving = true;
+              update_window_title();
+            }
 	  update_shield_panel();
 	}
     }
@@ -496,21 +654,20 @@ void ShieldSetWindow::on_change_largepic_clicked()
       std::string filename = "";
       ShieldStyle *ss = shield->getFirstShieldstyle(ShieldStyle::LARGE);
       if (ss->getImageName() != "")
-	filename = d_shieldset->getFile(ss->getImageName());
-      MaskedImageEditorDialog d(filename);
+	filename = d_shieldset->getFileFromConfigurationFile(ss->getImageName() +".png");
+      MaskedImageEditorDialog d(filename, d_shieldset);
       d.set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
       d.run();
       if (d.get_selected_filename() != "")
 	{
 	  std::string file = File::get_basename(d.get_selected_filename());
 	  if (d.get_selected_filename() != d_shieldset->getFile(file))
-	    {
-	      //fixme:warn on overrwite.
-	      File::copy (d.get_selected_filename(), 
-			  d_shieldset->getFile(file));
-	    }
-	  ss->setImageName(file);
-	  needs_saving = true;
+            {
+              d_shieldset->replaceFileInConfigurationFile(ss->getImageName()+".png", d.get_selected_filename());
+              ss->setImageName(file);
+              needs_saving = true;
+              update_window_title();
+            }
 	  update_shield_panel();
 	}
 
@@ -528,6 +685,7 @@ void ShieldSetWindow::on_player_color_changed()
       Shield *s = row[shields_columns.shield];
       s->setColor(player_colorbutton->get_color());
       needs_saving = true;
+      update_window_title();
     }
 }
 
@@ -543,6 +701,7 @@ void ShieldSetWindow::addNewShield(Shield::Colour owner, Gdk::Color colour)
       Gtk::TreeIter i = shields_list->append();
       (*i)[shields_columns.name] = name;
       (*i)[shields_columns.shield] = shield;
+      d_shieldset->push_back(shield);
     }
 }
 
@@ -553,3 +712,14 @@ void ShieldSetWindow::loadShield(Shield *shield)
   (*i)[shields_columns.name] = name;
   (*i)[shields_columns.shield] = shield;
 }
+
+void ShieldSetWindow::update_window_title()
+{
+  std::string title = "";
+  if (needs_saving)
+    title += "*";
+  title += File::get_basename(current_save_filename, true);
+  title += " - ";
+  title += _("LordsAWar! Shieldset Editor");
+  window->set_title(title);
+}
