@@ -1,4 +1,4 @@
-//  Copyright (C) 2009 Ben Asselstine
+//  Copyright (C) 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "File.h"
 #include "GraphicsCache.h"
 #include "tile-preview-scene.h"
+#include "tarhelper.h"
 
 
 TilesetExplosionPictureEditorDialog::TilesetExplosionPictureEditorDialog(Tileset *tileset)
@@ -57,7 +58,10 @@ TilesetExplosionPictureEditorDialog::TilesetExplosionPictureEditorDialog(Tileset
     xml->get_widget("scene_image", scene_image);
     
     if (d_tileset->getExplosionFilename().empty() == false)
-      selected_filename = d_tileset->getFile(d_tileset->getExplosionFilename());
+      {
+        selected_filename = d_tileset->getFileFromConfigurationFile(d_tileset->getExplosionFilename() +".png");
+        delfiles.push_back(selected_filename);
+      }
     on_large_toggled();
 }
 
@@ -76,6 +80,17 @@ int TilesetExplosionPictureEditorDialog::run()
     dialog->show_all();
     int response = dialog->run();
 
+    if (std::find(delfiles.begin(), delfiles.end(), selected_filename)
+        == delfiles.end() && response == Gtk::RESPONSE_ACCEPT)
+      {
+        d_tileset->replaceFileInConfigurationFile(d_tileset->getExplosionFilename()+".png", selected_filename);
+        d_tileset->setExplosionFilename(File::get_basename(selected_filename));
+      }
+    else if (response == Gtk::RESPONSE_ACCEPT)
+      response = Gtk::RESPONSE_CANCEL;
+    for (std::list<std::string>::iterator it = delfiles.begin(); 
+         it != delfiles.end(); it++)
+      File::erase(*it);
     return response;
 }
 
@@ -107,18 +122,7 @@ void TilesetExplosionPictureEditorDialog::update_panel()
 
 void TilesetExplosionPictureEditorDialog::show_explosion_image(std::string filename)
 {
-  std::vector<PixMask* > tilestyle_images;
-  std::vector<PixMask*> base_tilestyles;
-  base_tilestyles = 
-    disassemble_row(File::getMiscFile("various/editor/tilestyles.png"), 17);
-
   guint32 size = d_tileset->getTileSize();
-  std::vector<PixMask*>::iterator it;
-  for (it = base_tilestyles.begin(); it != base_tilestyles.end(); it++)
-    {
-      PixMask::scale(*it, size, size);
-      tilestyle_images.push_back(*it);
-    }
   TilePreviewScene *s;
   std::string scene;
   guint32 idx = d_tileset->getIndex(Tile::GRASS);
@@ -134,7 +138,7 @@ void TilesetExplosionPictureEditorDialog::show_explosion_image(std::string filen
       scene += "aaaaaa";
       scene += "aaaaaa";
       scene += "aaaaaa";
-      s = new TilePreviewScene(grass, NULL, tilestyle_images, 6, 6, scene);
+      s = new TilePreviewScene(grass, NULL, 6, 6, scene, size);
       update_scene(s, filename);
     }
   else if (small_explosion_radiobutton->get_active() == true)
@@ -146,7 +150,7 @@ void TilesetExplosionPictureEditorDialog::show_explosion_image(std::string filen
       scene += "aaaaaaa";
       scene += "aaaaaaa";
       scene += "aaaaaaa";
-      s = new TilePreviewScene(grass, NULL, tilestyle_images, 7, 7, scene);
+      s = new TilePreviewScene(grass, NULL, 7, 7, scene, size);
       update_scene(s, filename);
     }
 }
