@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,13 @@
 #include "File.h"
 #include "overviewmap.h"
 #include "GraphicsCache.h"
+#include "recently-edited-file.h"
+#include "recently-edited-file-list.h"
+#include "tile-size-editor-dialog.h"
+#include "editor-quit-dialog.h"
+#include "editor-recover-dialog.h"
+#include "tilestyle-organizer-dialog.h"
+#include "tileset-smallmap-building-colors-dialog.h"
 
 #include "ucompose.hpp"
 
@@ -56,7 +63,9 @@
 
 TileSetWindow::TileSetWindow(std::string load_filename)
 {
+  autosave = File::getSavePath() + "autosave" + Tileset::file_extension;
   needs_saving = false;
+  inhibit_needs_saving = false;
   d_tileset = NULL;
     Glib::RefPtr<Gtk::Builder> xml
 	= Gtk::Builder::create_from_file(get_glade_path() + "/tileset-window.ui");
@@ -106,6 +115,7 @@ TileSetWindow::TileSetWindow(std::string load_filename)
     tilestyle_combobox->append_text(_("Top-Left To Bottom-Right"));
     tilestyle_combobox->append_text(_("Bottom-Left To Top-Right"));
     tilestyle_combobox->append_text(_("Other"));
+    tilestyle_combobox->append_text(_("Unknown"));
     tilestyle_combo_container->add(*manage(tilestyle_combobox));
     tilestyle_combo_container->show_all();
     tilestyle_combobox->signal_changed().connect
@@ -159,6 +169,9 @@ TileSetWindow::TileSetWindow(std::string load_filename)
     xml->get_widget("save_tileset_menuitem", save_tileset_menuitem);
     save_tileset_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &TileSetWindow::on_save_tileset_activated));
+    xml->get_widget("save_as_menuitem", save_as_menuitem);
+    save_as_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &TileSetWindow::on_save_as_activated));
     xml->get_widget("quit_menuitem", quit_menuitem);
     quit_menuitem->signal_activate().connect
        (sigc::mem_fun(this, &TileSetWindow::on_quit_activated));
@@ -187,6 +200,14 @@ TileSetWindow::TileSetWindow(std::string load_filename)
     xml->get_widget("preview_tile_menuitem", preview_tile_menuitem);
     preview_tile_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &TileSetWindow::on_preview_tile_activated));
+    xml->get_widget("organize_tilestyles_menuitem", organize_tilestyles_menuitem);
+    organize_tilestyles_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &TileSetWindow::on_organize_tilestyles_activated));
+    xml->get_widget("smallmap_building_colors_menuitem", 
+                    smallmap_building_colors_menuitem);
+    smallmap_building_colors_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, 
+                     &TileSetWindow::on_smallmap_building_colors_activated));
     xml->get_widget ("help_about_menuitem", help_about_menuitem);
     help_about_menuitem->signal_activate().connect
        (sigc::mem_fun(this, &TileSetWindow::on_help_about_activated));
@@ -226,22 +247,14 @@ TileSetWindow::TileSetWindow(std::string load_filename)
       (sigc::mem_fun(this, &TileSetWindow::on_remove_tilestyleset_clicked));
     xml->get_widget("tilestyleset_frame", tilestyleset_frame);
     xml->get_widget("tilestyle_frame", tilestyle_frame);
-    xml->get_widget("image_filechooser_button", image_filechooser_button);
-    image_filechooser_button->signal_selection_changed().connect
+    xml->get_widget("image_button", image_button);
+    image_button->signal_clicked().connect
       (sigc::mem_fun(*this, &TileSetWindow::on_image_chosen));
-    xml->get_widget("refresh_button", refresh_button);
-    refresh_button->signal_clicked().connect
-      (sigc::mem_fun(this, &TileSetWindow::on_refresh_clicked));
-
-    Gtk::FileFilter sav_filter;
-    sav_filter.add_pattern("*.png");
-    image_filechooser_button->set_filter(sav_filter);
-    image_filechooser_button->set_current_folder (File::getUserTilesetDir());
 
     xml->get_widget("tilestyle_standard_image", tilestyle_standard_image);
-    tilestyle_standard_images = 
-      disassemble_row(File::getMiscFile("various/editor/tilestyles.png"), 17);
 
+    if (load_filename != "")
+      current_save_filename = load_filename;
     update_tile_panel();
     update_tilestyleset_panel();
     update_tilestyle_panel();
@@ -256,7 +269,36 @@ TileSetWindow::TileSetWindow(std::string load_filename)
     tile_smallmap_surface = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(0), 32, 32, 24);
     tile_smallmap_surface_gc = Gdk::GC::create(tile_smallmap_surface);
 
-    inhibit_image_change = false;
+    if (File::exists(autosave))
+      {
+        std::string m;
+        std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Tileset::file_extension);
+        if (files.size() == 0)
+          m = _("Do you want to recover the session?");
+        else
+          {
+            RecentlyEditedTilesetFile *r = dynamic_cast<RecentlyEditedTilesetFile*>(files.front());
+            if (r->getName() == "")
+              m = String::ucompose(_("Do you want to recover %1 (%2 tiles)?"),
+                                   File::get_basename(r->getFileName(), true),
+                                   r->getNumberOfTiles());
+            else
+              m = String::ucompose
+                (_("Do you want to recover %1 (%2, %3 tiles)?"),
+                                   File::get_basename(r->getFileName(), true),
+                                   r->getName(), r->getNumberOfTiles());
+          }
+        EditorRecoverDialog d(m);
+        int response = d.run();
+        d.hide();
+        //ask if we want to recover the autosave.
+        if (response == Gtk::RESPONSE_ACCEPT)
+          {
+            load_tileset (autosave);
+            return;
+          }
+      }
+
     if (load_filename.empty() == false)
       load_tileset(load_filename);
 }
@@ -267,6 +309,7 @@ TileSetWindow::update_tileset_menuitems()
   if (d_tileset == NULL)
     {
       save_tileset_menuitem->set_sensitive(false);
+      save_as_menuitem->set_sensitive(false);
       army_unit_selector_menuitem->set_sensitive(false);
       edit_tileset_info_menuitem->set_sensitive(false);
       explosion_picture_menuitem->set_sensitive(false);
@@ -274,10 +317,12 @@ TileSetWindow::update_tileset_menuitems()
       bridges_picture_menuitem->set_sensitive(false);
       fog_picture_menuitem->set_sensitive(false);
       flags_picture_menuitem->set_sensitive(false);
+      organize_tilestyles_menuitem->set_sensitive(false);
     }
   else
     {
       save_tileset_menuitem->set_sensitive(true);
+      save_as_menuitem->set_sensitive(true);
       army_unit_selector_menuitem->set_sensitive(true);
       edit_tileset_info_menuitem->set_sensitive(true);
       explosion_picture_menuitem->set_sensitive(true);
@@ -285,6 +330,7 @@ TileSetWindow::update_tileset_menuitems()
       bridges_picture_menuitem->set_sensitive(true);
       fog_picture_menuitem->set_sensitive(true);
       flags_picture_menuitem->set_sensitive(true);
+      organize_tilestyles_menuitem->set_sensitive(true);
     }
 }
 
@@ -332,85 +378,58 @@ TileSetWindow::update_tilestyle_panel()
   TileStyle *t = get_selected_tilestyle ();
   if (t)
     {
-      tilestyle_combobox->set_active(t->getType());
+      int idx = t->getType();
+      tilestyle_combobox->set_active(idx);
       Glib::RefPtr<Gdk::Pixbuf> pixbuf= t->getImage()->to_pixbuf();
       tilestyle_image->clear();
       tilestyle_image->property_pixbuf() = pixbuf;
       tilestyle_image->show_all();
-      int idx = t->getType();
       tilestyle_standard_image->property_pixbuf() = 
-	tilestyle_standard_images[idx]->to_pixbuf();
+        GraphicsCache::getInstance()->getDefaultTileStylePic(idx, d_tileset->getTileSize())->to_pixbuf();
     }
 }
 
-int get_image_height(std::string filename)
-{
-  Glib::RefPtr<Gdk::Pixbuf> row = Gdk::Pixbuf::create_from_file(filename);
-  if (row)
-    return row->get_height();
-  else
-    return 0;
-}
-
-int get_image_width (std::string filename)
-{
-  Glib::RefPtr<Gdk::Pixbuf> row = Gdk::Pixbuf::create_from_file(filename);
-  if (row)
-    return row->get_width();
-  else
-    return 0;
-}
-
 void TileSetWindow::fill_tilestyleset_info(TileStyleSet *t)
 {
-  std::string subdir;
-  if (t->getSubDir() == "")
-    subdir = d_tileset->getSubDir();
-  else
-    subdir = t->getSubDir();
-  std::string n = d_tileset->getFile(t->getName());
-  int height = get_image_height (n);
-  if (height)
+  if (!t || t->getName() == "")
     {
-      d_tileset->setTileSize(height);
-      TileStyleSet *set = get_selected_tilestyleset ();
-      set->instantiateImages(d_tileset->getTileSize(), 
-			     d_tileset->getFile(set->getName()));
+      tilestyles_list->clear();
+      image_button->set_label(_("no image set"));
+      update_tilestyle_panel();
+      return;
     }
-  inhibit_image_change = true;
-  image_filechooser_button->set_filename(n);
-  refresh_button->set_sensitive(true);
+
+  std::string n = d_tileset->getFileFromConfigurationFile(t->getName() + ".png");
+  TileStyleSet *set = get_selected_tilestyleset ();
+  set->instantiateImages(d_tileset->getTileSize(), n);
+  File::erase(n);
+  if (t->getName().empty() == false)
+    image_button->set_label(File::get_basename(t->getName() + ".png", true));
+  else
+    image_button->set_label(_("no image set"));
   //add the tilestyles to the tilestyles_treeview
   tilestyles_list->clear();
   for (unsigned int i = 0; i < t->size(); i++)
     {
       Gtk::TreeIter l = tilestyles_list->append();
       (*l)[tilestyles_columns.name] = 
-	String::ucompose("0x%1", Glib::ustring::format(std::hex, std::setfill(L'0'), std::setw(2), (*t)[i]->getId()));
+        "0x" + TileStyle::idToString((*t)[i]->getId());
       (*l)[tilestyles_columns.tilestyle] = (*t)[i];
     }
+  tilestyles_treeview->set_cursor (Gtk::TreePath ("0"));
 }
 
 void
 TileSetWindow::update_tilestyleset_panel()
 {
-  if (tilestylesets_treeview->get_selection()->get_selected() == 0)
-    {
-      //clear all values
-      tilestyleset_frame->set_sensitive(false);
-      if (d_tileset)
-	image_filechooser_button->set_current_folder(d_tileset->getDirectory());
-      else
-	image_filechooser_button->set_current_folder(File::getUserTilesetDir());
-      tilestyles_list->clear();
-      refresh_button->set_sensitive(false);
-      return;
-    }
   tilestyleset_frame->set_sensitive(true);
   TileStyleSet *t = get_selected_tilestyleset ();
-  if (t && t->getName() != "")
+  if (t)
+    fill_tilestyleset_info(t);
+  else
     {
-      fill_tilestyleset_info(t);
+      fill_tilestyleset_info(NULL);
+      tilestyleset_frame->set_sensitive(false);
     }
 }
 
@@ -420,9 +439,12 @@ TileSetWindow::update_tile_panel()
   Gdk::Color black("black");
   //if nothing selected in the treeview, then we don't show anything in
   //the tile panel
-  if (tiles_treeview->get_selection()->get_selected() == 0)
+  Tile *t = get_selected_tile ();
+      
+  if (t == NULL)
     {
       //clear all values
+      inhibit_needs_saving = true;
       tile_smallmap_image->clear();
       tile_vbox->set_sensitive(false);
       tile_type_combobox->set_active(0);
@@ -433,19 +455,15 @@ TileSetWindow::update_tile_panel()
       tile_smallmap_second_colorbutton->set_color(black);
       tile_smallmap_third_colorbutton->set_color(black);
       tilestylesets_list->clear();
+      inhibit_needs_saving = false;
       return;
     }
-  tile_vbox->set_sensitive(true);
-  Tile *t = get_selected_tile ();
 
-  if (t)
-    {
-      tile_smallmap_first_colorbutton->set_color(black);
-      tile_smallmap_second_colorbutton->set_color(black);
-      tile_smallmap_third_colorbutton->set_color(black);
-      fill_tile_info(t);
-    }
+  tile_vbox->set_sensitive(true);
+  fill_tile_info(t);
+  fill_tilestylesets();
 }
+
 TileSetWindow::~TileSetWindow()
 {
   tile_smallmap_surface.reset();
@@ -469,13 +487,13 @@ bool TileSetWindow::on_delete_event(GdkEventAny *e)
   return true;
 }
 
-
 void TileSetWindow::on_new_tileset_activated()
 {
   std::string name = "";
   int id = Tilesetlist::getNextAvailableId(0);
   Tileset *tileset = new Tileset(id, name);
-  TileSetInfoDialog d(tileset, false);
+  TileSetInfoDialog d(tileset, File::getUserTilesetDir(), "", false,
+                      _("Make a New Tileset"));
   d.set_parent_window(*window);
   int response = d.run();
   if (response != Gtk::RESPONSE_ACCEPT)
@@ -486,12 +504,15 @@ void TileSetWindow::on_new_tileset_activated()
   if (d_tileset)
     delete d_tileset;
   d_tileset = tileset;
-  tiles_list->clear();
-
-  std::string dir = File::getUserTilesetDir() + d_tileset->getSubDir();
+  std::string dir = File::getUserTilesetDir();
   d_tileset->setDirectory(dir);
-  File::create_dir(dir);
   current_save_filename = d_tileset->getConfigurationFile();
+  RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
+  refl->updateEntry(current_save_filename);
+  refl->save();
+  d_tileset->setDirectory(File::get_dirname(autosave));
+  d_tileset->setBaseName(File::get_basename(autosave));
+  d_tileset->save(autosave, Tileset::file_extension);
 
   //copy images??
   update_tileset_buttons();
@@ -499,10 +520,8 @@ void TileSetWindow::on_new_tileset_activated()
   update_tileset_menuitems();
   update_tile_preview_menuitem();
 
-  XML_Helper helper(current_save_filename, std::ios::out, false);
-  d_tileset->save(&helper);
-  helper.close();
   needs_saving = true;
+  update_window_title();
 }
 
 void TileSetWindow::on_load_tileset_activated()
@@ -517,7 +536,7 @@ void TileSetWindow::on_load_tileset_activated()
   chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
   chooser.set_default_response(Gtk::RESPONSE_ACCEPT);
-      
+
   chooser.show_all();
   int res = chooser.run();
 
@@ -525,39 +544,101 @@ void TileSetWindow::on_load_tileset_activated()
     {
       load_tileset(chooser.get_filename());
       chooser.hide();
+      needs_saving = false;
+      update_window_title();
     }
+  update_tile_panel();
+}
+
+void TileSetWindow::on_save_as_activated()
+{
+  std::string orig_basename = d_tileset->getBaseName();
+  guint32 orig_id = d_tileset->getId();
+  d_tileset->setId(Tilesetlist::getNextAvailableId(orig_id));
+  TileSetInfoDialog d(d_tileset, File::getUserTilesetDir(), "", false,
+                        _("Save a Copy of a Tileset"));
+  d.set_parent_window(*window);
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    {
+      std::string new_basename = d_tileset->getBaseName();
+      guint32 new_id = d_tileset->getId();
+      d_tileset->setId(orig_id);
+      d_tileset->setBaseName(orig_basename);
+      bool success = Tilesetlist::getInstance()->addToPersonalCollection(d_tileset, new_basename, new_id);
+      if (success)
+        {
+          save_tileset_menuitem->set_sensitive(true);
+          current_save_filename = d_tileset->getConfigurationFile();
+          RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
+          refl->updateEntry(current_save_filename);
+          refl->save();
+          load_tileset(current_save_filename);
+          needs_saving = false;
+          update_window_title();
+        }
+      else
+        {
+          std::string msg;
+          msg = _("Error!  Tileset could not be saved.");
+          Gtk::MessageDialog dialog(*window, msg);
+          dialog.run();
+          dialog.hide();
+        }
+    }
+  else
+    d_tileset->setId(orig_id);
 }
 
 void TileSetWindow::on_save_tileset_activated()
 {
   if (current_save_filename.empty())
     current_save_filename = d_tileset->getConfigurationFile();
-  needs_saving = false;
-  
+
+  guint32 suggested_tile_size = d_tileset->calculate_preferred_tile_size();
+  if (suggested_tile_size != d_tileset->getTileSize())
+    {
+      TileSizeEditorDialog d(d_tileset->getTileSize(), suggested_tile_size);
+      int response = d.run();
+      if (response == Gtk::RESPONSE_ACCEPT)
+        d_tileset->setTileSize(d.get_selected_tilesize());
+    }
   //Reorder the tileset according to the treeview
   d_tileset->clear();
   for (Gtk::TreeIter i = tiles_list->children().begin(),
        end = tiles_list->children().end(); i != end; ++i) 
     d_tileset->push_back((*i)[tiles_columns.tile]);
-
-  XML_Helper helper(current_save_filename, std::ios::out, false);
-  d_tileset->save(&helper);
-  helper.close();
+  bool success = d_tileset->save(autosave, Tileset::file_extension);
+  if (success)
+    {
+      RecentlyEditedFileList::getInstance()->updateEntry(current_save_filename);
+      RecentlyEditedFileList::getInstance()->save();
+      success = Tileset::copy (autosave, current_save_filename);
+      if (success == true)
+        {
+          needs_saving = false;
+          update_window_title();
+          tileset_saved.emit(d_tileset->getId());
+        }
+    }
+  if (!success)
+    {
+      std::string msg;
+      msg = _("Error!  Tileset could not be saved.");
+      Gtk::MessageDialog dialog(*window, msg);
+      dialog.run();
+      dialog.hide();
+    }
 }
 
 bool TileSetWindow::quit()
 {
   if (needs_saving)
     {
-      Gtk::Dialog* dialog;
-      Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + 
-					 "/editor-quit-dialog.ui");
-      xml->get_widget("dialog", dialog);
-      dialog->set_transient_for(*window);
-      int response = dialog->run();
-      dialog->hide();
-      delete dialog;
+      EditorQuitDialog d;
+      int response = d.run();
+      d.set_parent_window(*window);
+      d.hide();
       
       if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
 	return false;
@@ -569,6 +650,9 @@ bool TileSetWindow::quit()
     }
   else
     window->hide();
+  File::erase(autosave);
+  if (d_tileset)
+    delete d_tileset;
   return true;
 }
 
@@ -584,11 +668,15 @@ void TileSetWindow::on_quit_activated()
 
 void TileSetWindow::on_edit_tileset_info_activated()
 {
-  TileSetInfoDialog d(d_tileset, true);
+  TileSetInfoDialog d(d_tileset, File::get_dirname(current_save_filename), 
+                      File::get_basename(current_save_filename), true);
   d.set_parent_window(*window);
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
-    needs_saving = true;
+    {
+      needs_saving = true;
+      update_window_title();
+    }
 }
 
 void TileSetWindow::on_help_about_activated()
@@ -652,13 +740,14 @@ void TileSetWindow::fill_tilestylesets()
     {
       Gtk::TreeIter l = tilestylesets_list->append();
       (*l)[tilestylesets_columns.name] = (*it)->getName();
-      (*l)[tilestylesets_columns.subdir] = (*it)->getSubDir();
       (*l)[tilestylesets_columns.tilestyleset] = *it;
     }
+  tilestylesets_treeview->set_cursor (Gtk::TreePath ("0"));
 }
 
 void TileSetWindow::fill_tile_info(Tile *tile)
 {
+  inhibit_needs_saving = true;
   tile_name_entry->set_text(tile->getName());
   tile_type_combobox->set_active(tile->getTypeIndex());
   tile_moves_spinbutton->set_value(tile->getMoves());
@@ -667,6 +756,7 @@ void TileSetWindow::fill_tile_info(Tile *tile)
   fill_tilestylesets();
   fill_colours(tile);
   fill_tile_smallmap(tile);
+  inhibit_needs_saving = false;
 }
 
 void TileSetWindow::on_add_tile_clicked()
@@ -682,6 +772,7 @@ void TileSetWindow::on_add_tile_clicked()
   tiles_treeview->set_cursor (Gtk::TreePath (String::ucompose("%1", d_tileset->size() - 1)));
   update_tile_preview_menuitem();
   needs_saving = true;
+  update_window_title();
 }
 
 void TileSetWindow::on_remove_tile_clicked()
@@ -707,6 +798,7 @@ void TileSetWindow::on_remove_tile_clicked()
 	}
   
       needs_saving = true;
+      update_window_title();
     }
   update_tile_preview_menuitem();
 }
@@ -722,7 +814,11 @@ void TileSetWindow::on_tile_first_color_changed()
       Tile *t = row[tiles_columns.tile];
       t->getSmallTile()->setColor(tile_smallmap_first_colorbutton->get_color());
       fill_tile_smallmap(t);
-      needs_saving = true;
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -737,7 +833,11 @@ void TileSetWindow::on_tile_second_color_changed()
       Tile *t = row[tiles_columns.tile];
       t->getSmallTile()->setSecondColor(tile_smallmap_second_colorbutton->get_color());
       fill_tile_smallmap(t);
-      needs_saving = true;
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -752,7 +852,11 @@ void TileSetWindow::on_tile_third_color_changed()
       Tile *t = row[tiles_columns.tile];
       t->getSmallTile()->setThirdColor(tile_smallmap_third_colorbutton->get_color());
       fill_tile_smallmap(t);
-      needs_saving = true;
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -796,7 +900,11 @@ void TileSetWindow::on_tile_pattern_changed()
       t->getSmallTile()->setPattern(pattern);
       fill_colours(t);
       fill_tile_smallmap(t);
-      needs_saving = true;
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -811,7 +919,11 @@ void TileSetWindow::on_tile_type_changed()
       Gtk::TreeModel::Row row = *iterrow;
       Tile *t = row[tiles_columns.tile];
       t->setTypeByIndex(idx);
-      needs_saving = true;
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -826,7 +938,12 @@ void TileSetWindow::on_tile_name_changed()
       row[tiles_columns.name] = tile_name_entry->get_text();
       Tile *t = row[tiles_columns.tile];
       t->setName(tile_name_entry->get_text());
-      needs_saving = true;
+
+      if (inhibit_needs_saving == false)
+        {
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 
@@ -854,10 +971,9 @@ void TileSetWindow::fill_tile_smallmap(Tile *tile)
   tile_smallmap_image->property_pixmap() = tile_smallmap_surface;
 }
 
-void TileSetWindow::on_add_tilestyleset_clicked()
+void TileSetWindow::choose_and_add_or_replace_tilestyleset(std::string replace_filename)
 {
-
-  Gtk::FileChooserDialog chooser(*window, _("Choose a Name"),
+  Gtk::FileChooserDialog chooser(*window, _("Choose an Image"),
 				 Gtk::FILE_CHOOSER_ACTION_OPEN);
   Gtk::FileFilter sav_filter;
   sav_filter.add_pattern("*.png");
@@ -871,24 +987,46 @@ void TileSetWindow::on_add_tilestyleset_clicked()
   chooser.show_all();
   int res = chooser.run();
 
-  if (res == Gtk::RESPONSE_ACCEPT)
-    {
-      std::string filename = chooser.get_filename();
-      chooser.hide();
-      //add a new empty tile to the tileset
-      TileStyleSet *t = new TileStyleSet();
-      //add it to the treeview
-      Gtk::TreeIter i = tilestylesets_list->append();
-      t->setName("");
-      (*i)[tilestylesets_columns.name] = t->getName();
-      (*i)[tilestylesets_columns.tilestyleset] = t;
+  if (res != Gtk::RESPONSE_ACCEPT)
+    return;
 
-      Tile *tile = get_selected_tile ();
-      tile->push_back(t);
-      tilestylesets_treeview->set_cursor (Gtk::TreePath (String::ucompose("%1", tile->size() - 1)));
-      image_filechooser_button->set_filename(filename);
-      needs_saving = true;
-    }
+  std::string selected_filename = chooser.get_filename();
+  if (selected_filename.empty())
+    return;
+
+  if (TileStyleSet::validate_image(selected_filename) == false)
+    return;
+
+  if (replace_filename.empty() == false)
+    on_remove_tilestyleset_clicked();
+  
+  Tile *tile = get_selected_tile();
+  bool success = d_tileset->addTileStyleSet(tile, selected_filename);
+  if (!success)
+    return;
+
+  if (replace_filename.empty() == true)
+    d_tileset->replaceFileInConfigurationFile(replace_filename, 
+                                              selected_filename);
+  else
+    d_tileset->addFileInConfigurationFile(selected_filename);
+
+  //now make a new one
+  TileStyleSet *set = tile->back();
+  Gtk::TreeIter i = tilestylesets_list->append();
+  (*i)[tilestylesets_columns.name] = set->getName();
+  (*i)[tilestylesets_columns.tilestyleset] = set;
+
+  tilestylesets_treeview->set_cursor (Gtk::TreePath (String::ucompose("%1", tile->size() - 1)));
+
+  needs_saving = true;
+  update_window_title();
+}
+
+void TileSetWindow::on_add_tilestyleset_clicked()
+{
+  choose_and_add_or_replace_tilestyleset("");
+  return;
 }
 
 TileStyle * TileSetWindow::get_selected_tilestyle ()
@@ -953,6 +1091,8 @@ void TileSetWindow::on_remove_tilestyleset_clicked()
 	    }
 	}
       needs_saving = true;
+      fill_tilestyleset_info(NULL);
+      update_window_title();
     }
 }
 
@@ -964,69 +1104,96 @@ void TileSetWindow::on_tilestyle_changed()
       t->setType(TileStyle::Type(tilestyle_combobox->get_active_row_number()));
       int idx = t->getType();
       tilestyle_standard_image->property_pixbuf() = 
-	tilestyle_standard_images[idx]->to_pixbuf();
+        GraphicsCache::getInstance()->getDefaultTileStylePic(idx, d_tileset->getTileSize())->to_pixbuf();
     }
 }
 
 void TileSetWindow::on_image_chosen()
 {
-  std::string selected_filename = image_filechooser_button->get_filename();
-  if (selected_filename.empty())
-    return;
-  if (inhibit_image_change == true)
-    {
-      inhibit_image_change = false;
-      return;
-    }
+  TileStyleSet *set = get_selected_tilestyleset();
 
-  unsigned int height = get_image_height (selected_filename);
-  unsigned int width = get_image_width (selected_filename);
-  if ((width % height) != 0)
-    return;
+  choose_and_add_or_replace_tilestyleset(set->getName() + ".png");
 
-  std::string file = File::get_basename(selected_filename);
-      
-  if (selected_filename != d_tileset->getFile(file))
-    File::copy (selected_filename, d_tileset->getFile(file));
-
-  d_tileset->setTileSize(height);
-
-  on_remove_tilestyleset_clicked();
-
-  //now make a new one
-  TileStyleSet *set = new TileStyleSet();
-  Gtk::TreeIter i = tilestylesets_list->append();
-
-  set->setSubDir(d_tileset->getSubDir());
-  (*i)[tilestylesets_columns.subdir] = set->getSubDir();
-
-
-  set->setName(File::get_basename(selected_filename));
-  (*i)[tilestylesets_columns.name] = set->getName();
-  (*i)[tilestylesets_columns.tilestyleset] = set;
-  Tile *tile = get_selected_tile ();
-  tile->push_back(set);
-
-  int no = width / height;
-  for (int j = 0; j < no; j++)
-    {
-      int id = d_tileset->getFreeTileStyleId();
-      if (id >= 0)
-	{
-	  TileStyle *style = new TileStyle();
-	  style->setId(id);
-	  style->setType(TileStyle::LONE);
-	  set->push_back(style);
-	}
-    }
-      
-  needs_saving = true;
 }
 
-void TileSetWindow::on_refresh_clicked()
+void TileSetWindow::on_organize_tilestyles_activated()
 {
-  TileStyleSet *set = get_selected_tilestyleset ();
-  set->instantiateImages(d_tileset->getTileSize(), d_tileset->getFile(set->getName()));
+  TileStyleOrganizerDialog d(get_selected_tile());
+  d.tilestyle_selected.connect
+    (sigc::mem_fun(this, &TileSetWindow::on_tilestyle_id_selected));
+  d.set_parent_window(*window);
+  d.run();
+}
+
+void TileSetWindow::on_smallmap_building_colors_activated()
+{
+  TilesetSmallmapBuildingColorsDialog d(d_tileset);
+  d.set_parent_window(*window);
+  d.run();
+}
+    
+void TileSetWindow::on_tilestyle_id_selected(guint32 id)
+{
+  Tile *t = NULL;
+  TileStyleSet *s = NULL;
+  TileStyle *style = NULL;
+  bool found = d_tileset->getTileStyle(id, &t, &s, &style);
+  if (found)
+    {
+      select_tile(t);
+      select_tilestyleset(s);
+      select_tilestyle(style);
+    }
+}
+
+void TileSetWindow::select_tile(Tile *tile)
+{
+  for (guint32 i = 0; i < tiles_list->children().size(); i++)
+    {
+      Gtk::TreeIter iter = tiles_list->children()[i];
+      Gtk::TreeModel::Row row = *iter;
+      Tile *t = row[tiles_columns.tile];
+      if (tile == t)
+        {
+          Glib::RefPtr<Gtk::TreeSelection> s = tiles_treeview->get_selection();
+          s->select(row);
+          return;
+        }
+    }
+}
+
+void TileSetWindow::select_tilestyleset(TileStyleSet *set)
+{
+  for (guint32 i = 0; i < tilestylesets_list->children().size(); i++)
+    {
+      Gtk::TreeIter iter = tilestylesets_list->children()[i];
+      Gtk::TreeModel::Row row = *iter;
+      TileStyleSet *t = row[tilestylesets_columns.tilestyleset];
+      if (set == t)
+        {
+          Glib::RefPtr<Gtk::TreeSelection> s = 
+            tilestylesets_treeview->get_selection();
+          s->select(row);
+          return;
+        }
+    }
+}
+
+void TileSetWindow::select_tilestyle(TileStyle *style)
+{
+  for (guint32 i = 0; i < tilestyles_list->children().size(); i++)
+    {
+      Gtk::TreeIter iter = tilestyles_list->children()[i];
+      Gtk::TreeModel::Row row = *iter;
+      TileStyle *t = row[tilestyles_columns.tilestyle];
+      if (style == t)
+        {
+          Glib::RefPtr<Gtk::TreeSelection> s = 
+            tilestyles_treeview->get_selection();
+          s->select(row);
+          return;
+        }
+    }
 }
 
 void TileSetWindow::on_preview_tile_activated()
@@ -1044,6 +1211,8 @@ void TileSetWindow::on_preview_tile_activated()
       if (idx > -1)
         sec = (*d_tileset)[idx];
       TilePreviewDialog d(tile, sec, d_tileset->getTileSize());
+      d.tilestyle_selected.connect
+        (sigc::mem_fun(this, &TileSetWindow::on_tilestyle_id_selected));
       d.set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
       d.run();
     }
@@ -1051,52 +1220,78 @@ void TileSetWindow::on_preview_tile_activated()
       
 void TileSetWindow::on_roads_picture_activated()
 {
-  ImageEditorDialog d(d_tileset->getFile(d_tileset->getRoadsFilename()),
-		      ROAD_TYPES);
+  std::string filename = "";
+  if (d_tileset->getRoadsFilename().empty() == false)
+    filename = d_tileset->getFileFromConfigurationFile(d_tileset->getRoadsFilename() +".png");
+  ImageEditorDialog d(filename, ROAD_TYPES);
   d.set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
   d.set_parent_window(*window);
   int response = d.run();
+  if (filename.empty() == false)
+    File::erase(filename);
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string filename = d.get_selected_filename();
-      std::string name = File::get_basename(filename);
-      File::copy(filename, d_tileset->getFile(name));
-      d_tileset->setRoadsFilename(name);
-      needs_saving = true;
+      if (d.get_selected_filename() != filename)
+        {
+          d_tileset->replaceFileInConfigurationFile(d_tileset->getRoadsFilename()+".png", d.get_selected_filename());
+
+          d_tileset->setRoadsFilename
+            (File::get_basename(d.get_selected_filename()));
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
+
 void TileSetWindow::on_bridges_picture_activated()
 {
-  ImageEditorDialog d(d_tileset->getFile(d_tileset->getBridgesFilename()),
-		      BRIDGE_TYPES);
+  std::string filename = "";
+  if (d_tileset->getBridgesFilename().empty() == false)
+    filename = d_tileset->getFileFromConfigurationFile(d_tileset->getBridgesFilename() +".png");
+  ImageEditorDialog d(filename, BRIDGE_TYPES);
   d.set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
   d.set_parent_window(*window);
   int response = d.run();
+  if (filename.empty() == false)
+    File::erase(filename);
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string filename = d.get_selected_filename();
-      std::string name = File::get_basename(filename);
-      File::copy(filename, d_tileset->getFile(name));
-      d_tileset->setBridgesFilename(name);
-      needs_saving = true;
+      if (d.get_selected_filename() != filename)
+        {
+          d_tileset->replaceFileInConfigurationFile(d_tileset->getBridgesFilename()+".png", d.get_selected_filename());
+
+          d_tileset->setBridgesFilename
+            (File::get_basename(d.get_selected_filename()));
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
 void TileSetWindow::on_fog_picture_activated()
 {
-  ImageEditorDialog d(d_tileset->getFile(d_tileset->getFogFilename()),
-		      FOG_TYPES);
+  std::string filename = "";
+  if (d_tileset->getFogFilename().empty() == false)
+    filename = d_tileset->getFileFromConfigurationFile(d_tileset->getFogFilename() +".png");
+  ImageEditorDialog d(filename, FOG_TYPES);
   d.set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
   d.set_parent_window(*window);
   int response = d.run();
+  if (filename.empty() == false)
+    File::erase(filename);
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string filename = d.get_selected_filename();
-      std::string name = File::get_basename(filename);
-      File::copy(filename, d_tileset->getFile(name));
-      d_tileset->setFogFilename(name);
-      needs_saving = true;
+      if (d.get_selected_filename() != filename)
+        {
+          d_tileset->replaceFileInConfigurationFile(d_tileset->getFogFilename()+".png", d.get_selected_filename());
+
+          d_tileset->setFogFilename
+            (File::get_basename(d.get_selected_filename()));
+          needs_saving = true;
+          update_window_title();
+        }
     }
 }
+
 void TileSetWindow::on_flags_picture_activated()
 {
   TilesetFlagEditorDialog d(d_tileset);
@@ -1105,17 +1300,11 @@ void TileSetWindow::on_flags_picture_activated()
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string filename= d.get_selected_filename();
-
-      //copy the file, and set the image name
-      std::string name = File::get_basename(filename);
-      if (filename != d_tileset->getFile(name))
-	File::copy(filename, d_tileset->getFile(name));
-
-      d_tileset->setFlagsFilename(name);
       needs_saving = true;
+      update_window_title();
     }
 }
+
 void TileSetWindow::on_army_unit_selector_activated()
 {
   TilesetSelectorEditorDialog d(d_tileset);
@@ -1123,21 +1312,8 @@ void TileSetWindow::on_army_unit_selector_activated()
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string small = d.get_small_selector_filename();
-      std::string large = d.get_large_selector_filename();
-
-      //copy the files, and set the image names
-      std::string name = File::get_basename(small);
-      if (small != d_tileset->getFile(name))
-	File::copy(small, d_tileset->getFile(name));
-      d_tileset->setSmallSelectorFilename(name);
-
-      name = File::get_basename(large);
-      if (large != d_tileset->getFile(name))
-	File::copy(large, d_tileset->getFile(name));
-      d_tileset->setLargeSelectorFilename(name);
-
       needs_saving = true;
+      update_window_title();
     }
 }
 
@@ -1148,38 +1324,55 @@ void TileSetWindow::on_explosion_picture_activated()
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
-      std::string filename = d.get_selected_filename();
-      std::string name = File::get_basename(filename);
-      std::string dest =d_tileset->getFile(name);
-      if (filename != d_tileset->getFile(name))
-	{
-	  File::copy (filename, d_tileset->getFile(name));
-	  d_tileset->setExplosionFilename(name);
-	}
       needs_saving = true;
+      update_window_title();
     }
 }
 
-void TileSetWindow::load_tileset(std::string filename)
+bool TileSetWindow::load_tileset(std::string filename)
 {
+  std::string old_current_save_filename = current_save_filename;
   current_save_filename = filename;
+  if (filename != autosave)
+    Tileset::copy(current_save_filename, autosave);
+  else
+    {
+      //go get the real name of the file
+      std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Tileset::file_extension);
+      if (files.size() > 0)
+        current_save_filename = files.front()->getFileName();
+    }
 
-  Tileset *tileset = Tileset::create(filename);
+  bool unsupported_version = false;
+  Tileset *tileset = Tileset::create(autosave, unsupported_version);
   if (tileset == NULL)
     {
       std::string msg;
-      msg = "The tileset could not be loaded.";
+      if (unsupported_version)
+        msg = _("Error!  The version of the tileset is unsupported.");
+      else
+        msg = _("Error!  Tileset could not be loaded.");
       Gtk::MessageDialog dialog(*window, msg);
+      current_save_filename = old_current_save_filename;
       dialog.run();
       dialog.hide();
+      return false;
+    }
+  if (File::nameEndsWith(current_save_filename, "/autosave" + Tileset::file_extension) == false)
+    {
+      RecentlyEditedFileList::getInstance()->addEntry(current_save_filename);
+      RecentlyEditedFileList::getInstance()->save();
     }
-  tiles_list->clear();
   if (d_tileset)
     delete d_tileset;
   d_tileset = tileset;
 
-  d_tileset->setSubDir(File::get_basename(filename));
+  d_tileset->setBaseName(File::get_basename(autosave));
   d_tileset->instantiateImages();
+
+  tilestyles_list->clear();
+  tilestylesets_list->clear();
+  tiles_list->clear();
   for (Tileset::iterator i = d_tileset->begin(); i != d_tileset->end(); ++i)
     {
       Gtk::TreeIter l = tiles_list->append();
@@ -1196,4 +1389,17 @@ void TileSetWindow::load_tileset(std::string filename)
   update_tilestyleset_panel();
   update_tilestyle_panel();
   update_tile_preview_menuitem();
+  update_window_title();
+  return true;
+}
+
+void TileSetWindow::update_window_title()
+{
+  std::string title = "";
+  if (needs_saving)
+    title += "*";
+  title += File::get_basename(current_save_filename, true);
+  title += " - ";
+  title += _("LordsAWar! Tileset Editor");
+  window->set_title(title);
 }
