@@ -97,10 +97,9 @@ Fight::Fight(Stack* attacker, Stack* defender, FightType type)
     //
 
     City *city = GameMap::getCity(defender->getPos());
-    Vector<int> p = defender->getPos();
+    //Vector<int> p = defender->getPos();
 
-
-    if (city)
+    if (city && city->isBurnt() == false)
       {
 	std::vector<Stack*> stacks = city->getDefenders();
 	for (std::vector<Stack*>::iterator it = stacks.begin(); 
