@@ -1,4 +1,4 @@
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -118,6 +118,20 @@ void Game::addPlayer(Player *p)
       connections[p->getId()].push_back
 	(p->getStacklist()->sgrouped.connect
 	 (sigc::mem_fun(this, &Game::on_stack_grouped)));
+      connections[p->getId()].push_back
+	(p->stole_gold.connect
+	 (sigc::mem_fun(stole_gold, &sigc::signal<void, Player*, 
+                        guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->sunk_ships.connect
+	 (sigc::mem_fun(sunk_ships, &sigc::signal<void, Player*, 
+                        guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->bags_picked_up.connect
+	 (sigc::mem_fun(bags_picked_up, &sigc::signal<void, Hero*, guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->mp_added_to_hero_stack.connect
+	 (sigc::mem_fun(mp_added_to_hero_stack, &sigc::signal<void, Hero*, guint32>::emit)));
     }
       
       
@@ -164,13 +178,19 @@ void Game::addPlayer(Player *p)
   connections[p->getId()].push_back
     (p->fight_started.connect (sigc::mem_fun(*this, &Game::on_fight_started)));
   connections[p->getId()].push_back
-    (p->ruinfight_started.connect
-     (sigc::mem_fun
-      (ruinfight_started, &sigc::signal<void, Stack *, Stack *>::emit)));
+    (p->using_item.connect (sigc::mem_fun(*this, &Game::on_use_item)));
   connections[p->getId()].push_back
-    (p->ruinfight_finished.connect
-     (sigc::mem_fun
-      (ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
+    (p->ruinfight_started.connect (sigc::mem_fun(*this, &Game::on_ruinfight_started)));
+
+    //(p->ruinfight_started.connect
+     //(sigc::mem_fun
+      //(ruinfight_started, &sigc::signal<void, Stack *, Stack *>::emit)));
+  connections[p->getId()].push_back
+    (p->ruinfight_finished.connect (sigc::mem_fun(*this, &Game::on_ruinfight_finished)));
+  //connections[p->getId()].push_back
+    //(p->ruinfight_finished.connect
+     //(sigc::mem_fun
+      //(ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
   connections[p->getId()].push_back
     (p->cityfight_finished.connect (sigc::mem_fun(*this, &Game::on_city_fight_finished))); 
   if (p->getType() == Player::NETWORKED && p == Playerlist::getActiveplayer())
@@ -382,6 +402,10 @@ void Game::move_map_dir(int diffx, int diffy)
 
 void Game::move_selected_stack_dir(int diffx, int diffy)
 {
+  static bool currently_moving = false;
+  if (currently_moving == true)
+    return;
+  currently_moving = true;
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
   if (!stack)
     return;
@@ -399,6 +423,7 @@ void Game::move_selected_stack_dir(int diffx, int diffy)
       Playerlist::getActiveplayer()->setActivestack(0);
       unselect_active_stack();
     }
+  currently_moving = false;
 }
 
 void Game::move_selected_stack_northwest()
@@ -553,7 +578,7 @@ void Game::center_selected_stack()
     select_active_stack();
 }
 
-void Game::search_stack(Stack *stack)
+void Game::search_stack(Stack *stack, bool &gotquest, bool &stackdied)
 {
   Player *player = Playerlist::getActiveplayer();
   Ruin* ruin = GameMap::getRuin(stack);
@@ -565,8 +590,9 @@ void Game::search_stack(Stack *stack)
        ruin->isHidden() == false))
     {
       Reward *reward;
-
-      reward = player->stackSearchRuin(stack, ruin);
+      reward = player->stackSearchRuin(stack, ruin, stackdied);
+      if (stackdied)
+        return;
       if (ruin->hasSage() == true)
 	{
           Sage *sage = ruin->generateSage();
@@ -613,6 +639,7 @@ void Game::search_stack(Stack *stack)
 
 	  if (q)
             {
+              gotquest = true;
               if (player->isComputer() == false)
                 {
                   Hero *hero;
@@ -626,11 +653,46 @@ void Game::search_stack(Stack *stack)
     }
 }
 
+void Game::select_item_to_use()
+{
+  Player *active = Playerlist::getActiveplayer();
+  //emit a signal that makes a dialog appear that lets us pick an item to use.
+  std::list<Item*> items = active->getUsableItems();
+  if (items.size() == 0)
+    return;
+  Item *item = select_item.emit(items);
+  if (item != NULL)
+    on_use_item(item);
+}
+
+void Game::on_use_item(Item *item)
+{
+  Player *active = Playerlist::getActiveplayer();
+  Stack *stack = NULL;
+  Hero *hero = NULL;
+  active->getItemHolder(item, &stack, &hero);
+  Player *victim = NULL;
+
+  //ask the user a series of questions on how to use the item
+  if (item->usableOnVictimPlayer())
+    victim = select_item_victim_player.emit();
+          
+  // now we act on the answers and execute the usage of the item
+  if (item->usableOnVictimPlayer() == true && victim)
+    active->heroUseItem(hero, item, victim);
+
+  if (item->usableOnVictimPlayer() == false)
+    active->heroUseItem(hero, item, NULL);
+}
+
 void Game::search_selected_stack()
 {
   Player *player = Playerlist::getActiveplayer();
   Stack* stack = player->getActivestack();
-  return search_stack(stack);
+  bool stack_died = false;
+  bool got_quest = false;
+  search_stack(stack, got_quest, stack_died);
+  return;
 }
 
 void Game::stackUpdate(Stack* s)
@@ -877,6 +939,7 @@ void Game::update_control_panel()
       can_park_selected_stack.emit(false);
       can_deselect_selected_stack.emit(false);
       can_search_selected_stack.emit(false);
+      can_use_item.emit(false);
       can_plant_standard_selected_stack.emit(false);
       can_move_selected_stack.emit(false);
       can_move_selected_stack_along_path.emit(false);
@@ -929,60 +992,14 @@ void Game::update_control_panel()
 	((!stack->getPath()->empty() && stack->enoughMoves()) ||
 	 (!stack->getPath()->empty() && stack->getPath()->getMovesExhaustedAtPoint() > 0));
 
-      /*
-       * a note about searching.
-       * ruins can be searched by stacks that have a hero, and when the
-       * hero has moves left.  also the ruin must be unexplored.
-       * temples can be searched by any stack, when the stack has 
-       * movement left.
-       */
       if (stack->getMoves() > 0)
-	{
-	  Temple *temple;
-	  temple = GameMap::getTemple(stack);
-	  can_search_selected_stack.emit(temple);
-	  can_move_selected_stack.emit(true);
-	}
+        can_move_selected_stack.emit(true);
 
-      if (stack->hasHero())
-	{
-	  Ruin *ruin = GameMap::getRuin(stack);
-	  if (stack->getFirstHero()->getMoves() > 0 && ruin)
-	    can_search_selected_stack.emit(!ruin->isSearched());
+      can_plant_standard_selected_stack.emit(GameMap::can_plant_flag(stack));
 
-	  //does the hero have the player's standard?
-	  for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-	    {
-	      if ((*it)->isHero())
-		{
-		  Hero *hero = dynamic_cast<Hero*>((*it));
-		  if (hero->getBackpack()->getPlantableItem(player))
-		    {
-			  //can't plant on city/ruin/temple/signpost
-			  City *city = GameMap::getCity(stack->getPos());
-			  Temple *temple = GameMap::getTemple(stack);
-			  Ruin *ruin = GameMap::getRuin(stack);
-			  Signpost *sign = GameMap::getSignpost(stack);
-			  if (!city && !temple && !ruin && !sign)
-			    {
-			      GameMap *gm = GameMap::getInstance();
-			      MapBackpack *backpack;
-			      Vector<int> pos = stack->getPos();
-			      backpack = gm->getTile(pos)->getBackpack();
-			      bool standard_already_planted = 
-				backpack->getFirstPlantedItem() != NULL;
-			      //are there any other standards here?
-			      if (standard_already_planted == false)
-				can_plant_standard_selected_stack.emit(true);
-			    }
-		    }
-		}
-	    }
-	}
-      else
-	{
-	  can_plant_standard_selected_stack.emit(false);
-	}
+      can_search_selected_stack.emit(GameMap::can_search(stack));
+          
+      can_use_item.emit(player->hasUsableItem());
 
       if (GameMap::getSignpost(stack))
 	can_change_signpost.emit(true);
@@ -992,7 +1009,6 @@ void Game::update_control_panel()
     }
   else
     {
-      can_search_selected_stack.emit(false);
       can_move_selected_stack.emit(false);
       can_move_selected_stack_along_path.emit(false);
       can_disband_stack.emit(false);
@@ -1452,12 +1468,30 @@ void Game::stack_leaves_tile(Stack *stack, Vector<int> tile)
     }
 }
     
-void Game::stack_searches_ruin(Ruin *ruin, Stack *stack)
+bool Game::stack_searches_ruin(Ruin *ruin, Stack *stack)
 {
-  search_stack(stack);
+  bool stack_died = false;
+  bool hero_got_quest = false;
+  search_stack(stack, hero_got_quest, stack_died);
+  return stack_died;
 }
     
-void Game::stack_searches_temple(Temple *temple, Stack *stack)
+bool Game::stack_searches_temple(Temple *temple, Stack *stack)
 {
-  search_stack(stack);
+  bool stack_died = false;
+  bool hero_got_quest = false;
+  search_stack(stack, hero_got_quest, stack_died);
+  return hero_got_quest;
+}
+
+void Game::on_ruinfight_started(Stack *attacker, Stack *defender)
+{
+  if (Playerlist::getActiveplayer()->isComputer() == false)
+    ruinfight_started.emit(attacker, defender);
+}
+
+void Game::on_ruinfight_finished(Fight::Result result)
+{
+  if (Playerlist::getActiveplayer()->isComputer() == false)
+    ruinfight_finished.emit(result);
 }
