@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -46,6 +46,9 @@
 #include "gui/image-helpers.h"
 #include "PathCalculator.h"
 #include "stacktile.h"
+#include "action.h"
+#include "tileset.h"
+#include "tilesetlist.h"
 
 #include "timing.h"
 
@@ -133,6 +136,7 @@ bool GameBigMap::on_selection_timeout()
   return Timing::CONTINUE;
 }
 
+                        
 void GameBigMap::mouse_button_event(MouseButtonEvent e)
 {
   if (input_locked)
@@ -146,6 +150,11 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
   if (e.button == MouseButtonEvent::LEFT_BUTTON
       && e.state == MouseButtonEvent::PRESSED)
     {
+      city_unqueried.emit();
+      ruin_queried.emit(0, true);
+      temple_queried.emit(0, true);
+      signpost_queried.emit(0);
+      stack_unqueried.emit();
       bool double_clicked = false;
       static Glib::TimeVal last_clicked;
       if (last_clicked.as_double() == 0.0)
@@ -238,7 +247,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	  // clicked on the already active stack
 	  if (stack->getPos() == tile)
 	    {
-	      if (double_clicked == true)
+	      if (double_clicked == true && is_control_key_down() == false)
 		{
 		  StackTile *stile = GameMap::getStacks(stack->getPos());
 		  std::list<Stack *> stks= stile->getFriendlyStacks(active);
@@ -252,11 +261,29 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		    }
 		  if (path_calculator)
 		    delete path_calculator;
+		      
+                  stack = active->getActivestack();
 		  path_calculator = new PathCalculator(stack);
 		  draw(Playerlist::getViewingplayer());
 		  stack_grouped_or_ungrouped.emit(stack);
 		  return;
 		}
+              else if (double_clicked == true && is_control_key_down() == true)
+                {
+                  if (active->setPathOfStackToPreviousDestination(stack))
+                    {
+                      Rectangle old_view = view;
+                      active->stackMove(stack);
+                      if (!active->getActivestack())
+                        {
+                          unselect_active_stack();
+                          determine_mouse_cursor(NULL, tile);
+                        }
+                      set_view (old_view);
+                      view_changed.emit(view);
+                    }
+                  return;
+                }
 	      else
 		{
 		  // clear the path
@@ -424,7 +451,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      //check if we dropped on the same tile that the stack lives on.
 	      if (mouse_pos_to_tile(e.pos) != stack->getPos())
 		{
-		  int ts = GameMap::getInstance()->getTileset()->getTileSize();
+		  int ts = GameMap::getInstance()->getTileSize();
 		  e.pos = tile_to_buffer_pos (stack->getLastPointInPath());
 		  e.pos.x -= ts/2;
 		  e.pos.y -= ts/2;
@@ -545,7 +572,7 @@ void GameBigMap::zoom_in()
     return;
   if ((zoom_step / 100.0) + magnification_factor <= max_magnification_factor / 100.0)
     {
-      int ts = GameMap::getInstance()->getTileset()->getTileSize();
+      int ts = GameMap::getInstance()->getTileSize();
       Rectangle new_view;
       double mag = magnification_factor + (zoom_step / 100.0);
       new_view.w = image.get_width() / (ts * mag) + 1;
@@ -563,7 +590,7 @@ void GameBigMap::zoom_out()
     return;
   if (magnification_factor - (zoom_step / 100.0) >= min_magnification_factor / 100.0)
     {
-      int ts = GameMap::getInstance()->getTileset()->getTileSize();
+      int ts = GameMap::getInstance()->getTileSize();
       Rectangle new_view;
       double mag = magnification_factor - (zoom_step / 100.0);
       new_view.w = image.get_width() / (ts * mag) + 1;
@@ -677,7 +704,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		    d_cursor = GraphicsCache::HAND;
 		  else
 		    {
-		      if (t->getMaptileType() == Tile::WATER &&
+		      if (t->getType() == Tile::WATER &&
 			  GameMap::getBridge(tile) == NULL)
 			{
 			  if (stack->isFlying() == true)
@@ -781,7 +808,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
       if (mouse_state == NONE && length(delta) <= 2)
 	return;
 
-      int ts = GameMap::getInstance()->getTileset()->getTileSize();
+      int ts = GameMap::getInstance()->getTileSize();
       Vector<int> screen_dim(image.get_width(), image.get_height());
       view_pos = clip(Vector<int>(0, 0),
 		      view_pos + delta,
@@ -846,8 +873,9 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	  d_cursor == GraphicsCache::GOTO_ARROW || 
 	  d_cursor == GraphicsCache::TARGET)
 	{
-	  guint32 moves = 0, turns = 0;
-	  Path *new_path = path_calculator->calculate(tile, moves, turns, true);
+	  guint32 moves = 0, turns = 0, left = 0;
+	  Path *new_path = path_calculator->calculate(tile, moves, turns, left,
+                                                      true);
 	  if (new_path->size())
 	    stack->setPath(*new_path);
 	  delete new_path;
@@ -893,7 +921,7 @@ void GameBigMap::after_draw()
   if (blank_screen == true)
     return;
   GraphicsCache *gc = GraphicsCache::getInstance();
-  int tilesize = GameMap::getInstance()->getTileset()->getTileSize();
+  int tilesize = GameMap::getInstance()->getTileSize();
 
   Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 
@@ -949,7 +977,7 @@ void GameBigMap::after_draw()
 	  static int bigframe = -1;
 	  static int smallframe = -1;
 
-	  Tileset *t = GameMap::getInstance()->getTileset();
+          Tileset *t = GameMap::getTileset();
 	  bigframe++;
 	  if (bigframe >= (int)t->getNumberOfSelectorFrames())
 	    bigframe = 0;
