@@ -221,7 +221,7 @@ void Driver::run()
 	    }
 	  else
 	    {
-	      found = d_load_filename.find(".sav");
+	      found = d_load_filename.find(SAVE_EXT);
 	      if (found != std::string::npos)
 		on_load_requested(d_load_filename);
 	      else
@@ -609,6 +609,12 @@ void Driver::on_game_ended()
   splash_window->show();
 }
 
+void Driver::on_game_ended_and_start_new()
+{
+  on_game_ended();
+  splash_window->open_new_game_dialog();
+}
+
 void Driver::init_game_window()
 {
   if (game_window)
@@ -617,6 +623,8 @@ void Driver::init_game_window()
 
     game_window->game_ended.connect(
 	sigc::mem_fun(*this, &Driver::on_game_ended));
+    game_window->game_ended_start_new.connect(
+	sigc::mem_fun(*this, &Driver::on_game_ended_and_start_new));
     game_window->show_lobby.connect(
 	sigc::mem_fun(*this, &Driver::on_show_lobby_requested));
     game_window->quit_requested.connect(
@@ -652,7 +660,7 @@ GameScenario *Driver::load_game(std::string file_path)
 void Driver::on_new_pbm_game_requested(GameParameters g)
 {
   std::string filename;
-  std::string temp_filename = File::getSavePath() + "pbmtmp.sav";
+  std::string temp_filename = File::getSavePath() + "pbmtmp" + SAVE_EXT;
       
   GameScenario *game_scenario = 
     create_new_scenario(g, GameScenario::PLAY_BY_MAIL);
@@ -673,7 +681,7 @@ void Driver::on_new_pbm_game_requested(GameParameters g)
   Gtk::FileChooserDialog chooser(*splash_window->get_window(), _("Save the scenario and mail it to the first player"),
 				 Gtk::FILE_CHOOSER_ACTION_SAVE);
   Gtk::FileFilter sav_filter;
-  sav_filter.add_pattern("*.sav");
+  sav_filter.add_pattern("*" + SAVE_EXT);
   chooser.set_filter(sav_filter);
   chooser.set_current_folder(Glib::get_home_dir());
 
@@ -708,29 +716,33 @@ void Driver::on_new_pbm_game_requested(GameParameters g)
 
 void Driver::stressTestNextRound()
 {
-  //static time_t prev_round_start = time(NULL);
   static int count = 1;
-  //if (count == 1)
-    //{
-      //FILE * fileptr = fopen("/tmp/crapola.csv", "w");
-      //fclose(fileptr);
-    //}
+  /*
+  static time_t prev_round_start = time(NULL);
+  if (count == 1)
+    {
+      FILE * fileptr = fopen("/tmp/crapola.csv", "w");
+      fclose(fileptr);
+    }
+  time_t now = time(NULL);
+  */
   count++;
-  //time_t now = time(NULL);
   printf ("starting round %d!\n", count);
   /*
   FILE * fileptr = fopen("/tmp/crapola.csv", "a");
   int total_fights = Playerlist::getInstance()->countFightsThisTurn();
   int total_moves = Playerlist::getInstance()->countMovesThisTurn();
-  fprintf(fileptr, "%d, %d, %d, %d, %d, %d, %d, %d", count, 
+  fprintf(fileptr, "%d, %d, %d, %d, %d, %d, %d, %d, %d, %d", count, 
           now - prev_round_start,
           Stacklist::getNoOfStacks(), 
           Stacklist::getNoOfArmies(), 
           total_fights,
           total_moves,
+          Ruinlist::getInstance()->countUnexploredRuins(Playerlist::getInstance()->getNeutral()),
+          Playerlist::getInstance()->countBlessings(),
           Citylist::getInstance()->countCities(Playerlist::getInstance()->getNeutral()), 
           Playerlist::getInstance()->countPlayersAlive());
-  if (Playerlist::getInstance()->countPlayersAlive() == 2)
+  //if (Playerlist::getInstance()->countPlayersAlive() == 2)
     {
       Playerlist *pl = Playerlist::getInstance();
       for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
@@ -739,15 +751,15 @@ void Driver::stressTestNextRound()
             continue;
           if ((*it) == pl->getNeutral())
             continue;
-          fprintf(fileptr,", %d (%d)", Citylist::getInstance()->countCities(*it), (*it)->getHeroes().size() + (*it)->countAllies());
+          fprintf(fileptr,", %d: (%d %d)", (*it)->getId(), Citylist::getInstance()->countCities(*it), (*it)->getHeroes().size() + (*it)->countAllies());
         }
     }
   fprintf(fileptr,"\n");
   fclose(fileptr);
       
   prev_round_start = now;
-  */
   sleep (1);
+  */
 }
 
 void Driver::stress_test()
@@ -781,7 +793,7 @@ void Driver::stress_test()
   g.map.forest = 3;
   g.map.hills = 5;
   g.map.mountains = 5;
-  g.map.cities = 110;
+  g.map.cities = 20;
   g.map.ruins = 15;
   g.map.temples = 3;
   g.map.signposts = 10;
@@ -828,14 +840,6 @@ void Driver::stress_test()
       game_scenario->initialize(g);
     }
 
-  //this is a bit unfortunate... we have to instantiate images just to get stack positions into the stack tiles.  is there a better way?
-  /*/
-  GameMap::getInstance()->getTileset()->instantiateImages();
-  GameMap::getInstance()->getShieldset()->instantiateImages();
-  GameMap::getInstance()->getCityset()->instantiateImages();
-  guint32 armyset = Playerlist::getInstance()->getNeutral()->getArmyset();
-  Armysetlist::getInstance()->getArmyset(armyset)->instantiateImages();
-  */
   Game game(game_scenario, nextTurn);
   game.get_smallmap().set_slide_speed(0);
   Configuration::s_displaySpeedDelay = 0;
@@ -846,6 +850,9 @@ void Driver::stress_test()
   printf("duration: %d mins, turns: %d ", mins, game_scenario->getRound());
   fflush(stdout);
   printf("winner type: %s\n", Player::playerTypeToString(Player::Type(Playerlist::getInstance()->getFirstLiving()->getType())).c_str());
+  Glib::ustring s = String::ucompose("/tmp/run-seed-%1", 
+                                     Main::instance().random_number_seed);
+  game_scenario->saveGame(s + ".sav");
 
 }
 	
