@@ -89,6 +89,9 @@ class GameWindow: public Decorated
 
     // emitted when the game has ended and it is time to show the splash again
     sigc::signal<void> game_ended;
+
+    // emitted when the game has ended and we want to start a new game.
+    sigc::signal<void> game_ended_start_new;
     
     sigc::signal<void> show_lobby;
 
@@ -115,6 +118,7 @@ class GameWindow: public Decorated
     Gtk::MenuItem *end_turn_menuitem;
     Gtk::MenuItem *move_all_menuitem;
     Gtk::MenuItem *search_menuitem;
+    Gtk::MenuItem *use_menuitem;
     Gtk::MenuItem *disband_menuitem;
     Gtk::MenuItem *stack_info_menuitem;
     Gtk::MenuItem *signpost_menuitem;
@@ -225,6 +229,7 @@ class GameWindow: public Decorated
     void on_save_game_as_activated();
     void on_show_lobby_activated();
     void on_quit_activated();
+    void on_new_game_activated();
     void on_game_stopped();
     void on_quests_activated();
     void on_disband_activated();
@@ -322,9 +327,15 @@ class GameWindow: public Decorated
     void on_game_over(Player *winner);
     void on_player_died(Player *player);
     void on_advice_asked(float percent);
+    void on_gold_stolen(Player *victim, guint32 gold_pieces);
+    void on_ships_sunk(Player *victim, guint32 num_armies);
+    void on_bags_picked_up(Hero *hero, guint32 num_bags);
+    void on_mp_added_to_hero_stack(Hero *hero, guint32 mp);
     void on_stack_moves(Stack *stack, Vector<int> pos);
     void end_turn_play_by_mail ();
     void on_commentator_comments(std::string comment);
+    Item* on_select_item(std::list<Item*> items);
+    Player *on_select_item_victim_player();
 
     // quest manager callbacks
     void on_quest_completed(Quest *quest, Reward *reward);
