@@ -1,9 +1,9 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -18,10 +18,8 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
-#include <gtkmm/eventbox.h>
+#include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
-#include <gtkmm/label.h>
 
 #include "hero-offer-dialog.h"
 
@@ -40,26 +38,21 @@ HeroOfferDialog::HeroOfferDialog(Player *player, HeroProto *h, City *c, int gold
     city = c;
     hero = h;
     
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create(get_glade_path()
-				    + "/hero-offer-dialog.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path()
+				    + "/hero-offer-dialog.ui");
 
-    Gtk::Dialog *d = 0;
-    xml->get_widget("dialog", d);
-    dialog.reset(d);
+    xml->get_widget("dialog", dialog);
 
-    decorate(dialog.get());
-    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    decorate(dialog);
+    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
     xml->get_widget("map_image", map_image);
 
-    heromap.reset(new HeroMap(city));
+    heromap = new HeroMap(city);
     heromap->map_changed.connect(
 	sigc::mem_fun(this, &HeroOfferDialog::on_map_changed));
 
-    Gtk::EventBox *map_eventbox;
-    xml->get_widget("map_eventbox", map_eventbox);
-
     set_title(String::ucompose(_("Hero offer for %1"),
 				       player->getName()));
 
@@ -88,6 +81,11 @@ HeroOfferDialog::HeroOfferDialog(Player *player, HeroProto *h, City *c, int gold
     label->set_text(s);
 }
 
+HeroOfferDialog::~HeroOfferDialog()
+{
+  delete heromap;
+  delete dialog;
+}
 void HeroOfferDialog::on_male_toggled()
 {
     if (male_radiobutton->get_active())
@@ -119,21 +117,23 @@ bool HeroOfferDialog::run()
     int response = dialog->run();
     Sound::getInstance()->haltMusic();
 
-    if (response == 0)		// accepted
+    if (response == Gtk::RESPONSE_ACCEPT)	// accepted
       {
         hero->setName(name_entry->get_text());
-        if (male_radiobutton->get_active())
+        if (male_radiobutton->get_active() == true && 
+	    hero->getGender() == Hero::FEMALE)
           hero->setGender(Hero::MALE);
-        else
-          hero->setGender(Hero::FEMALE);
+        else if (male_radiobutton->get_active() == false &&
+		 hero->getGender() == Hero::MALE)
+	  hero->setGender(Hero::FEMALE);
 	return true;
       }
     else
 	return false;
 }
 
-void HeroOfferDialog::on_map_changed(SDL_Surface *map)
+void HeroOfferDialog::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
 {
-    map_image->property_pixbuf() = to_pixbuf(map);
+    map_image->property_pixmap() = map;
 }
 
