@@ -50,7 +50,7 @@ int main(int argc, char* argv[])
 
   initialize_configuration();
   Vector<int>::setMaximumWidth(1000);
-  RecentlyPlayedGameList::getInstance()->loadFromFile(File::getSavePath() + "/recently-played.xml");
+  RecentlyPlayedGameList::getInstance()->loadFromFile(File::getSavePath() + "/" + RECENTLY_PLAYED_LIST);
 
   #if ENABLE_NLS
   //cout << "Configuration::s_lang.c_str(): " << Configuration::s_lang.c_str() << endl;
@@ -92,6 +92,7 @@ int main(int argc, char* argv[])
 		  cerr <<_("non-numerical value for seed value") <<endl;
 		  exit(-1);
 		}
+              kit.random_number_seed = seed;
 	      srand(seed);
 	    }
 	  else if (parameter == "--turn")
