@@ -1,4 +1,4 @@
-// Copyright (C) 2008, 2009 Ben Asselstine
+// Copyright (C) 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -149,6 +149,36 @@ bool NetworkPlayer::chooseQuest(Hero *hero)
   return true;
 }
 
+bool NetworkPlayer::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
+{
+  assert (false);
+  return true;
+}
+
+bool NetworkPlayer::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
+{
+  assert (false);
+  return true;
+}
+
+bool NetworkPlayer::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
+{
+  assert (false);
+  return true;
+}
+
+bool NetworkPlayer::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
+{
+  assert (false);
+  return true;
+}
+
+bool NetworkPlayer::computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns)
+{
+  assert (false);
+  return true;
+}
+
 void NetworkPlayer::decodeActions(std::list<Action *> actions)
 {
   if (isDead())
@@ -254,6 +284,9 @@ void NetworkPlayer::decodeAction(const Action *a)
     case Action::CITY_LOOT:
       return decodeActionLoot
 	(dynamic_cast<const Action_Loot*>(a));
+    case Action::USE_ITEM:
+      return decodeActionUseItem
+	(dynamic_cast<const Action_UseItem*>(a));
     }
 
   return;
@@ -500,6 +533,7 @@ void NetworkPlayer::decodeActionEquip(const Action_Equip *action)
   Hero *hero = dynamic_cast<Hero *>(stack->getArmyById(action->getHeroId()));
   Item *item = 0;
 
+  bool splash = false;
   switch (action->getToBackpackOrToGround())
   {
   case Action_Equip::BACKPACK:
@@ -509,7 +543,7 @@ void NetworkPlayer::decodeActionEquip(const Action_Equip *action)
 
   case Action_Equip::GROUND:
     item = hero->getBackpack()->getItemById(action->getItemId());
-    doHeroDropItem(hero, item, action->getItemPos());
+    doHeroDropItem(hero, item, action->getItemPos(), splash);
     break;
   }
 }
@@ -671,4 +705,20 @@ void NetworkPlayer::decodeActionLoot (const Action_Loot *action)
   doLootCity(looted, action->getAmountToAdd(), action->getAmountToSubtract());
 }
 
+void NetworkPlayer::decodeActionUseItem(const Action_UseItem *action)
+{
+  Stack *stack = d_stacklist->getArmyStackById(action->getHeroId());
+  if (stack == NULL)
+    {
+      printf ("couldn't find hero with id %d\n", action->getHeroId());
+    }
+  assert (stack != NULL);
+  Hero *hero = dynamic_cast<Hero *>(stack->getArmyById(action->getHeroId()));
+  assert (hero != NULL);
+  Item *item = hero->getBackpack()->getItemById(action->getItemId());
+  assert (item != NULL);
+  Player *victim = Playerlist::getInstance()->getPlayer(action->getVictimPlayerId());
+  doHeroUseItem(hero, item, victim);
+}
+
 // End of file
