@@ -1,4 +1,4 @@
-// Copyright (C) 2008, 2009 Ben Asselstine
+// Copyright (C) 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -59,6 +59,11 @@ class NetworkPlayer : public Player
 	virtual bool chooseTreachery (Stack *stack, Player *player, Vector <int> pos);
         virtual Army::Stat chooseStat(Hero *hero);
         virtual bool chooseQuest(Hero *hero);
+        virtual bool computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns);
+        virtual bool computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns);
+        virtual bool computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns);
+        virtual bool computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns);
+        virtual bool computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns);
 
 	void decodeAction(const Action *action);
 	void decodeActions(std::list<Action *> actions);
@@ -104,6 +109,7 @@ class NetworkPlayer : public Player
 	void decodeActionCityTooPoorToProduce(const Action_CityTooPoorToProduce *action);
 	void decodeActionInitTurn(const Action_InitTurn*action);
 	void decodeActionLoot(const Action_Loot*action);
+	void decodeActionUseItem(const Action_UseItem*action);
 };
 
 #endif // NETWORK_PLAYER_H
