@@ -3,7 +3,7 @@
 // Copyright (C) 2003 Marek Publicewicz
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2005 Bryan Duff
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -37,8 +37,8 @@
 #include "defs.h"
 #include "callback-enums.h"
 
-class Stacklist;
 class XML_Helper;
+class Stacklist;
 class Hero;
 class HeroProto;
 class Action;
@@ -229,6 +229,8 @@ class Player: public sigc::trackable
         //! Set the fight order of the player.
 	void setFightOrder(std::list<guint32> order);
 
+        //! Set path of the stack to the previously moved stack's destination.
+        bool setPathOfStackToPreviousDestination(Stack *stack);
 
 
 	// Get Methods
@@ -352,7 +354,7 @@ class Player: public sigc::trackable
         void dumpHistorylist() const;
 
 	//! Check the player's history to see if we've conquered the given city.
-	bool conqueredCity(City *c) const;
+	bool conqueredCity(City *c, guint32 &turns_ago) const;
 
         //! Check the player's history to see if we've explored the given ruin.
         bool searchedRuin(Ruin *r) const;
@@ -402,6 +404,14 @@ class Player: public sigc::trackable
         //! Remove a Stack from the player's Stacklist.
         bool deleteStack(Stack* stack);
 
+        //! Return a list of all of the player's items that can be used.
+        std::list<Item*> getUsableItems() const;
+
+        //! Return whether or not the player has any items that can be used.
+        bool hasUsableItem() const;
+
+        //! Return which stack and hero the item belongs to.
+        bool getItemHolder(Item *item, Stack **stack, Hero **hero) const;
 
 	// Methods that operate on the player's diplomatic data members.
 
@@ -525,17 +535,19 @@ class Player: public sigc::trackable
          * @param defender         The list of defending stacks.  This list
 	 *                         consists of a single Army unit in a 
 	 *                         single Stack.
+         * @param stackdied        Whether or not the stack went away because
+         *                         of the searching of the ruin.
 	 *
          *  If the defender dies in the fight, the defender pointer is set 
 	 *  to 0.
 	 *  If the Hero loses the battle, only the Hero unit is removed
 	 *  from the attacker's stack.
-	 *
+         *
          * @return One of Fight::ATTACKER_WON, Fight::DEFENDER_WON, or
 	 *         Fight::DRAW (Fight::Result).
          */
 	//! Callback to adjudicate fights in ruins.
-        Fight::Result stackRuinFight(Stack** attacker, Stack** defender);
+        Fight::Result stackRuinFight(Stack** attacker, Stack** defender, bool &stackdied);
         /** 
 	 * A stack searches a ruin.  The stack must contain a hero.
 	 *
@@ -544,12 +556,14 @@ class Player: public sigc::trackable
 	 *
          * @param stack            The stack which searches the ruin.
          * @param ruin             The ruin to be searched.
+         * @param stackdied        Whether or not the stack went away because
+         *                         of the searching of the ruin.
 	 *
          * @return reward          A pointer to the received Reward.  Return 
 	 *                         NULL if the keeper could not be defeated.
          */
 	//! Callback to have a stack visit a ruin.
-        Reward* stackSearchRuin(Stack* stack, Ruin* ruin);
+        Reward* stackSearchRuin(Stack* stack, Ruin* ruin, bool &stackdied);
 
         /** 
 	 * A stack visits a temple and becomes blessed. By blessing, the 
@@ -658,11 +672,13 @@ class Player: public sigc::trackable
 	 * @param item             The Item to drop onto the ground.
 	 * @param pos              The position of the tile on the game map to 
 	 *                         drop the item onto.
+         * @param splash           Whether or not the item sunk in the water
+         *                         after dropping it.
 	 *
          * @return False on error, true otherwise.
 	 */
         //! Callback to have a Hero drop an Item.
-        bool heroDropItem(Hero *hero, Item *item, Vector<int> pos);
+        bool heroDropItem(Hero *hero, Item *item, Vector<int> pos, bool &splash);
 
 	/**
 	 * Callback to drop a all items at a particular position on the 
@@ -679,11 +695,13 @@ class Player: public sigc::trackable
 	 * @param hero             The Hero that holds the items.
 	 * @param pos              The position of the tile on the game map to 
 	 *                         drop the item onto.
+         * @param splash           Whether or not the items sunk in the water
+         *                         after dropping them.
 	 *
          * @return False on error, true otherwise.
 	 */
         //! Callback to have a Hero drop all items.
-        bool heroDropAllItems(Hero *hero, Vector<int> pos);
+        bool heroDropAllItems(Hero *hero, Vector<int> pos, bool &splash);
 
 	/**
 	 * Callback to pickup an Item at a particular position on the game 
@@ -710,6 +728,9 @@ class Player: public sigc::trackable
 	//! Pick up all of the items at the given location on the game map.
 	bool heroPickupAllItems(Hero *h, Vector<int> pos);
 
+        //! Have the given hero use the given item, on the given player.
+        bool heroUseItem(Hero *h, Item *item, Player *player);
+
 	/**
 	 * Completing a Quest entails that the Hero is going to receive a
 	 * reward, but that happens in Player::giveReward.
@@ -1019,6 +1040,17 @@ class Player: public sigc::trackable
         //! Decision callback for when a hero visits a temple.
         virtual bool chooseQuest(Hero *hero) = 0;
 
+        //! Decision callback for when an ai player considers going to a ruin.
+        virtual bool computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns) = 0;
+        //! Decision callback for when an ai player considers picking up a bag.
+        virtual bool computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns) = 0;
+        //! Decision callback for when the ai going to a temple.
+        virtual bool computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns) = 0;
+        //! Decision callback for when the ai considers obtaining a quest.
+        virtual bool computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns) = 0;
+        //! Decision callback for considering the next target in a quest.
+        virtual bool computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns) = 0;
+
 	// Player related actions the player can take.
 
         /** 
@@ -1087,8 +1119,6 @@ class Player: public sigc::trackable
 	 * @param dist         The maximum number of tiles that a temple
 	 *                     can be away from the stack, and be considered
 	 *                     for visiting.
-	 * @param mp           The maximum number of movement points that a
-	 *                     stack needs to have to reach the temple.
 	 * @param percent_can_be_blessed  If the stack has this many army 
 	 *                                units that have not been blessed
 	 *                                at the temple (expressed as a
@@ -1103,18 +1133,24 @@ class Player: public sigc::trackable
 	 *
 	 * Returns true if the stack moved, false if it stayed still.
 	 */
-	bool AI_maybeVisitTempleForBlessing(Stack *s, int dist, int mp, 
+	bool AI_maybeVisitTempleForBlessing(Stack *s, int dist,
 					    double percent_can_be_blessed, 
 					    bool &blessed, bool &stack_died);
 
-        bool AI_maybeVisitTempleForQuest(Stack *s, int dist, int max_mp, 
+        bool AI_maybeVisitTempleForQuest(Stack *s, int dist, bool &got_quest,
                                          bool &stack_died);
 
-        bool AI_maybeVisitRuin(Stack *s, int dist, int max_mp, 
+        bool AI_maybeVisitRuin(Stack *s, int dist, bool &visited_ruin,
                                bool &stack_died);
 
         Vector<int> AI_getQuestDestination(Quest *quest, Stack *stack) const;
         bool AI_invadeCityQuestPreference(City *c, CityDefeatedAction &action) const;
+        bool AI_maybeContinueQuest(Stack *s, Quest *quest, 
+                                   bool &completed_quest, bool &stack_died);
+
+	bool AI_maybePickUpItems (Stack *s, int dist, bool &picked_up,
+				  bool &stack_died);
+
 	/** 
 	 * Callback to decide if we perform treachery on a friendly player.
 	 *
@@ -1232,7 +1268,12 @@ class Player: public sigc::trackable
 	//! Propose a new diplomatic state wrt another player
 	void proposeDiplomacy (DiplomaticProposal proposal, Player *player);
 
-
+        /**
+         * Account for the dead armies in the given list of stacks.  For
+         * each dead army we increment a counter for that kind of army unit.
+         */
+        //! Keeps stats of what kind of units we killed in a battle.
+        void tallyDeadArmyTriumphs(std::list<Stack*> &stacks);
 
 
 	// Signals
@@ -1269,28 +1310,23 @@ class Player: public sigc::trackable
         //! Emitted whever a player's army gets a new medal.
         sigc::signal<void, Army*, int> snewMedalArmy;
 
-	/**
-	 * @param army         The army that has died.
-	 * @param culprit_ids  A std::vector of enemy Army Ids to have 
-	 *                     participated in the death of this player's Army.
-	 */
-        //! Emitted when a player's Army dies.
-        sigc::signal<void, Army*, std::vector<guint32> > sdyingArmy;
-
 	/**
 	 * @param ruin    The ruin being searched.
 	 * @param stack   The stack doing the searching (must contain Hero).
-	 * @param reward  The reward received.
+         *
+         * Returns whether or not the stack was deleted as a result.
 	 */
-        //! Emitted whenever the player successfully searched a ruin.
-        sigc::signal<void, Ruin*, Stack*> ssearchingRuin;
+        //! Emitted by the player to search a ruin.
+        sigc::signal<bool, Ruin*, Stack*> ssearchingRuin;
 
 	/**
 	 * @param temple  The temple being visited.
-	 * @param stack   The stack that has gotten blessed.
+	 * @param stack   The stack to be blessed.
+         *
+         * Returns whether or not a hero got a quest.
 	 */
-        //! Emitted whenever the player visits a temple.
-        sigc::signal<void, Temple*, Stack*> svisitingTemple;
+        //! Emitted by the player to visit a temple.
+        sigc::signal<bool, Temple*, Stack*> svisitingTemple;
 
 	/**
 	 * @param city   The city being occupied.
@@ -1402,8 +1438,16 @@ class Player: public sigc::trackable
 
         sigc::signal<void, int> hero_arrives_with_allies;
 
+        sigc::signal<void, Item*> using_item;
+
         sigc::signal<void, NetworkAction *> acting;
         sigc::signal<void, NetworkHistory *> history_written;
+
+        //! Results of using items
+        sigc::signal<void, Player*, guint32> stole_gold;
+        sigc::signal<void, Player*, guint32> sunk_ships;
+        sigc::signal<void, Hero *, guint32> bags_picked_up;
+        sigc::signal<void, Hero *, guint32> mp_added_to_hero_stack;
         
 	void loadPbmGame();
 	//! Check the history to see if we ever conquered the given city.
@@ -1575,9 +1619,11 @@ class Player: public sigc::trackable
         void doCityBuyProduction(City *c, int slot, int type);
         void doCityChangeProduction(City *c, int slot);
         void doGiveReward(Stack *s, Reward *reward);
-        void doHeroDropItem(Hero *hero, Item *item, Vector<int> pos);
-	bool doHeroDropAllItems(Hero *h, Vector<int> pos);
+        void doHeroDropItem(Hero *hero, Item *item, Vector<int> pos, bool &splash);
+	bool doHeroDropAllItems(Hero *h, Vector<int> pos, bool &splash);
+        bool doHeroUseItem(Hero *h, Item *item, Player *victim);
         void doHeroPickupItem(Hero *hero, Item *item, Vector<int> pos);
+        bool doHeroPickupAllItems(Hero *h, Vector<int> pos);
         void doHeroGainsLevel(Hero *hero, Army::Stat stat);
         bool doStackDisband(Stack *stack);
         void doSignpostChange(Signpost *signpost, std::string message);
@@ -1605,16 +1651,9 @@ class Player: public sigc::trackable
 
 	void AI_maybeBuyScout(City *c);
 
-
-	bool AI_maybePickUpItems (Stack *s, int dist, int mp, bool &picked_up,
-				  bool &stack_died);
-
-
-
 	bool AI_maybeVector(City *c, guint32 safe_mp, guint32 min_defenders,
 			    City *target, City **vector_city = NULL);
 
-
 	void AI_setupVectoring(guint32 safe_mp, guint32 min_defenders,
 			       guint32 mp_to_front);
 
@@ -1640,6 +1679,7 @@ class Player: public sigc::trackable
         void getHeroes(const std::list<Stack*> stacks, 
 		       std::vector<guint32>& heroes);
 
+
         /** 
 	 * Goes through a list of stacks and removes all armies with less
          * than 1 hitpoint.  It also removes empty stacks. 
@@ -1649,11 +1689,23 @@ class Player: public sigc::trackable
          * @param culprits         The list of heroes responsible for killing
 	 *                         the armies.  This is needed for tracking
 	 *                         the progress of a Quest.
-         * @return The sum of the XP of the killed armies.
+         * @return The number of armies removed because they were killed.
          */
 	//! Remove dead Armies from a list of stacks after a fight.
-        double removeDeadArmies(std::list<Stack*>& stacks,
+        guint32 removeDeadArmies(std::list<Stack*>& stacks,
                                 std::vector<guint32>& culprits);
+
+        guint32 removeDeadArmies(std::list<Stack*>& stacks);
+
+        guint32 removeDeadArmies(Stack *stack);
+
+        double countXPFromDeadArmies(std::list<Stack*>& stacks);
+
+        void handleDeadHeroes(std::list<Stack*> &stacks);
+
+        void healInjuredArmies(std::list<Stack*> &stacks);
+
+        void handleDeadArmiesForQuests(std::list<Stack*> &stacks, std::vector<guint32> &culprits);
         
         /** 
 	 * Increases the number of experience points of a stack
@@ -1700,6 +1752,8 @@ class Player: public sigc::trackable
 	static void pruneCityProductions(std::list<Action*> actions);
 
         std::list<Action *> getActionsThisTurn(int type) const;
+
+        bool computerSearch(Stack *s, MoveResult *r);
 };
 
 #endif // PLAYER_H
