@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "xmlhelper.h"
 #include "ucompose.hpp"
 #include "shieldset.h"
+#include "tarhelper.h"
 
 std::string Shield::d_tag = "shield";
 
@@ -54,14 +55,13 @@ Gdk::Color Shield::get_default_color_for_no(int player_no)
   switch (player_no % MAX_PLAYERS)
     {
     case Shield::WHITE: c.set_rgb_p(252.0/255.0,252.0/255.0,252.0/255.0); break;
-    //case 1: color.r = 80; color.b = 28; color.g = 172; break;
     case Shield::GREEN: c.set_rgb_p(80.0/255.0, 195.0/255.0, 28.0/255.0); break;
     case Shield::YELLOW: c.set_rgb_p(252.0/255.0,236.0/255.0,32.0/255.0); break;
-    case Shield::LIGHT_BLUE: c.set_rgb_p(0,252.0/255.0,252.0/255.0); break;
-    case Shield::RED: c.set_rgb_p(252.0/255.0,160.0/255.0,0);break;
-    case Shield::DARK_BLUE: 
+    case Shield::DARK_BLUE: c.set_rgb_p(0,252.0/255.0,252.0/255.0); break;
+    case Shield::ORANGE: c.set_rgb_p(252.0/255.0,160.0/255.0,0);break;
+    case Shield::LIGHT_BLUE: 
 		      c.set_rgb_p(44.0/255.0,184.0/255.0,252.0/255.0); break;
-    case Shield::ORANGE: c.set_rgb_p(196.0/255.0, 28.0/255.0, 0); break;
+    case Shield::RED: c.set_rgb_p(196.0/255.0, 28.0/255.0, 0); break;
     case Shield::BLACK: c.set_rgb_p(0,0,0); break;
     }
     
@@ -116,11 +116,26 @@ bool Shield::save(XML_Helper *helper) const
 	
 void Shield::instantiateImages(Shieldset *s)
 {
+  bool broken = false;
+  Tar_Helper t(s->getConfigurationFile(), std::ios::in, broken);
+  if (broken)
+    return;
   for (iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getImageName().empty() == false)
-	(*it)->instantiateImages(s->getFile((*it)->getImageName()), s);
+        {
+          std::string pngfile = t.getFile((*it)->getImageName() + ".png", 
+                                          broken);
+          if (broken == false)
+            {
+              (*it)->instantiateImages(pngfile, s);
+              File::erase(pngfile);
+            }
+          else
+            return;
+        }
     }
+  t.Close();
 }
 void Shield::uninstantiateImages()
 {
@@ -132,7 +147,7 @@ ShieldStyle *Shield::getFirstShieldstyle(ShieldStyle::Type type)
 {
   for (iterator i = begin(); i != end(); i++)
     {
-      if ((*i)->getType() == type)
+      if (ShieldStyle::Type((*i)->getType()) == type)
 	return *i;
     }
   return NULL;
