@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <iostream>
 #include <expat.h>
+#include <assert.h>
 #include "rectangle.h"
 #include <sigc++/functors/mem_fun.h>
 
@@ -25,11 +26,12 @@
 #include "File.h"
 #include "defs.h"
 #include "ucompose.hpp"
+#include "tarhelper.h"
 
 using namespace std;
 
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
-//#define debug(x)
+//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x)
 
 Shieldsetlist* Shieldsetlist::s_instance = 0;
 
@@ -58,7 +60,7 @@ void Shieldsetlist::loadShieldsets(std::list<std::string> shieldsets)
         Shieldset *shieldset = loadShieldset(*i);
 	if (!shieldset)
 	  continue;
-	add(shieldset);
+	add(shieldset, *i);
       }
 }
 
@@ -85,6 +87,18 @@ std::list<std::string> Shieldsetlist::getNames() const
     names.push_back((*it)->getName());
   return names;
 }
+
+std::list<std::string> Shieldsetlist::getValidNames() const
+{
+  std::list<std::string> names;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->validate() == true)
+        names.push_back((*it)->getName());
+    }
+  return names;
+}
+
 std::string Shieldsetlist::getShieldsetDir(std::string name) const
 {
   DirMap::const_iterator it = d_dirs.find(name);
@@ -96,24 +110,45 @@ std::string Shieldsetlist::getShieldsetDir(std::string name) const
 
 Shieldset* Shieldsetlist::loadShieldset(std::string name)
 {
-  debug("Loading shieldset " <<File::get_basename(File::get_dirname(name)));
+  bool unsupported_version = false;
+  debug("Loading shieldset " <<File::get_basename(name));
 
-  Shieldset *shieldset = Shieldset::create(name);
+  Shieldset *shieldset = Shieldset::create(name, unsupported_version);
   if (!shieldset)
-    return NULL;
-
-  if (shieldset->validate() == false)
     {
-      cerr << "Error!  shieldset: `" << shieldset->getName() << 
-	"' is invalid.  Skipping." << endl;
-      delete shieldset;
+      cerr << "Error!  shieldset: `" << File::get_basename(name, true) << 
+	"' is malformed.  Skipping." << endl;
       return NULL;
     }
 
+  if (d_shieldsets.find(shieldset->getBaseName()) != d_shieldsets.end())
+    {
+      Shieldset *s = (*d_shieldsets.find(shieldset->getBaseName())).second;
+      cerr << "Error!  shieldset: `" << shieldset->getConfigurationFile() << 
+	"' shares a duplicate shieldset basename `" << s->getBaseName() << "' with `" << s->getConfigurationFile() 
+	<< "'.  Skipping." << endl;
+      delete shieldset;
+      return NULL;
+    }
+    
+  if (d_dirs.find(shieldset->getName()) != d_dirs.end())
+    {
+      std::string basename = (*d_dirs.find(shieldset->getName())).second;
+      if (basename != "")
+        {
+          Shieldset *s = (*d_shieldsets.find(basename)).second;
+          cerr << "Error!  shieldset: `" << shieldset->getConfigurationFile() 
+            << "' shares a duplicate shieldset name `" << s->getName() << 
+            "' with `" << s->getConfigurationFile() << "'.  Skipping." << endl;
+          delete shieldset;
+        }
+      return NULL;
+    }
+    
   if (d_shieldsetids.find(shieldset->getId()) != d_shieldsetids.end())
     {
       Shieldset *s = (*d_shieldsetids.find(shieldset->getId())).second;
-      cerr << "Error!  shieldset: `" << shieldset->getName() << 
+      cerr << "Error!  shieldset: `" << shieldset->getConfigurationFile() << 
 	"' shares a duplicate shieldset id with `" << s->getConfigurationFile() 
 	<< "'.  Skipping." << endl;
       delete shieldset;
@@ -123,13 +158,13 @@ Shieldset* Shieldsetlist::loadShieldset(std::string name)
   return shieldset;
 }
 
-void Shieldsetlist::add(Shieldset *shieldset)
+void Shieldsetlist::add(Shieldset *shieldset, std::string file)
 {
-  std::string subdir = File::get_basename(shieldset->getDirectory());
+  std::string basename = File::get_basename(file);
   push_back(shieldset);
-  shieldset->setSubDir(subdir);
-  d_dirs[shieldset->getName()] = subdir;
-  d_shieldsets[subdir] = shieldset;
+  shieldset->setBaseName(basename);
+  d_dirs[shieldset->getName()] = basename;
+  d_shieldsets[basename] = shieldset;
   d_shieldsetids[shieldset->getId()] = shieldset;
 }
         
@@ -169,84 +204,109 @@ Shieldset *Shieldsetlist::getShieldset(guint32 id)  const
   return (*it).second;
 }
 
-Shieldset *Shieldsetlist::getShieldset(std::string dir) const
+Shieldset *Shieldsetlist::getShieldset(std::string bname) const
 { 
-  ShieldsetMap::const_iterator it = d_shieldsets.find(dir);
+  ShieldsetMap::const_iterator it = d_shieldsets.find(bname);
   if (it == d_shieldsets.end())
     return NULL;
   return (*it).second;
 }
 
-bool Shieldsetlist::addToPersonalCollection(Shieldset *shieldset, std::string &new_subdir, guint32 &new_id)
+Shieldset *Shieldsetlist::import(Tar_Helper *t, std::string f, bool &broken)
+{
+  bool unsupported_version = false;
+  std::string filename = t->getFile(f, broken);
+  Shieldset *shieldset = Shieldset::create(filename, unsupported_version);
+  assert (shieldset != NULL);
+  shieldset->setBaseName(File::get_basename(f));
+
+  std::string basename = "";
+  guint32 id = 0;
+  addToPersonalCollection(shieldset, basename, id);
+
+  return shieldset;
+
+}
+
+std::string Shieldsetlist::findFreeBaseName(std::string basename, guint32 max, guint32 &num) const
+{
+  std::string new_basename;
+  for (unsigned int count = 1; count < max; count++)
+    {
+      new_basename = String::ucompose("%1%2", basename, count);
+      if (getShieldset(new_basename) == NULL)
+        {
+          num = count;
+          break;
+        }
+      else
+        new_basename = "";
+    }
+  return new_basename;
+}
+
+bool Shieldsetlist::addToPersonalCollection(Shieldset *shieldset, std::string &new_basename, guint32 &new_id)
 {
   //do we already have this one?
       
-  if (getShieldset(shieldset->getSubDir()) == getShieldset(shieldset->getId()) 
-      && getShieldset(shieldset->getSubDir()) != NULL)
+  if (getShieldset(shieldset->getBaseName()) == getShieldset(shieldset->getId()) 
+      && getShieldset(shieldset->getBaseName()) != NULL)
     {
       shieldset->setDirectory(getShieldset(shieldset->getId())->getDirectory());
       return true;
     }
 
-  //if the subdir conflicts with any other subdir, then change it.
-  if (getShieldset(shieldset->getSubDir()) != NULL)
+  //if the basename conflicts with any other basename, then change it.
+  if (getShieldset(shieldset->getBaseName()) != NULL)
     {
-      bool found = false;
-      for (int count = 0; count < 100; count++)
-	{
-	  new_subdir = String::ucompose("%1%2", shieldset->getSubDir(), count);
-	  if (getShieldset(new_subdir) == NULL)
-	    {
-	      found = true;
-	      break;
-	    }
-	}
-      if (found == false)
-	return false;
-      shieldset->setSubDir(new_subdir);
+      if (new_basename != "" && getShieldset(new_basename) == NULL)
+        ;
+      else
+        {
+          guint32 num = 0;
+          std::string new_basename = findFreeBaseName(shieldset->getBaseName(), 100, num);
+          if (new_basename == "")
+            return false;
+        }
     }
-  else
-    new_subdir = shieldset->getSubDir();
+  else if (new_basename == "")
+    new_basename = shieldset->getBaseName();
 
   //if the id conflicts with any other id, then change it
   if (getShieldset(shieldset->getId()) != NULL)
     {
-      new_id = Shieldsetlist::getNextAvailableId(shieldset->getId());
-      shieldset->setId(new_id);
+      if (new_id != 0 && getShieldset(new_id) == NULL)
+        shieldset->setId(new_id);
+      else
+        {
+          new_id = Shieldsetlist::getNextAvailableId(shieldset->getId());
+          shieldset->setId(new_id);
+        }
     }
   else
     new_id = shieldset->getId();
 
   //make the directory where the shieldset is going to live.
-  std::string directory = 
-    File::getUserShieldsetDir() + shieldset->getSubDir() + "/";
+  std::string file = File::getUserShieldsetDir() + new_basename + Shieldset::file_extension;
 
-  if (File::create_dir(directory) == false)
-    return false;
+  shieldset->save(file, Shieldset::file_extension);
 
-  //okay now we copy the image files into the new directory 
-  std::list<std::string> files;
-  shieldset->getFilenames(files);
-  for (std::list<std::string>::iterator it = files.begin(); it != files.end();
-       it++)
-    File::copy(shieldset->getFile(*it), directory + *it);
-
-  //save out the shieldset file
-  shieldset->setDirectory(directory);
-  XML_Helper helper(shieldset->getConfigurationFile(), std::ios::out, false);
-  shieldset->save(&helper);
-  helper.close();
+  if (new_basename != shieldset->getBaseName())
+    shieldset->setBaseName(new_basename);
+  shieldset->setDirectory(File::get_dirname(file));
+  add (shieldset, file);
   return true;
 }
 
 int Shieldsetlist::getNextAvailableId(int after)
 {
+  bool unsupported_version = false;
   std::list<guint32> ids;
   std::list<std::string> shieldsets = Shieldset::scanSystemCollection();
   for (std::list<std::string>::const_iterator i = shieldsets.begin(); 
        i != shieldsets.end(); i++)
     {
-      Shieldset *shieldset = Shieldset::create(*i);
+      Shieldset *shieldset = Shieldset::create(*i, unsupported_version);
       if (shieldset != NULL)
 	{
 	  ids.push_back(shieldset->getId());
@@ -257,7 +317,7 @@ int Shieldsetlist::getNextAvailableId(int after)
   for (std::list<std::string>::const_iterator i = shieldsets.begin(); 
        i != shieldsets.end(); i++)
     {
-      Shieldset *shieldset = Shieldset::create(*i);
+      Shieldset *shieldset = Shieldset::create(*i, unsupported_version);
       if (shieldset != NULL)
 	{
 	  ids.push_back(shieldset->getId());
@@ -271,3 +331,23 @@ int Shieldsetlist::getNextAvailableId(int after)
     }
   return -1;
 }
+
+bool Shieldsetlist::contains(std::string name) const
+{
+  std::list<std::string> n = getNames();
+  for (std::list<std::string>::iterator it = n.begin(); it != n.end(); it++)
+    {
+      if (*it == name)
+        return true;
+    }
+  return false;
+}
+
+guint32 Shieldsetlist::getShieldsetId(std::string basename) const
+{
+  Shieldset *ss = getShieldset(basename);
+  if (ss)
+    return ss->getId();
+  else
+    return 0;
+}
