@@ -3,7 +3,7 @@
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2004, 2005 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -170,8 +170,8 @@ void Stack::moveToDest(Vector<int> dest, bool skipping)
 	      ship_load_unload = true;
 	      Vector<int> pos = getPos();
 	      GameMap *gm = GameMap::getInstance();
-	      bool to_water = (gm->getTile(dest.x,dest.y)->getMaptileType() 
-			       == Tile::WATER);
+	      bool to_water = (gm->getTile(dest.x,dest.y)->getType() 
+                               == Tile::WATER);
 	      for (Stack::iterator it = begin(); it != end(); it++)
 		{
 		  if (to_water && 
@@ -189,7 +189,7 @@ void Stack::moveToDest(Vector<int> dest, bool skipping)
 	(*it)->setInShip(false);
     }
 
-  guint32 maptype = GameMap::getInstance()->getTile(dest.x,dest.y)->getMaptileType();
+  guint32 maptype = GameMap::getInstance()->getTile(dest.x,dest.y)->getType();
   //how many moves does the stack need to travel to dest?
   int needed_moves = calculateTileMovementCost(dest);
 
@@ -290,9 +290,15 @@ void Stack::decrementMoves(guint32 moves)
   debug("decrement_moves()");
 
   for (iterator it = begin(); it != end(); it++)
-    {
-      (*it)->decrementMoves(moves);
-    }
+    (*it)->decrementMoves(moves);
+}
+
+void Stack::incrementMoves(guint32 moves)
+{
+  debug("increment_moves()");
+
+  for (iterator it = begin(); it != end(); it++)
+    (*it)->incrementMoves(moves);
 }
 
 // Purpose: Return the strongest army of a group
@@ -385,7 +391,7 @@ guint32 Stack::calculateTileMovementCost(Vector<int> pos) const
   Maptile* tile = GameMap::getInstance()->getTile(pos);
   guint32 moves = tile->getMoves();
   guint32 bonus = calculateMoveBonus();
-  if (bonus & tile->getMaptileType() && moves > 1)
+  if (bonus & tile->getType() && moves > 1)
     moves = 2;
   else if (isFlying() && moves > 1)
     moves = 2;
@@ -394,7 +400,7 @@ guint32 Stack::calculateTileMovementCost(Vector<int> pos) const
 
 Vector<int> Stack::getFirstPointInPath() const
 {
-  if (d_path->size() == 0)
+  if (hasPath() == false)
     return Vector<int>(-1,-1);
   Vector<int> p = *(d_path->begin());
   return p;
@@ -423,7 +429,7 @@ Vector<int> Stack::getLastPointInPath() const
 
 bool Stack::enoughMoves() const
 {
-  if (d_path->size() == 0)
+  if (hasPath() == false)
     return true; //we have enough moves to move nowhere!
 
   Vector<int> p = getFirstPointInPath();
@@ -830,6 +836,15 @@ Stack* Stack::createNonUniqueStack(Player *player, Vector<int> pos)
   return new Stack(0, player, pos);
 }
 
+guint32 Stack::getMaxMoves() const
+{
+  GameMap *gm = GameMap::getInstance();
+  if (gm->getTile(getPos())->getType() != Tile::WATER)
+    return getMaxLandMoves();
+  else
+    return getMaxBoatMoves();
+}
+
 guint32 Stack::getMaxLandMoves() const
 {
   if (empty())
@@ -1110,4 +1125,117 @@ guint32 Stack::countItems() const
     }
   return count;
 }
+
+bool Stack::hasUsableItem() const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == false)
+        continue;
+      Hero *hero = dynamic_cast<Hero*>(*it);
+      Backpack *backpack = hero->getBackpack();
+      if (backpack->hasUsableItem() == true)
+        return true;
+    }
+  return false;
+}
+        
+void Stack::getUsableItems(std::list<Item*> &items) const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == false)
+        continue;
+      Hero *hero = dynamic_cast<Hero*>(*it);
+      Backpack *backpack = hero->getBackpack();
+      backpack->getUsableItems(items);
+    }
+  return;
+}
+
+Hero* Stack::getHeroWithItem(Item *item) const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == false)
+        continue;
+      Hero *hero = dynamic_cast<Hero*>(*it);
+      Backpack *backpack = hero->getBackpack();
+      if (backpack->getItemById(item->getId()) != NULL)
+        return hero;
+    }
+  return NULL;
+}
+
+void Stack::kill()
+{
+  for (iterator it = begin(); it != end(); it++)
+    (*it)->kill();
+}
+
+//! Sink the stack.  return true if we've sunk any part of the stack.
+bool Stack::killArmyUnitsInBoats()
+{
+  bool retval = false;
+  GameMap *gm = GameMap::getInstance();
+  if (gm->getTile(getPos())->getType() != Tile::WATER)
+    return retval;
+  if (isFlying())
+    return retval;
+  std::list<Army*> flyers;
+  std::list<Army*> landedother;
+  std::list<Army*> landedhero;
+  for (iterator it = this->begin(); it != this->end(); it++)
+    {
+      guint32 bonus = (*it)->getStat(Army::MOVE_BONUS);
+      if (bonus == Tile::GRASS || (bonus & Tile::WATER) == 0 || 
+	  (bonus & Tile::FOREST) == 0 || (bonus & Tile::HILLS) == 0 ||
+	  (bonus & Tile::MOUNTAIN) == 0 || (bonus & Tile::SWAMP) == 0)
+	{
+	  if ((*it)->isHero())
+            landedhero.push_back(*it);
+	  else
+            landedother.push_back(*it);
+	}
+      else
+        flyers.push_back(*it);
+    }
+  //sink the landed others.
+  for (std::list<Army*>::iterator it = landedother.begin(); 
+       it != landedother.end(); it++)
+    {
+      retval = true;
+      (*it)->kill();
+    }
+  int num_heroes_to_sink = landedhero.size() - flyers.size();
+  if (num_heroes_to_sink > 0)
+    {
+      //sink the unlucky heroes and any items they might have.
+      for (std::list<Army*>::reverse_iterator it = landedhero.rbegin();
+           it != landedhero.rend(); it++)
+        {
+          retval = true;
+          (*it)->kill();
+          num_heroes_to_sink--;
+          if (num_heroes_to_sink <= 0)
+            break;
+        }
+    }
+      
+  for (std::list<Army*>::iterator it = landedhero.begin(); 
+       it != landedhero.end(); it++)
+    {
+      if ((*it)->getHP() > 0)
+        (*it)->setInShip(false); //we're being carried by a flyer
+    }
+  return retval;
+}
+          
+guint32 Stack::countBlessings() const
+{
+  int count = 0;
+  for (const_iterator it = begin(); it != end(); it++)
+    count += (*it)->countBlessings();
+  return count;
+}
 // End of file
