@@ -40,6 +40,7 @@ class ArmyProdBase;
 class XML_Helper;
 class Temple;
 class Hero;
+class Item;
 
 //! A set of up to eight Army units that move as a single entity on the map.
 /** 
@@ -90,6 +91,9 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 	//! Returns the max MP the stack would have if it were in the water
         guint32 getMaxBoatMoves() const;
 
+        //! Returns the maximum number of movements points for this stack.
+        guint32 getMaxMoves() const;
+
         //! Returns the Path object of the stack.
         Path* getPath() const {return d_path;}
 
@@ -104,6 +108,7 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 
         guint32 countAllies() const;
 
+
 	// Set Methods
 
         //! Change the loyalty of the stack.
@@ -149,6 +154,9 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
         //! Reduces movement points of the Army units in the Stack.
         void decrementMoves(guint32 moves);
 
+        //! Increases movement points of the Army units in the Stack.
+        void incrementMoves(guint32 moves);
+
         //! Sets the stack's position to the next point in it's Path.
         void moveOneStep(bool skipping = false);
 
@@ -213,6 +221,13 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 	//! Return a new stack that holds armies that have some mp.
 	Stack *splitArmiesWithMovement(guint32 mp = 1);
 
+        //! Drown the non-flying units over water.  sets hitpoints to zero.
+        bool killArmyUnitsInBoats();
+
+        //! Sets the hitpoints of all army units in the stack to zero.
+        void kill();
+
+
 	//! Add an army to this stack.
 	/**
 	 * This method should be used instead of push_back.
@@ -338,6 +353,10 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 	//! Calculate if the Stack is in a boat.
         bool hasShip () const;
 
+        //! Check to see if the stack has any items that can be used.
+        bool hasUsableItem() const;
+
+        void getUsableItems(std::list<Item*> &items) const;
 	/**
 	 * Calculate the number of movement points it costs for the Stack
 	 * to move to an adjacent tile.
@@ -370,6 +389,9 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 
 	//! Returns how many armies in the stack have visited the given temple.
 	guint32 countArmiesBlessedAtTemple(guint32 temple_id) const;
+        
+	//! Returns how many blessings have been bestowed on units in the stack.
+        guint32 countBlessings() const;
 
         //! Returns how many items this stack has.
         guint32 countItems() const;
@@ -411,6 +433,9 @@ class Stack : public ::UniquelyIdentified, public Movable, public Ownable, publi
 	//! Does this stack have 8 units in it?
 	bool isFull() const;
 
+        //! Return the hero that owns this given item.
+        Hero* getHeroWithItem(Item *item) const;
+
 	// Signals
 
 	//! Emitted when this stack dies.
