@@ -39,6 +39,7 @@
 #include "LocationList.h"
 #include "GameMap.h"
 #include "stacktile.h"
+#include "Item.h"
 
 std::string Stacklist::d_tag = "stacklist";
 using namespace std;
@@ -61,15 +62,6 @@ Vector<int> Stacklist::getPosition(guint32 id)
     return Vector<int>(-1,-1);
 }
 
-//We only expect one ambiguity at a time with stacks of the same player. This
-//never happens except when a stack comes to halt on another stack during
-//long movements
-//It also happens when two stacks fight.
-Stack* Stacklist::getAmbiguity(Stack* s)
-{
-  return GameMap::getInstance()->getTile(s->getPos())->getStacks()->getOtherStack(s);
-}
-
 //search all player's stacklists to find this stack
 bool Stacklist::deleteStack(Stack* s)
 {
@@ -465,7 +457,8 @@ bool Stacklist::canJumpOverTooLargeStack(Stack *s)
       if (moves > mp)
 	return false;
       mp -= moves;
-      if (GameMap::getEnemyCity(*it) != NULL)
+      City *enemy = GameMap::getEnemyCity(*it);
+      if (enemy != NULL && enemy->isBurnt() == false)
         return false;
       if (GameMap::getEnemyStack(*it) != NULL)
         return false;
@@ -588,14 +581,16 @@ void Stacklist::drainAllMovement()
       (*ait)->decrementMoves((*ait)->getMoves());
 }
 
-void Stacklist::changeOwnership(Stack *stack, Player *new_owner)
+Stack* Stacklist::changeOwnership(Stack *stack, Player *new_owner)
 {
   if (new_owner != stack->getOwner())
     {
       Stack *new_stack = new Stack(*stack);
       stack->getOwner()->getStacklist()->flRemove(stack);
       new_owner->addStack(new_stack);
+      return new_stack;
     }
+  return stack;
 }
 
 std::list<Vector<int> > Stacklist::getPositions() const
@@ -620,4 +615,73 @@ std::list<Stack*> Stacklist::getStacksWithItems() const
     }
   return stacks;
 }
+
+std::list<Stack*> Stacklist::kill()
+{
+  std::list<Stack*> stacks;
+  for (iterator it = begin(); it != end(); it++)
+    {
+      (*it)->kill();
+      stacks.push_back(*it);
+    }
+  return stacks;
+}
+
+std::list<Stack*> Stacklist::killArmyUnitsInBoats()
+{
+  std::list<Stack*> stacks;
+  for (iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->hasShip())
+        {
+          if ((*it)->killArmyUnitsInBoats())
+            stacks.push_back(*it);
+        }
+    }
+  return stacks;
+}
+
+std::list<Item*> Stacklist::getUsableItems() const
+{
+  std::list<Item*> items;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->hasUsableItem())
+        (*it)->getUsableItems(items);
+    }
+  return items;
+}
+
+bool Stacklist::hasUsableItem() const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->hasUsableItem())
+        return true;
+    }
+  return false;
+}
+        
+bool Stacklist::getItemHolder(Item *item, Stack **stack, Hero **hero) const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      *hero = (*it)->getHeroWithItem(item);
+      if (*hero != NULL)
+        {
+          *stack = *it;
+          return true;
+        }
+    }
+  return false;
+}
+
+guint32 Stacklist::countBlessingsOnArmyUnits() const
+{
+  guint32 count = 0;
+  for (const_iterator it = begin(); it != end(); it++)
+    count += (*it)->countBlessings();
+  return count;
+}
+
 // End of file
