@@ -1,6 +1,6 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 #include "gui/image-helpers.h"
 #include "GraphicsCache.h"
 #include "tilesetlist.h"
+#include "tarhelper.h"
+#include "Configuration.h"
 
 using namespace std;
 
@@ -40,11 +42,13 @@ using namespace std;
 
 std::string Tileset::d_tag = "tileset";
 std::string Tileset::d_road_smallmap_tag = "road_smallmap";
+std::string Tileset::d_ruin_smallmap_tag = "ruin_smallmap";
+std::string Tileset::d_temple_smallmap_tag = "temple_smallmap";
 std::string Tileset::file_extension = TILESET_EXT;
 
 Tileset::Tileset(guint32 id, std::string name)
 	: Set(), d_name(name), d_copyright(""), d_license(""), d_id(id), 
-	d_tileSize(DEFAULT_TILE_SIZE), d_subdir("")
+	d_tileSize(DEFAULT_TILE_SIZE), d_basename("")
 {
   d_info = "";
   d_large_selector = "";
@@ -54,6 +58,8 @@ Tileset::Tileset(guint32 id, std::string name)
   d_bridges = "";
   d_flags = "";
   d_road_color.set_rgb_p(0,0,0);
+  d_ruin_color.set_rgb_p(100,100,100);
+  d_temple_color.set_rgb_p(100,100,100);
   for (unsigned int i = 0; i < ROAD_TYPES; i++)
     roadpic[i] = NULL;
   for (unsigned int i = 0; i < BRIDGE_TYPES; i++)
@@ -92,6 +98,8 @@ Tileset::Tileset(XML_Helper *helper, std::string directory)
   helper->getData(d_flags, "flags");
   helper->registerTag(Tile::d_tag, sigc::mem_fun((*this), &Tileset::loadTile));
   helper->registerTag(Tileset::d_road_smallmap_tag, sigc::mem_fun((*this), &Tileset::loadTile));
+  helper->registerTag(Tileset::d_ruin_smallmap_tag, sigc::mem_fun((*this), &Tileset::loadTile));
+  helper->registerTag(Tileset::d_temple_smallmap_tag, sigc::mem_fun((*this), &Tileset::loadTile));
   helper->registerTag(SmallTile::d_tag, sigc::mem_fun((*this), &Tileset::loadTile));
   helper->registerTag(TileStyle::d_tag, sigc::mem_fun((*this), &Tileset::loadTile));
   helper->registerTag(TileStyleSet::d_tag, sigc::mem_fun((*this), &Tileset::loadTile));
@@ -119,6 +127,7 @@ Tileset::~Tileset()
   uninstantiateImages();
   for (unsigned int i=0; i < size(); i++)
     delete (*this)[i];
+  clean_tmp_dir();
 }
 
 int Tileset::getIndex(Tile::Type type) const
@@ -150,6 +159,18 @@ bool Tileset::loadTile(string tag, XML_Helper* helper)
       return true;
     }
 
+  if (tag == Tileset::d_ruin_smallmap_tag)
+    {
+      helper->getData(d_ruin_color, "color");
+      return true;
+    }
+
+  if (tag == Tileset::d_temple_smallmap_tag)
+    {
+      helper->getData(d_temple_color, "color");
+      return true;
+    }
+
   if (tag == SmallTile::d_tag)
     {
       Tile *tile = this->back();
@@ -177,7 +198,6 @@ bool Tileset::loadTile(string tag, XML_Helper* helper)
       // create a new tile style set with the information we got
       // put it on the latest tile
       TileStyleSet* tilestyleset = new TileStyleSet(helper);
-      tilestyleset->setSubDir(getSubDir());
       tile->push_back(tilestyleset);
       return true;
     }
@@ -215,6 +235,12 @@ bool Tileset::save(XML_Helper *helper) const
   retval &= helper->openTag(d_road_smallmap_tag);
   retval &= helper->saveData("color", d_road_color);
   retval &= helper->closeTag();
+  retval &= helper->openTag(d_ruin_smallmap_tag);
+  retval &= helper->saveData("color", d_ruin_color);
+  retval &= helper->closeTag();
+  retval &= helper->openTag(d_temple_smallmap_tag);
+  retval &= helper->saveData("color", d_temple_color);
+  retval &= helper->closeTag();
   for (Tileset::const_iterator i = begin(); i != end(); ++i)
     retval &= (*i)->save(helper);
   retval &= helper->closeTag();
@@ -222,6 +248,65 @@ bool Tileset::save(XML_Helper *helper) const
   return retval;
 }
 
+bool Tileset::save(std::string filename, std::string extension) const
+{
+  bool broken = false;
+  std::string goodfilename = File::add_ext_if_necessary(filename, extension);
+  std::string tmpfile = "lw.XXXX";
+  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
+  close (fd);
+  XML_Helper helper(tmpfile, std::ios::out, Configuration::s_zipfiles);
+  helper.begin(LORDSAWAR_TILESET_VERSION);
+  broken = !save(&helper);
+  helper.close();
+  if (broken == true)
+    return false;
+  std::string tmptar = tmpfile + ".tar";
+  Tar_Helper t(tmptar, std::ios::out, broken);
+  if (broken == true)
+    return false;
+  t.saveFile(tmpfile, File::get_basename(goodfilename, true));
+  //now the images, go get 'em from the tarball we were made from.
+  std::list<std::string> delfiles;
+  Tar_Helper orig(getConfigurationFile(), std::ios::in, broken);
+  if (broken == false)
+    {
+      std::list<std::string> files = orig.getFilenamesWithExtension(".png");
+      for (std::list<std::string>::iterator it = files.begin(); 
+           it != files.end(); it++)
+        {
+          std::string pngfile = orig.getFile(*it, broken);
+          if (broken == false)
+            {
+              t.saveFile(pngfile);
+              delfiles.push_back(pngfile);
+            }
+          else
+            break;
+        }
+      orig.Close();
+    }
+  else
+    {
+      FILE *fileptr = fopen (getConfigurationFile().c_str(), "r");
+      if (fileptr)
+        fclose (fileptr);
+      else
+        broken = false;
+    }
+  t.Close();
+  for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
+    File::erase(*it);
+  File::erase(tmpfile);
+  if (broken == false)
+    {
+      if (File::copy(tmptar, goodfilename) == 0)
+        File::erase(tmptar);
+    }
+
+  return !broken;
+}
+
 int Tileset::getFreeTileStyleId() const
 {
   int ids[65535];
@@ -232,7 +317,7 @@ int Tileset::getFreeTileStyleId() const
 	{
 	  for (std::vector<TileStyle*>::const_iterator k = (*j)->begin(); k != (*j)->end(); k++)
 	    {
-	      ids[(*k)->getId()] = 1;
+	      ids[(*k)->getId()]++;
 	    }
 	}
     }
@@ -262,12 +347,9 @@ int Tileset::getLargestTileStyleId() const
   return largest;
 }
 
-void Tileset::setSubDir(std::string dir)
+void Tileset::setBaseName(std::string bname)
 {
-  d_subdir = dir;
-  for (Tileset::iterator i = begin(); i != end(); ++i)
-    for (Tile::iterator j = (*i)->begin(); j != (*i)->end(); j++)
-      (*j)->setSubDir(dir);
+  d_basename = bname;
 }
 
 guint32 Tileset::getDefaultTileSize()
@@ -283,8 +365,8 @@ bool Tileset::validate() const
     {
       if ((*i)->validate() == false)
 	{
-	  fprintf (stderr, "`%s' fails validation\n", 
-		   (*i)->getName().c_str());
+	  fprintf (stderr, "`%s' tile fails validation in %s\n", 
+		   (*i)->getName().c_str(), getConfigurationFile().c_str());
 	  return false;
 	}
     }
@@ -294,40 +376,70 @@ bool Tileset::validate() const
 class TilesetLoader
 {
 public:
-    TilesetLoader(std::string filename) 
+    TilesetLoader(std::string filename, bool &broken, bool &unsupported)
       {
+        unsupported_version = false;
 	tileset = NULL;
 	dir = File::get_dirname(filename);
+        file = File::get_basename(filename);
 	if (File::nameEndsWith(filename, Tileset::file_extension) == false)
 	  filename += Tileset::file_extension;
-	XML_Helper helper(filename, ios::in, false);
+        Tar_Helper t(filename, std::ios::in, broken);
+        if (broken)
+          return;
+        std::string lwtfilename = 
+          t.getFirstFile(Tileset::file_extension, broken);
+        if (broken)
+          return;
+	XML_Helper helper(lwtfilename, ios::in, false);
 	helper.registerTag(Tileset::d_tag, sigc::mem_fun((*this), &TilesetLoader::load));
 	if (!helper.parse())
 	  {
-	    std::cerr << "Error, while loading an tileset. Tileset Name: ";
-	    std::cerr <<dir<<std::endl <<std::flush;
+            unsupported = unsupported_version;
+	    std::cerr << "Error, while loading a tileset. Tileset File: ";
+	    std::cerr << filename << std::endl <<std::flush;
 	    if (tileset != NULL)
 	      delete tileset;
 	    tileset = NULL;
 	  }
+        File::erase(lwtfilename);
+        helper.close();
+        t.Close();
       };
     bool load(std::string tag, XML_Helper* helper)
       {
 	if (tag == Tileset::d_tag)
 	  {
-	    tileset = new Tileset(helper, dir);
-	    return true;
+            if (helper->getVersion() == LORDSAWAR_TILESET_VERSION)
+              {
+                tileset = new Tileset(helper, dir);
+                tileset->setBaseName(file);
+                return true;
+              }
+            else
+              {
+                unsupported_version = true;
+                return false;
+              }
 	  }
 	return false;
       };
     std::string dir;
+    std::string file;
     Tileset *tileset;
+    bool unsupported_version;
 };
-Tileset *Tileset::create(std::string file)
+
+
+Tileset *Tileset::create(std::string file, bool &unsupported_version)
 {
-  TilesetLoader d(file);
+  bool broken = false;
+  TilesetLoader d(file, broken, unsupported_version);
+  if (broken)
+    return NULL;
   return d.tileset;
 }
+
 void Tileset::getFilenames(std::list<std::string> &files)
 {
   for (iterator it = begin(); it != end(); it++)
@@ -519,8 +631,12 @@ void Tileset::instantiateImages()
   int size = getTileSize();
   debug("Loading images for tileset " << getName());
   uninstantiateImages();
+  bool broken = false;
+  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
+  if (broken)
+    return;
   for (iterator it = begin(); it != end(); it++)
-    (*it)->instantiateImages(size, this);
+    (*it)->instantiateImages(size, &t);
   std::string explosion_filename = "";
   std::string roads_filename = "";
   std::string bridges_filename = "";
@@ -529,44 +645,61 @@ void Tileset::instantiateImages()
   std::string selector_filename = "";
   std::string small_selector_filename = "";
 
-  if (getExplosionFilename().empty() == false)
-    explosion_filename = getFile(getExplosionFilename());
-  if (getRoadsFilename().empty() == false)
-    roads_filename = getFile(getRoadsFilename());
-  if (getBridgesFilename().empty() == false)
-    bridges_filename = getFile(getBridgesFilename());
-  if (getFogFilename().empty() == false)
-    fog_filename = getFile(getFogFilename());
-  if (getFlagsFilename().empty() == false)
-    flags_filename = getFile(getFlagsFilename());
-  if (getLargeSelectorFilename().empty() == false)
-    selector_filename = getFile(getLargeSelectorFilename());
-  if (getSmallSelectorFilename().empty() == false)
-    small_selector_filename = getFile(getSmallSelectorFilename());
-  instantiateImages(explosion_filename, roads_filename, bridges_filename, 
-		    fog_filename, flags_filename, selector_filename, 
-		    small_selector_filename);
+  if (getExplosionFilename().empty() == false && !broken)
+    explosion_filename = t.getFile(getExplosionFilename() + ".png", broken);
+  if (getRoadsFilename().empty() == false && !broken)
+    roads_filename = t.getFile(getRoadsFilename() + ".png", broken);
+  if (getBridgesFilename().empty() == false && !broken)
+    bridges_filename = t.getFile(getBridgesFilename() + ".png", broken);
+  if (getFogFilename().empty() == false && !broken)
+    fog_filename = t.getFile(getFogFilename() + ".png", broken);
+  if (getFlagsFilename().empty() == false && !broken)
+    flags_filename = t.getFile(getFlagsFilename() + ".png", broken);
+  if (getLargeSelectorFilename().empty() == false && !broken)
+    selector_filename = t.getFile(getLargeSelectorFilename() + ".png", broken);
+  if (getSmallSelectorFilename().empty() == false && !broken)
+    small_selector_filename = 
+      t.getFile(getSmallSelectorFilename() + ".png", broken);
+  if (!broken)
+    instantiateImages(explosion_filename, roads_filename, bridges_filename, 
+                      fog_filename, flags_filename, selector_filename, 
+                      small_selector_filename);
+  if (explosion_filename.empty() == false)
+    File::erase(explosion_filename);
+  if (roads_filename.empty() == false)
+    File::erase(roads_filename);
+  if (bridges_filename.empty() == false)
+    File::erase(bridges_filename);
+  if (fog_filename.empty() == false)
+    File::erase(fog_filename);
+  if (flags_filename.empty() == false)
+    File::erase(flags_filename);
+  if (selector_filename.empty() == false)
+    File::erase(selector_filename);
+  if (small_selector_filename.empty() == false)
+    File::erase(small_selector_filename);
+  t.Close();
 }
 
 std::string Tileset::getConfigurationFile() const
 {
-  return getDirectory() + d_subdir + file_extension;
+  return getDirectory() + d_basename + file_extension;
 }
 
 std::list<std::string> Tileset::scanUserCollection()
 {
-  return File::scanFiles(File::getUserTilesetDir(), file_extension);
+  return File::scanForFiles(File::getUserTilesetDir(), file_extension);
 }
 
 std::list<std::string> Tileset::scanSystemCollection()
 {
-  std::list<std::string> retlist = File::scanFiles(File::getTilesetDir(), 
-						   file_extension);
+  std::list<std::string> retlist = File::scanForFiles(File::getTilesetDir(), 
+                                                      file_extension);
   if (retlist.empty())
     {
       std::cerr << "Couldn't find any tilesets (*" << file_extension << 
-        ") in directories below: " << File::getTilesetDir() << std::endl;
-      std::cerr << "Please check the path settings in /etc/lordsawarrc or ~/.lordsawarrc" << std::endl;
+        ") in : " << File::getTilesetDir() << std::endl;
+      std::cerr << "Please check the path settings in ~/.lordsawarrc" << std::endl;
       std::cerr << "Exiting!" << std::endl;
       exit(-1);
     }
@@ -582,4 +715,153 @@ TileStyle *Tileset::getTileStyle(guint32 id) const
   else
     return (*it).second;
 }
+
+void Tileset::reload()
+{
+  bool broken = false;
+  bool unsupported_version = false;
+  TilesetLoader d(getConfigurationFile(), broken, unsupported_version);
+  if (!broken && d.tileset && d.tileset->validate())
+    {
+      //steal the values from d.tileset and then don't delete it.
+      uninstantiateImages();
+      for (iterator it = begin(); it != end(); it++)
+        delete *it;
+      std::string basename = d_basename;
+      *this = *d.tileset;
+      instantiateImages();
+      d_basename = basename;
+    }
+}
+
+std::string Tileset::getFileFromConfigurationFile(std::string file)
+{
+  bool broken = false;
+  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
+  if (broken == false)
+    {
+      std::string filename = t.getFile(file, broken);
+      t.Close();
+  
+      if (broken == false)
+        return filename;
+    }
+  return "";
+}
+
+bool Tileset::addFileInConfigurationFile(std::string new_file)
+{
+  return replaceFileInConfigurationFile("", new_file);
+}
+bool Tileset::replaceFileInConfigurationFile(std::string file, std::string new_file)
+{
+  bool broken = false;
+  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
+  if (broken == false)
+    {
+      broken = t.replaceFile(file, new_file);
+      t.Close();
+    }
+  return broken;
+}
+
+guint32 Tileset::calculate_preferred_tile_size() const
+{
+  guint32 tilesize = 0;
+  std::map<guint32, guint32> sizecounts;
+
+  if (roadpic[0])
+    sizecounts[roadpic[0]->get_unscaled_width()]++;
+  if (bridgepic[0])
+    sizecounts[bridgepic[0]->get_unscaled_width()]++;
+  if (flagpic[0])
+    sizecounts[flagpic[0]->get_unscaled_width()]++;
+  if (selector[0])
+    sizecounts[selector[0]->get_unscaled_width()]++;
+  if (smallselector[0])
+    sizecounts[smallselector[0]->get_unscaled_width()]++;
+  if (fogpic[0])
+    sizecounts[fogpic[0]->get_unscaled_width()]++;
+  if (explosion)
+    sizecounts[explosion->get_unscaled_width()]++;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      Tile *tile = *it;
+      for (Tile::const_iterator i = tile->begin(); i != tile->end(); i++)
+        {
+          TileStyle *tilestyle = (*i)->front();
+          if (tilestyle && tilestyle->getImage())
+            sizecounts[tilestyle->getImage()->get_unscaled_width()]++;
+        }
+    }
+
+  guint32 maxcount = 0;
+  for (std::map<guint32, guint32>::iterator it = sizecounts.begin(); 
+       it != sizecounts.end(); it++)
+    {
+      if ((*it).second > maxcount)
+        {
+          maxcount = (*it).second;
+          tilesize = (*it).first;
+        }
+    }
+  if (tilesize == 0)
+    tilesize = DEFAULT_TILE_SIZE;
+  return tilesize;
+}
+
+bool Tileset::copy(std::string src, std::string dest)
+{
+  return Tar_Helper::copy(src, dest);
+}
+        
+bool Tileset::addTileStyleSet(Tile *tile, std::string filename)
+{
+  bool success = true;
+  TileStyle::Type tilestyle_type;
+  if (tile->getType() == Tile::GRASS)
+    tilestyle_type = TileStyle::LONE;
+  else
+    tilestyle_type = TileStyle::UNKNOWN;
+  TileStyleSet *set = 
+    new TileStyleSet(filename, d_tileSize, success, tilestyle_type);
+  if (!success)
+    {
+      delete set;
+      return success;
+    }
+  guint32 first_free_id = getFreeTileStyleId();
+  for (TileStyleSet::iterator it = set->begin(); it != set->end(); it++)
+    (*it)->setId(first_free_id);
+
+  tile->push_back(set);
+  for (TileStyleSet::iterator it = ++set->begin(); it != set->end(); it++)
+    (*it)->setId(getFreeTileStyleId());
+  return success;
+}
+
+bool Tileset::getTileStyle(guint32 id, Tile **tile, TileStyleSet **set, TileStyle ** style) const
+{
+  for (const_iterator t = begin(); t != end(); t++)
+    for (std::list<TileStyleSet*>::const_iterator i = (*t)->begin(); 
+         i != (*t)->end(); i++)
+      for (std::vector<TileStyle*>::const_iterator j = (*i)->begin(); 
+           j != (*i)->end(); j++)
+        if ((*j)->getId() == id)
+          {
+            if (tile)
+              *tile = *t;
+            if (set)
+              *set = *i;
+            if (style)
+              *style = *j;
+            return true;
+          }
+  return false;
+}
+
+void Tileset::clean_tmp_dir() const
+{
+  return Tar_Helper::clean_tmp_dir(getConfigurationFile());
+}
 //End of file
