@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <iostream>
 #include <algorithm>
 #include <expat.h>
+#include <assert.h>
 #include "rectangle.h"
 #include "ucompose.hpp"
 #include <sigc++/functors/mem_fun.h>
@@ -26,11 +27,13 @@
 #include "File.h"
 #include "defs.h"
 #include "tileset.h"
+#include "tarhelper.h"
+#include "SmallTile.h"
 
 using namespace std;
 
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
-//#define debug(x)
+//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x)
 
 Tilesetlist* Tilesetlist::s_instance = 0;
 
@@ -73,6 +76,24 @@ void Tilesetlist::getSizes(std::list<guint32> &sizes) const
     }
 }
 
+std::list<std::string> Tilesetlist::getValidNames() const
+{
+  std::list<std::string> names;
+  for (const_iterator it = begin(); it != end(); it++)
+    if ((*it)->validate() == true)
+      names.push_back((*it)->getName());
+  return names;
+}
+
+std::list<std::string> Tilesetlist::getValidNames(guint32 tilesize) const
+{
+  std::list<std::string> names;
+  for (const_iterator it = begin(); it != end(); it++)
+    if ((*it)->getTileSize() == tilesize && (*it)->validate() == true)
+      names.push_back((*it)->getName());
+  return names;
+}
+
 std::list<std::string> Tilesetlist::getNames() const
 {
   std::list<std::string> names;
@@ -90,23 +111,31 @@ std::list<std::string> Tilesetlist::getNames(guint32 tilesize) const
   return names;
 }
 
-Tileset *Tilesetlist::loadTileset(std::string filename)
+Tileset *Tilesetlist::loadTileset(std::string name)
 {
-  debug("Loading tileset " <<File::get_basename(File::get_dirname(filename)));
-  Tileset *tileset = Tileset::create(filename);
+  debug("Loading tileset " <<File::get_basename(name));
+  bool unsupported_version = false;
+  Tileset *tileset = Tileset::create(name, unsupported_version);
   if (tileset == NULL)
-    return NULL;
-  if (tileset->validate() == false)
     {
-      cerr<< "Error!  Tileset: `" << tileset->getConfigurationFile() <<
-	"' fails validation.  Skipping.\n",
+      cerr<< "Error!  Tileset: `" << File::get_basename(name, true) <<
+	"' is malformed.  Skipping.\n";
+        return NULL;
+    }
+  if (d_tilesets.find(tileset->getBaseName()) != d_tilesets.end())
+    {
+      Tileset *t = (*d_tilesets.find(tileset->getBaseName())).second;
+      cerr << "Error!  tileset: `" << tileset->getConfigurationFile() << 
+	"' shares a duplicate tileset subdir `" << t->getBaseName() << "' with `" << t->getConfigurationFile() 
+	<< "'.  Skipping." << endl;
       delete tileset;
       return NULL;
     }
+    
   if (d_tilesetids.find(tileset->getId()) != d_tilesetids.end())
     {
       Tileset *t = (*d_tilesetids.find(tileset->getId())).second;
-      cerr << "Error!  tileset: `" << tileset->getName() << 
+      cerr << "Error!  tileset: `" << tileset->getConfigurationFile() << 
 	"' shares a duplicate tileset id with `" << 
 	t->getConfigurationFile() << "'.  Skipping." << endl;
       delete tileset;
@@ -115,13 +144,13 @@ Tileset *Tilesetlist::loadTileset(std::string filename)
   return tileset;
 }
 
-void Tilesetlist::add(Tileset *tileset)
+void Tilesetlist::add(Tileset *tileset, std::string file)
 {
-  std::string subdir = File::get_basename(tileset->getDirectory());
+  std::string basename = File::get_basename(file);
   push_back(tileset); 
-  tileset->setSubDir(subdir);
-  d_dirs[String::ucompose("%1 %2", tileset->getName(), tileset->getTileSize())] = subdir;
-  d_tilesets[subdir] = tileset;
+  tileset->setBaseName(basename);
+  d_dirs[String::ucompose("%1 %2", tileset->getName(), tileset->getTileSize())] = basename;
+  d_tilesets[basename] = tileset;
   d_tilesetids[tileset->getId()] = tileset;
 }
 
@@ -142,19 +171,20 @@ void Tilesetlist::loadTilesets(std::list<std::string> tilesets)
       Tileset *tileset = loadTileset(*i);
       if (!tileset)
 	continue;
-      add(tileset);
+      add(tileset, *i);
     }
 }
 
 int Tilesetlist::getNextAvailableId(int after)
 {
+  bool unsupported_version = false;
   std::list<guint32> ids;
   std::list<std::string> tilesets = Tileset::scanSystemCollection();
   //there might be IDs in invalid tilesets.
   for (std::list<std::string>::const_iterator i = tilesets.begin(); 
        i != tilesets.end(); i++)
     {
-      Tileset *tileset = Tileset::create(*i);
+      Tileset *tileset = Tileset::create(*i, unsupported_version);
       if (tileset != NULL)
 	{
 	  ids.push_back(tileset->getId());
@@ -165,7 +195,7 @@ int Tilesetlist::getNextAvailableId(int after)
   for (std::list<std::string>::const_iterator i = tilesets.begin(); 
        i != tilesets.end(); i++)
     {
-      Tileset *tileset = Tileset::create(*i);
+      Tileset *tileset = Tileset::create(*i, unsupported_version);
       if (tileset != NULL)
 	{
 	  ids.push_back(tileset->getId());
@@ -207,64 +237,127 @@ Tileset *Tilesetlist::getTileset(guint32 id) const
   return (*it).second;
 }
 
-bool Tilesetlist::addToPersonalCollection(Tileset *tileset, std::string &new_subdir, guint32 &new_id)
+Tileset *Tilesetlist::import(Tar_Helper *t, std::string f, bool &broken)
+{
+  bool unsupported_version = false;
+  std::string filename = t->getFile(f, broken);
+  if (broken)
+    return NULL;
+  Tileset *tileset = Tileset::create(filename, unsupported_version);
+  assert (tileset != NULL);
+  tileset->setBaseName(File::get_basename(f));
+
+  std::string basename = "";
+  guint32 id = 0;
+  addToPersonalCollection(tileset, basename, id);
+
+  return tileset;
+}
+
+std::string Tilesetlist::findFreeBaseName(std::string basename, guint32 max, guint32 &num) const
+{
+  std::string new_basename;
+  for (unsigned int count = 1; count < max; count++)
+    {
+      new_basename = String::ucompose("%1%2", basename, count);
+      if (getTileset(new_basename) == NULL)
+        {
+          num = count;
+          break;
+        }
+      else
+        new_basename = "";
+    }
+  return new_basename;
+}
+
+bool Tilesetlist::addToPersonalCollection(Tileset *tileset, std::string &new_basename, guint32 &new_id)
 {
   //do we already have this one?
       
-  if (getTileset(tileset->getSubDir()) == getTileset(tileset->getId()) &&
-      getTileset(tileset->getSubDir()) != NULL)
+  if (getTileset(tileset->getBaseName()) == getTileset(tileset->getId()) 
+      && getTileset(tileset->getBaseName()) != NULL)
     {
       tileset->setDirectory(getTileset(tileset->getId())->getDirectory());
       return true;
     }
 
-  //if the subdir conflicts with any other subdir, then change it.
-  if (getTileset(tileset->getSubDir()) != NULL)
+  //if the basename conflicts with any other basename, then change it.
+  if (getTileset(tileset->getBaseName()) != NULL)
     {
-      bool found = false;
-      for (int count = 0; count < 100; count++)
-	{
-	  new_subdir = String::ucompose("%1%2", tileset->getSubDir(), count);
-	  if (getTileset(new_subdir) == NULL)
-	    {
-	      found = true;
-	      break;
-	    }
-	}
-      if (found == false)
-	return false;
-      tileset->setSubDir(new_subdir);
+      if (new_basename != "" && getTileset(new_basename) == NULL)
+        ;
+      else
+        {
+          guint32 num = 0;
+          std::string new_basename = findFreeBaseName(tileset->getBaseName(), 100, num);
+          if (new_basename == "")
+            return false;
+        }
     }
-  else
-    new_subdir = tileset->getSubDir();
+  else if (new_basename == "")
+    new_basename = tileset->getBaseName();
 
   //if the id conflicts with any other id, then change it
-  if (getTileset(tileset->getId()))
+  if (getTileset(tileset->getId()) != NULL)
     {
-      new_id = Tilesetlist::getNextAvailableId(tileset->getId());
-      tileset->setId(new_id);
+      if (new_id != 0 && getTileset(new_id) == NULL)
+        tileset->setId(new_id);
+      else
+        {
+          new_id = Tilesetlist::getNextAvailableId(tileset->getId());
+          tileset->setId(new_id);
+        }
     }
   else
     new_id = tileset->getId();
 
   //make the directory where the tileset is going to live.
-  std::string directory = 
-    File::getUserTilesetDir() + tileset->getSubDir() + "/";
+  std::string file = File::getUserTilesetDir() + new_basename + Tileset::file_extension;
 
-  if (File::create_dir(directory) == false)
-    return false;
+  tileset->save(file, Tileset::file_extension);
 
-  //okay now we copy the image files into the new directory 
-  std::list<std::string> files;
-  tileset->getFilenames(files);
-  for (std::list<std::string>::iterator it = files.begin(); it != files.end();
-       it++)
-    File::copy(tileset->getFile(*it), directory + *it);
-
-  //save out the tileset file
-  tileset->setDirectory(directory);
-  XML_Helper helper(tileset->getConfigurationFile(), std::ios::out, false);
-  tileset->save(&helper);
-  helper.close();
+  if (new_basename != tileset->getBaseName())
+    tileset->setBaseName(new_basename);
+  tileset->setDirectory(File::get_dirname(file));
+  add (tileset, file);
   return true;
 }
+
+bool Tilesetlist::contains(std::string name) const
+{
+  std::list<std::string> n = getNames();
+  for (std::list<std::string>::iterator it = n.begin(); it != n.end(); it++)
+    {
+      if (*it == name)
+        return true;
+    }
+  return false;
+}
+
+guint32 Tilesetlist::getTilesetId(std::string basename) const
+{
+  Tileset *ts = getTileset(basename);
+  if (ts)
+    return ts->getId();
+  else
+    return 0;
+}
+
+SmallTile *Tilesetlist::getSmallTile(std::string basename, Tile::Type type) const
+{
+  Tileset *ts = getTileset(basename);
+  if (!ts)
+    return NULL;
+  int idx = ts->getIndex(type);
+  return (*ts)[idx]->getSmallTile();
+}
+
+Gdk::Color Tilesetlist::getColor(std::string basename, Tile::Type type) const
+{
+  SmallTile *smalltile = getSmallTile(basename, type);
+  if (!smalltile)
+    return Gdk::Color("black");
+  else
+    return smalltile->getColor();
+}
